/**@name LIBRARY: gcsAnalogComputed.ctl

@author: Artem Burmyakov (IT-CO-FE)
@author: Nikolay Kulman (IT-CO-FE)
@author: Teodora Miteva (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 17.06.2003
@brief This library contains AnalogComputed object functions.

Modifications History:
$Log: gcsAnalogComputed.ctl,v $
Revision 1.24  2009/04/02 12:11:58  gthomas
in _ProvideAccessToButtons(): removed exceptionInfo as returns errors in log when monitor or no user are logged in

Revision 1.23  2009/03/25 09:44:55  gamx
Bug in animation fixed (test cases 2-5 of widget animation):
Standard Unicos function to animate widget disconnection cannot be used as AnalogComputed does not have "AlarmText" graphical object
on the widget panel.

Revision 1.22  2009/03/19 09:09:49  gthomas
-Fixed typo in function call name <_getPlcDpName> in _FaceplateStatusRegisterCB
-Added missing variable <dsUserAccess> in _ProvideAccessToButtons()

Revision 1.21  2008/11/12 08:19:05  gthomas
-removed useless functions:
 _ButtonUserAccessGcs and _ButtonUserAccessUn

Revision 1.20  2008/10/15 14:54:15  gthomas
-Removed the <if statement> of the dpConnect in _ButtonRegisterCB function
to fix problem with select/deselect button animation.

Revision 1.19  2008/07/23 10:02:19  gthomas
-removed commented code (for error handling)

Revision 1.18  2008/07/22 14:49:49  gthomas
added missing ExceptionInfo missing in _ProvideAccessToButtons()

Revision 1.17  2008/07/18 07:16:55  gthomas
added new Access control functionality for unicos-4-1
modified  xx_ButtonUserAccess function.

Revision 1.16  2008/06/18 14:54:51  aabadz
Changes done following the recomendations from AB-CO (from Herve)

Revision 1.15  2008/03/17 10:52:36  aabadz
Fixed a problem found in animation when we have not database.

Revision 1.14  2008/03/17 10:15:44  aabadz
Fixed a problem found in animation when we have not database. Fixed now in all cases

Revision 1.13  2008/01/21 16:21:21  gamx
version after the final test:
bugs related to plc name definition are fixed

Revision 1.12  2008/01/16 15:57:13  aabadz
Some sintactic errors fixed

Revision 1.11  2008/01/15 11:00:33  aabadz
Modifed ObjectListGetValueTime because the code that we had before return nothing instead of  the time and the value

Revision 1.10  2008/01/11 13:03:30  gamx
messages of errors have been improved

Revision 1.9  2008/01/10 15:44:06  gamx
reviewed:
1) exception handling;
2) naming;
3) documenting;
4) functions organisation (complex ones are splitted)

Revision 1.8  2007/10/30 16:09:45  gthomas
Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
	gcsAnalogComputed_ObjectListGetValueTime (string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
	gcsAnalogComputed_MenuConfiguration (string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
	gcsAnalogComputed_HandleMenu (string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
	gcsAnalogComputed_FaceplateStatusRegisterCB (string sDp, bool bSystemConnected)
	gcsAnalogComputed_FaceplateConnect (string deviceName)
	gcsAnalogComputed_FaceplateDisconnect (string deviceName)
	gcsAnalogComputed_FaceplateStatusAnimationCB (string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid, string sDpPosOV, float fPosOV, ...)
	gcsAnalogComputed_AnimatePlcDisconnection ()
	gcsAnalogComputed_ButtonRegisterCB (string sDp, bool bSystemConnected)
	gcsAnalogComputed_ButtonAnimationCB (string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
	gcsAnalogComputed_ButtonUserAccess (string sDpName, string sDpType, dyn_string &dsAccess)
	gcsAnalogComputed_UserLoginGetButtonState (string deviceName, string sType, dyn_string &dsReturnData)
	gcsAnalogComputed_ButtonSetState (string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
	gcsAnalogComputed_ButtonDisconnection ()
	gcsAnalogComputed_WidgetRegisterCB (string sDp, bool bSystemConnected)
	gcsAnalogComputed_WidgetConnect (string deviceName, string sPlcName, string sSystemName)
	gcsAnalogComputed_WidgetDisconnect (string deviceName, string sPlcName, string sSystemName)
	gcsAnalogComputed_WidgetAnimationCB (string sDpLock, bool bLocked, string sDpManager, string sSelectedManager, string sDpStsReg01, bit32 bit32StsReg01, ...)
	gcsAnalogComputed_WidgetDisconnection (string sWidgetType)
	AnalogComputed_EventList_Configure_Init (dyn_string &exceptionInfo, const bool bUpdate=false)
	AnalogComputed_EventList_Configure_Clic (int iRow, string sColumn, string sValue, dyn_string &exceptionInfo)
	AnalogComputed_EventList_Configure_Save (dyn_string &exceptionInfo)
*/


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_ObjectListGetValueTime
/**
Function called from the objectList to return the time and value

	@param[in]	sDeviceName		Device name
	@param[in]	sDeviceType		Unicos object
	@param[out]	dsReturnData	Return value
*/
gcsAnalogComputed_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	//as object list is going to be removed, the function will not be used
	dsReturnData = makeDynString("", "");
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_MenuConfiguration
/**
Configuration of popup menu

	@param[in]  sDpName	Device name
	@param[in]  sDpType 	Unicos object
	@param[in]  dsAccessOk	List of authorized action
	@param[out] menuList 	Configuration of menu. Use defined constants UN_POPUPMENU_***
*/
gcsAnalogComputed_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT,
			 UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_HandleMenu
/**
Handle the answer of the popup menu

	@param[in]	sDeviceName 	The device dp name
	@param[in]	sDpType 	The device dp type
	@param[in]	menuList 	List of requested action
	@param[in]	menuAnswer 	Selected action from the menuList
*/
gcsAnalogComputed_HandleMenu(string sDeviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	gcsGenericObject_HandleEventMaskMenu(sDeviceName, menuAnswer, makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) unGenericObject_HandleUnicosMenu(sDeviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	//in case of errors in _HandleEventMaskMenu or _HandleUnicosMenu
	if (dynlen(exceptionInfo) > 0) unSendMessage_toExpertException("Right Click", exceptionInfo);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_FaceplateStatusRegisterCB
/**
Register faceplate callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsAnalogComputed_FaceplateStatusRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName;
	string sSystemName;
	string sPlcName;
	string sPlcDpName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		unDistributedControl_isRemote(bRemote, sSystemName);

		if(bRemote) g_bSystemConnected = bSystemConnected;
		else g_bSystemConnected = true;

		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
		gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
		else g_bUnSystemAlarmPlc = false;

		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
				gcsAnalogComputed_AnimatePlcDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				gcsAnalogComputed_FaceplateConnect(sDeviceName);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				gcsAnalogComputed_FaceplateDisconnect(sDeviceName);
				gcsAnalogComputed_AnimatePlcDisconnection();
				break;
			case UN_ACTION_NOTHING:
			default:
				break;
		}

	}

}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogParameter_FaceplateConnect
/**
	Connects callback functions to animate a faceplate.
	As it is not possible to pass constant values in callback functions and to get them inside callback function,
	global constants are used.
		@param[in]	sDeviceName	Device DP name
*/
gcsAnalogComputed_FaceplateConnect(string sDeviceName)
{
	int iRes = -1;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcName;
	string sSystemName;
	string sPlcDpName;

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	g_bCallbackConnected = false;

	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;


	if (g_bUnSystemAlarmPlc)
	{
		g_sPosOvFormat = dpGetFormat(sDeviceName + ".ProcessInput.PosOV");
		g_sPosOvUnit = dpGetUnit(sDeviceName + ".ProcessInput.PosOV");
		g_sAuPosRStFormat = dpGetFormat(sDeviceName + ".ProcessInput.AuPosRSt");
		g_sAuPosRStUnit = dpGetUnit(sDeviceName + ".ProcessInput.AuPosRSt");

		iRes = dpConnect("gcsAnalogComputed_FaceplateStatusAnimationCB",
			 sDeviceName + ".ProcessInput.StsReg01",
			 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			 sDeviceName + ".ProcessInput.PosOV",
			 sDeviceName + ".ProcessInput.PosOV:_online.._invalid",
			 sDeviceName + ".ProcessInput.AuPosRSt",
			 sDeviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
			 sPlcDpName + ".alarm",
			 sPlcDpName + ".enabled",
			 sDeviceName + ".eventMask");

		g_bCallbackConnected = (iRes >= 0);
	}
	else unAnalogComputed_AnimatePlcDisconnection();
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogParameter_FaceplateDisconnect
/**
  Disconnects faceplate from plc (stop any animation).

  	@param[in] sDeviceName	Device dp name
*/
gcsAnalogComputed_FaceplateDisconnect(string sDeviceName)
{
	int iRes = -1;
	string sPlcName, sSystemName;
	string sPlcDpName;
	dyn_string exceptionInfo;


	g_bCallbackConnected = false;

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;


	if (g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("gcsAnalogComputed_FaceplateStatusAnimationCB",
			 sDeviceName + ".ProcessInput.StsReg01",
			 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			 sDeviceName + ".ProcessInput.PosOV",
			 sDeviceName + ".ProcessInput.PosOV:_online.._invalid",
			 sDeviceName + ".ProcessInput.AuPosRSt",
			 sDeviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
			 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			 sDeviceName + ".eventMask");
	}

	g_bCallbackConnected = !(iRes >= 0);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_FaceplateStatusAnimationCB
/**
Animate faceplate
*/
gcsAnalogComputed_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
					     string sDpPosOV, float fPosOV, string sDp4, bool bPosOVInvalid,
					     string sDpAuto, float fAuPosRSt, string sDp6, bool bAuPosRStInvalid,
					     string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
					     string sDpEvent, int iMask)
{
	if(g_bSystemConnected)
		gcsGenericObject_DisplayValue(g_sPosOvFormat, g_sPosOvUnit, fPosOV, "StatusPosition", "unDisplayValue_Status",
			bPosOVInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		gcsGenericObject_DisplayValue(g_sAuPosRStFormat, g_sAuPosRStUnit, fAuPosRSt, "RequestAuto", "unDisplayValue_Request",
			bAuPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_ManualAuto", getBit(bit32StsReg01, UN_STSREG01_AUMRW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("eventState", iMask, 1, "",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value), "unFaceplate_OperationModeActive");
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_AnimatePlcDisconnection
/**
Animate faceplate disconnection
*/
gcsAnalogComputed_AnimatePlcDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("OperationMode_Auto", "OperationMode_Forced", "OperationMode_Manual",
		 "Warning_IOError", "Warning_IOSimulated", "Warning_ManualAuto", "eventState"));
	unGenericObject_DisplayValueDisconnect(makeDynString("RequestAuto", "StatusPosition"));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_ButtonRegisterCB
/**
Register callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsAnalogComputed_ButtonRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01, iManRegSet;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);

	if (sDeviceName)
	{
		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction)
		{
				case UN_ACTION_DISCONNECT:
					gcsAnalogComputed_ButtonDisconnection();
					break;
				case UN_ACTION_DPCONNECT:
					// Send 0 to ManReg01 if necessary
					iRes = dpGet(sDeviceName + ".ProcessOutput.ManReg01", iManReg01);
					if ((iRes < 0) || (iManReg01 != 0))
					{
						iManRegSet = dpSet(sDeviceName + ".ProcessOutput.ManReg01", 0);
						if (iManRegSet < 0)
						{
							dynAppend(exceptionInfo, "gcsAnalogComputed_ButtonRegisterCB: problems to set value for: " + sDeviceName + ".ProcessOutput.ManReg01");
							break;
						}
					}

					// call the gcsAnalogComputed_ButtonUserAccess to get the list of allowed buttons.
					sDpType=GCS_CONFIG_ANALOGCOMPUTED_DPT_NAME;
					unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);

					if (dynlen(exceptionInfo) <= 0) gcsAnalogComputed_ProvideAccessToButtons(sDeviceName, sDpType, dsFunctions, exceptionInfo);
					gcsAnalogComputed_ButtonsConnect(sDeviceName);
					break;
				case UN_ACTION_DPDISCONNECT_DISCONNECT:
					gcsAnalogComputed_ButtonsDisconnect(sDeviceName);
					gcsAnalogComputed_ButtonDisconnection();
					break;
				case UN_ACTION_NOTHING:
				default:
					break;
		}
	}
	else dynAppend(exceptionInfo, "gcsAnalogComputed_ButtonRegisterCB: device name is empty");

}





//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_ProvideAccessToButtons
/**
Defines the list of buttons available for a user. The result is passed in global variable.
	@param[in]	sDeviceName	Name of the device
	@param[in]	sDpType		Device Type
	@param[in] 	dsFunctions	A list of functions for a device, specified in device internal configuration.
	@param[out]	exceptionInfo	A list of exceptions.
*/
gcsAnalogComputed_ProvideAccessToButtons(string sDeviceName, string sDpType, dyn_string dsFunctions, dyn_string &exceptionInfo)
{
	string sFunction;
	dyn_string dsUserAccess;

	if (dynlen(dsFunctions) >= UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS)
		sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
	else
	{
		dynAppend(exceptionInfo, "gcsAnalogComputed_ButtonRegisterCB: the list of functions is not correct - number of functions is less than expected");
		return;
	}


	if (sFunction != "")
	{
		evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" +
					 "dyn_string dsUserAccessReturn;" +
					 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
					 "    {" +
						sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
					 "    }" +
					 "return dsUserAccessReturn; }", exceptionInfo, sDeviceName, sDpType);

		g_dsUserAccess=dsUserAccess;
	}
	else
		fwException_raise(exceptionInfo,"ERROR", "AnalogComputed_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
}









//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_ButtonsConnect
/**
Connects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
gcsAnalogComputed_ButtonsConnect(string sDeviceName)
{
	int iRes = -1;

	iRes = dpConnect("gcsAnalogComputed_ButtonAnimationCB",
		 sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
		 sDeviceName + ".statusInformation.selectedManager",
		 sDeviceName + ".ProcessInput.StsReg01",
		 sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv");

	g_bCallbackConnected = (iRes >= 0);
}






//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_ButtonsDisconnect
/**
Disconnects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
gcsAnalogComputed_ButtonsDisconnect(string sDeviceName)
{
	int iRes = -1;

	iRes = dpDisconnect("gcsAnalogComputed_ButtonAnimationCB",
				 sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
				 sDeviceName + ".statusInformation.selectedManager",
				 sDeviceName + ".ProcessInput.StsReg01",
				 sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv");

	g_bCallbackConnected = !(iRes >= 0);
}














//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_ButtonAnimationCB
/**
Animate buttons

@note This function works with an unicos button panel and needs global variables declared in this panel -
		"g_dsUserAccess" (dyn_string). Button identifier is added to this variable if user has access.
*/
gcsAnalogComputed_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
     string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
	gcsAnalogComputed_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), GCS_CONFIG_ANALOGCOMPUTED_DPT_NAME, g_dsUserAccess,
		makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, 0, 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_ButtonUserAccess
/**
Provides user access for buttons

	@param[in] 	sDpName		Device name
	@param[in] 	sDpType		dpType
	@param[out] 	dsAccess	List of allowed buttons
	@sa gcsAnalogComputed_ButtonUserAccess, 
*/
gcsAnalogComputed_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	
	bool operator, expert, admin;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;
	
	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
	(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
	bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	
	if (operator){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
				UN_FACEPLATE_BUTTON_AC_SET_VALUE, UN_FACEPLATE_BUTTON_SELECT));
	}
	if (expert){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_MASKEVENT));
	}
	if(admin){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_UserLoginGetButtonState
/**
Function is called before the button set state when a user logs in.

	@param[in]	deviceName		The device name
	@param[in]	sType			Device type <b>(Currently not used)</b>
	@param[out]	dsReturnData	The data are returned here
	@note	This library works with an unicos button panel and need global variables declared in this panel
		- g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
		- $sDpName, string, the device DP name
		- g_sFaceplateButtonType, string, faceplate type
*/
gcsAnalogComputed_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_ButtonSetState
/**
Set button state

	@param[in]	 sDeviceName		The device name
	@param[in]	 sDpType		Device type
	@param[in]	 dsUserAccess	User access
	@param[in]	 dsData			List of various parameters
*/
gcsAnalogComputed_ButtonSetState(string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, bButtonEnabled, bit1, bit2, bit3;
	dyn_string dsButtons;
	int i, iLength;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedUIManager;
	bit32 stsReg01Value, stsReg02Value;

	if (dynlen(dsData) >= 6)
	{
		bLocked = dsData[1];
		sSelectedUIManager = dsData[2];
		stsReg01Value = dsData[3];
		bStsReg01Bad = dsData[4];
		stsReg02Value = dsData[5];
		bStsReg02Bad = dsData[6];
	}


	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_AC_SET_VALUE,
		 		  UN_FACEPLATE_BUTTON_FORCED_MODE); // Each button except "Select" button
	iLength = dynlen(dsButtons);

	// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
	bSelected = (localManager == "S");
	// 2. Button state
	for(i = 1; i <= iLength; i++)
	{
		bButtonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad);
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_MMOST) == 0) && !(bStsReg01Bad);
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHMMO) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1);
				break;
			case UN_FACEPLATE_BUTTON_AC_SET_VALUE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad);
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad);
				bit3 = !(getBit(stsReg01Value, UN_STSREG01_AUINHMMO) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1 || bit2 || bit3);
				break;
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", bButtonEnabled);
		}
	// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_ButtonDisconnection
/**
Animate disconnection
*/
gcsAnalogComputed_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
		UN_FACEPLATE_BUTTON_AC_SET_VALUE, UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_SELECT));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_WidgetRegisterCB
/**
Register callback function

	@param[in]	sDp 			The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsAnalogComputed_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string sDeviceName, sPlcName, sPlcDpName,sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (sDeviceName == "")	// In case of disconnection
	{
		sDeviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);

	//sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote) g_bSystemConnected = bSystemConnected;
	else g_bSystemConnected = true;

	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

	switch (iAction)
	{
		case UN_ACTION_DISCONNECT:
			gcsAnalogComputed_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = sDeviceName;
			if(dpTypeName(sDeviceName) == GCS_CONFIG_ANALOGCOMPUTED_DPT_NAME)
			{
				gcsAnalogComputed_WidgetConnect(sDeviceName, sPlcName, sSystemName);
			}
			else
			{
				g_bUnSystemAlarmPlc = false;
				gcsAnalogComputed_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			gcsAnalogComputed_WidgetDisconnect (sDeviceName, sPlcName, sSystemName);
			gcsAnalogComputed_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING: break;
		default:
			break;
	}
}


//------------------------------------------------------------------------------------------------------------
//gcsAnalogComputed_WidgetConnect
/**
dpConnect to the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
gcsAnalogComputed_WidgetConnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes = -1;

	if (g_bUnSystemAlarmPlc)
	{
		g_sPosOvFormat = dpGetFormat(sDeviceName + ".ProcessInput.PosOV");
		g_sPosOvUnit = dpGetUnit(sDeviceName + ".ProcessInput.PosOV");

		iRes = dpConnect("gcsAnalogComputed_WidgetAnimationCB",
			sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			sDeviceName + ".statusInformation.selectedManager",
			sDeviceName + ".ProcessInput.StsReg01",
			sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sDeviceName + ".ProcessInput.PosOV",
			sDeviceName + ".ProcessInput.PosOV:_online.._invalid",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			sDeviceName + ".eventMask");
		g_bCallbackConnected = (iRes >= 0);
	}
	else
		gcsAnalogComputed_WidgetDisconnection(g_sWidgetType);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_WidgetDisconnect
/**
dpDisconnect from the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
gcsAnalogComputed_WidgetDisconnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes = -1;
	if (g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("gcsAnalogComputed_WidgetAnimationCB",
			sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			sDeviceName + ".statusInformation.selectedManager",
			sDeviceName + ".ProcessInput.StsReg01",
			sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sDeviceName + ".ProcessInput.PosOV",
			sDeviceName + ".ProcessInput.PosOV:_online.._invalid",
		 	sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			sDeviceName + ".eventMask");
		g_bCallbackConnected = !(iRes >= 0);
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_WidgetAnimationCB
/**
Animate widget
*/
gcsAnalogComputed_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
		string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
		string sDpPosOV, float fPosOV, string sDp6, bool bPosOVInvalid,
		string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
		string sEvtMask, int iMask)

{
	gcsGenericObject_WidgetXC_Animation(bLocked, sSelectedManager, bit32StsReg01, bStsReg01Invalid,
        sDpPosOV, fPosOV, bPosOVInvalid, bStsReg01Invalid || bPosOVInvalid, iAlValue, bPlcEnabled, sEvtMask, iMask);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogComputed_WidgetDisconnection
/**
Animate widget disconnection

	@param[in] sWidgetType Widget type. ex: "heater"
*/
gcsAnalogComputed_WidgetDisconnection(string sWidgetType)
{
	//Standard Unicos function to animate widget disconnection cannot be used as AnalogComputed does not have "AlarmText" graphical object
	//on the widget panel.
	//unGenericObject_WidgetDisconnection(sWidgetType, GCS_CONFIG_ANALOGCOMPUTED_DPT_NAME);
	
	setMultiValue("Body1", "foreCol", "unDataNoAccess",
		      "Body1", "backCol", "unWidget_Background",
		      "Body1", "fill", "[solid]",
		      "WarningText", "text", "", 
		      "ControlStateText", "text", "",
		      "SelectArea", "foreCol", "",
		      "LockBmp", "visible", false,
		      "WidgetArea", "visible", false);
		      
	eventState.visible	= false;
}
//@}
