/**@name LIBRARY: gcsAnalogStatus.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains AnalogStatus object functions.
Modifications History:
	
	$Id: gcsAnalogStatus.ctl 46255 2010-09-16 07:38:35Z gthomas $:
	Fixed variable typo in gcsAnalogStatus_WidgetAnimationCB()
			
	$Log: gcsAnalogStatus.ctl,v $
	Revision 1.19  2009/03/27 14:15:58  gthomas
	 in _WidgetRegisterCB() added evaluation on widget type for Word2AnalogStatus
	
	Revision 1.18  2009/03/25 10:07:01  gthomas
	-Fixed animation problem when the $sIdentifier of the widget refers to a DPname of a different DP type.
	Fixes made in _WidgetRegisterCB()

	Revision 1.17  2008/07/23 10:02:25  gthomas
	-removed commented code (for error handling)

	Revision 1.16  2008/06/13 12:29:47  aabadz
	Modifications asked by UNICOS for their version 4.X

	Revision 1.15  2008/06/03 08:01:29  aabadz
	Fixed error in background color animation

	Revision 1.14  2008/05/26 12:18:18  gthomas
	Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
	to be used with PVSS 3.6 QT

	Revision 1.13  2008/03/17 10:52:36  aabadz
	Fixed a problem found in animation when we have not database.

	Revision 1.12  2007/12/05 11:15:34  aabadz
	Corrections following the document from code review. Version after testing

	Revision 1.11  2007/11/27 14:15:14  aabadz
	Corrections following the document from code review

	Revision 1.10  2007/10/30 16:14:45  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

		17/12/2007 Enrique (AB/CO)
		- Delete functions:
				AnalogStatus_EventList_Configure_Init()
				AnalogStatus_EventList_Configure_Clic()
				AnalogStatus_EventList_Configure_Save()

		16/01/2008 : Arnaud AB/CO
		- Modify  WidgetAnimationCB to use it with UNICOS step and transition widgets


	  	20/08/2007: Enrique AB/CO
	  	-function gcsAnalogStatus_ObjectListGetValueTime added.
		11/05/2007: Herve AB/CO


		11/05/2004: Herve AB/CO
		- function gcsAnalogStatus_WidgetRegisterCB
		bug in case the PLC is not defined and if the PVSS system name contains '-',
		call unGenericObject_GetPLCNameFromDpName to get the PLC DPName instead (this function corrects the bugs).

*/
/*
Internal Functions :
	gcsAnalogStatus_MenuConfiguration (string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
	gcsAnalogStatus_HandleMenu (string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
	gcsAnalogStatus_WidgetRegisterCB (string sDp, bool bSystemConnected)
	gcsAnalogStatus_WidgetConnect (string deviceName, string sPlcName, string sSystemName)
	gcsAnalogStatus_WidgetDisconnect (string deviceName, string sPlcName, string sSystemName)
	gcsAnalogStatus_WidgetAnimationCB (string sDpLock, bool bLocked, string sDpManager, string sSelectedManager, string sDpAuPosRSt, float fAuPosRSt, ...)
	gcsAnalogStatus_WidgetDisconnection (string sWidgetType)
*/


//------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_MenuConfiguration
/**
Configuration of popup menu

	@param[in]  sDpName Device name
	@param[in]	sDpType Unicos object
	@param[in]	dsAccessOk	List of authorized action
	@param[out] menuList Configuration of menu. Use defined constants UN_POPUPMENU_***
*/
gcsAnalogStatus_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_DIAG_INFO_TEXT, UN_POPUPMENU_TREND_TEXT);

	unGenericObject_addDiagnosticAndInfoToActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_HandleMenu
/**
Handle the answer of the popup menu

	@param[in]	deviceName 	The device dp name
	@param[in]	sDpType 	The device dp type
	@param[in]	menuList 	List of requested action
	@param[in]	menuAnswer 	Selected action from the menuList
*/
gcsAnalogStatus_HandleMenu(string deviceName, string sDpType, dyn_string sMenuList, int sMenuAnswer)
{
	dyn_string exceptionInfo;

	unGenericObject_HandleUnicosMenu(deviceName, sDpType, sMenuList, sMenuAnswer, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_WidgetRegisterCB
/**
Register callback function

	@param[in]	sDp 	Flag for distributed system
	@param[in] 	bSystemConnected 	Flag
*/
gcsAnalogStatus_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string sDeviceName="", sPlcName="", sSystemName="";
	int iRes=0, iAction=UN_ACTION_NOTHING;
	dyn_string exceptionInfo;
	bool bRemote=false;

	sDeviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (sDeviceName == "")	// In case of disconnection
	{
		sDeviceName = g_sDpName;
	}

	// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	// get the PLC name
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	if ("" != sSystemName)
		unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	if(sPlcName != "")
	{
		string sPlcDpName = "";

		dyn_string dsPlcDpNameExceptions;
		gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);

		if (dynlen(dsPlcDpNameExceptions) == 0)
		{
			g_bUnSystemAlarmPlc = true;
		}
		else
		{
			g_bUnSystemAlarmPlc = false;
		}
	}
	else	g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

	switch (iAction)
	{

		case UN_ACTION_DISCONNECT:
			gcsAnalogStatus_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = sDeviceName;
			if( (dpTypeName(sDeviceName) == "AnalogStatus")||(dpTypeName(sDeviceName) == "Word2AnalogStatus") )
				//gcsAnalogStatus_WidgetConnect(sDeviceName, sPlcName, sSystemName);
  					iRes = dpConnect("gcsAnalogStatus_WidgetLockCallBack", sDeviceName + ".ProcessInput.AuPosRSt:_lock._alert_hdl._locked");
			else{
				g_bUnSystemAlarmPlc = false;
				gcsAnalogStatus_WidgetDisconnection(g_sWidgetType);
			}
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			if(g_bDisconnectCB) {
				switch(g_sWidgetType) {

				case "AS":
					gcsAnalogStatus_WidgetDisconnect(sDeviceName, sPlcName, sSystemName);
					gcsAnalogStatus_WidgetDisconnection(g_sWidgetType);
					break;

				case "step_PLC":
					gcsAnalogStatus_step_PLC_WidgetDisconnect(g_sDpName, iRes);
					break;
				case "transition_PLC":
					gcsAnalogStatus_transition_PLC_WidgetDisconnect(g_sDpName, iRes);
					break;

				default:
					break;
				}
			}
			break;

		case UN_ACTION_NOTHING:break;
		default:
			break;
	}
}

//--------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_WidgetConnect
/**
dpConnect to the DataPoint

	@param[in]	deviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/


gcsAnalogStatus_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
  //DebugN("gcsAnalogStatus_WidgetConnect is called!!!!!!!");
	int iRes;
 
	if (g_bUnSystemAlarmPlc)
	{
		g_sAuPosRStFormat = dpGetFormat(deviceName + ".ProcessInput.AuPosRSt");
		g_sAuPosRStUnit = dpGetUnit(deviceName + ".ProcessInput.AuPosRSt");

		string sPlcDpName = "";

		dyn_string dsPlcDpNameExceptions;
		gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);

		if (dynlen(dsPlcDpNameExceptions) == 0)
		{
			iRes = dpConnect("gcsAnalogStatus_WidgetAnimationCB",
				deviceName + ".statusInformation.selectedManager:_lock._original._locked",
				deviceName + ".statusInformation.selectedManager",
				deviceName + ".ProcessInput.AuPosRSt",
				deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled");
			g_bCallbackConnected = (iRes >= 0);

		}
	}
	else gcsAnalogStatus_WidgetDisconnection (g_sWidgetType);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_WidgetDisconnect
/**
dpDisconnect from the DataPoint

	@param[in]	deviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
gcsAnalogStatus_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	if (g_bUnSystemAlarmPlc)
	{
		string sPlcDpName = "";

		dyn_string dsPlcDpNameExceptions;
		gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);

		if (dynlen(dsPlcDpNameExceptions) == 0)
		{
			iRes = dpDisconnect("gcsAnalogStatus_WidgetAnimationCB",
				deviceName + ".statusInformation.selectedManager:_lock._original._locked",
				deviceName + ".statusInformation.selectedManager",
				deviceName + ".ProcessInput.AuPosRSt",
				deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
				sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName + ".alarm",
				sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName + ".enabled");
			g_bCallbackConnected = !(iRes >= 0);
		}
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_WidgetAnimationCB
/**
Animate widget
*/
gcsAnalogStatus_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
		  string sDpAuPosRSt, float fAuPosRSt, string sDp4, bool bAuPosRStInvalid,
		  string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled)
{
	string sFormattedValue;
	string sBody1Text, sBody1Color;
	string sWarningLetter, sWarningColor; // put new variables

	if((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)) // put here if sentance
	{
		sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
   //DebugN("WidgetAnimationCB unDataNotValid");
	}

	sFormattedValue = unGenericObject_FormatValue(g_sAuPosRStFormat, fAuPosRSt);
	sBody1Text = sFormattedValue + " " + g_sAuPosRStUnit;
	// 1. Control animation. Check if the datapoint is invalid, then will be displayed in blue colors
	if (bAuPosRStInvalid)
	{
    //DebugN("Check if the datapoint is invalid...");
		sBody1Color = "unDataNotValid";
		sWarningLetter = UN_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	}
	else
	{
		sBody1Color = "unWidget_ControlStateAuto";
	}
	if(g_sWidgetType=="AS"){
	// 8. Animate widget
    	 if(g_bSystemConnected){
         //DebugN("system connected");
      		setMultiValue("Body1", "text", sBody1Text,"Body1", "foreCol", sBody1Color,
                				"WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
                				"WidgetArea", "visible", true);
        }
	}
	if(g_sWidgetType=="step_PLC"){
    //DebugN("animate widget of type unStep_plc");
			// 8. Animate widget
			if(g_bSystemConnected){
		   		setValue("step_Plc","backCol",sBody1Color);
	  		}
	 }
	 if(g_sWidgetType=="transition_PLC"){
			// 8. Animate widget
			if(g_bSystemConnected){
	   			setValue("line_transition","foreCol",sBody1Color);
	  		}
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_WidgetDisconnection
/**
Animate widget disconnection

	@param[in] sWidgetType Widget type. ex: "heater"
*/
gcsAnalogStatus_WidgetDisconnection(string sWidgetType)
{
	switch(sWidgetType) {
		case "AS":
			setMultiValue("Body1", "foreCol", "unDataNoAccess","Body1", "backCol", "unWidget_Background", 
      				       "WarningText", "text", "", "WidgetArea", "visible", false);
			break;
		case "step_PLC":
			gcsAnalogStatus_step_PLC_disconnection();
			break;
		case "transition_PLC":
			gcsAnalogStatus_transition_PLC_disconnection();
			break;
		default:
			break;
	}
}




// gcsAnalogStatus__ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: XP.
	. distributed system: yes.
*/

gcsAnalogStatus_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime, sStatus,sColor;
	bool bBad;

	dpGet(sDeviceName+".ProcessInput.AuPosRSt", value,
	      sDeviceName+".ProcessInput.AuPosRSt:_online.._stime", sTime,
	      sDeviceName+".ProcessInput.AuPosRSt:_online.._bad", bBad);

	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.AuPosRSt"), value) + " "
							+ dpGetUnit(sDeviceName+".ProcessInput.AuPosRSt");

	// Optionally (when no stsReg* exists)
	// Status
	dsReturnData[3] = bBad;

	// Status to visualize
	sStatus="OK";
	dsReturnData[4] = sStatus;
	// Colors
	sColor="unEventList_ForeColor";
	dsReturnData[5]=sColor;
}

gcsAnalogStatus_step_PLC_WidgetConnect(string deviceName)
{
	int iRes, iAlertType1, iRangeType;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;


// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName))
			{
			g_bUnSystemAlarmPlc = true;

			iRes = dpConnect("gcsAnalogStatus_step_PLC_CB", deviceName + ".ProcessInput.AuPosRSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
			}
		else
			{
			g_bUnSystemAlarmPlc = false;
			gcsAnalogStatus_step_PLC_disconnection();
			}
		}
	else
		{
		g_bUnSystemAlarmPlc = false;
		gcsAnalogStatus_step_PLC_disconnection();
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_step_PLC_CB
/**
Purpose: callback function for step PLC

Parameters:
	- sDp1, string, input, datapoint nane
	- fPosSt, float, input, value of datapoint
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsAnalogStatus_step_PLC_CB (string sDp1, int iPosSt,
			 string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled)
{
string sColor;

	if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value))
		{
		sColor = "unDataNotValid";
		//DebugN("StepPLC unDataNotValid");
	}
	else {

		if (iPosSt==g_iValue)
			sColor=g_sColor_Active;
		else
			sColor=g_sColor_Passive;
	}
	if(g_bSystemConnected){
		setValue("step_Plc","backCol",sColor);
	}

}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_step_PLC_WidgetDisconnect
/**
Purpose: Animate Step widget disconnect

Parameters:
	- deviceName, string, Device Name to disconnect
	- iRes, int, output, Disconnection result

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsAnalogStatus_step_PLC_WidgetDisconnect(string deviceName, int &iRes)
{
string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		// get the PLC name
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		iRes = dpDisconnect("gcsAnalogStatus_step_PLC_CB", deviceName + ".ProcessInput.AuPosRSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_step_PLC_disconnection
/**
Purpose: Animate Step widget disconnection

Parameters:

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsAnalogStatus_step_PLC_disconnection()
{
  //DebugN("called gcsAnalogStatus_step_PLC_disconnection()");
	step_Plc.backCol = "unDataNoAccess";
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_transition_PLC_WidgetConnect
/**
Purpose: Animate Transition widget connection

Parameters:
	- iRes, string, input, device Name to connect

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsAnalogStatus_transition_PLC_WidgetConnect(string deviceName)
{
int iRes, iAlertType1, iRangeType;
dyn_string exceptionInfo;
string sPlcName, sSystemName;

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

	if (g_iBit > UN_TRANSITION_PLC_MAX_POSITION)
		sPlcName="";

	if (sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName))
			{
			g_bUnSystemAlarmPlc = true;
			iRes = dpConnect("gcsAnalogStatus_transition_PLC_CB", deviceName + ".ProcessInput.AuPosRSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
			}
		else
			{
			g_bUnSystemAlarmPlc = false;
			gcsAnalogStatus_transition_PLC_disconnection();
			}
		}
	else {
		g_bUnSystemAlarmPlc = false;
		gcsAnalogStatus_transition_PLC_disconnection();
		}

}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_transition_PLC_CB
/**
Purpose: callback function for step PLC

Parameters:
	- sDp1, string, input, datapoint nane
	- iPosSt, iloat, input, value of datapoint (int to convert in bit32)
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsAnalogStatus_transition_PLC_CB (string sDp1, int iPosSt,
			 string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled)
{
	string sColor;
	bit32 bitPost;

	bitPost=iPosSt;

	if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value)){
		//DebugN("transitionPLC unDataNotValid");
		sColor = "unDataNotValid";
	}
	else {
		if (getBit(bitPost,g_iBit))
			sColor=g_sColor_Active;
		else
			sColor=g_sColor_Passive;
	}
	if(g_bSystemConnected)
		setValue("line_transition","foreCol",sColor);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_transition_PLC_WidgetDisconnect
/**
Purpose: Animate Transition widget disconnect

Parameters:
	- deviceName, string, input, device name to disconnect
	- iRes, int, output, Disconnection result

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsAnalogStatus_transition_PLC_WidgetDisconnect(string deviceName, int &iRes)
{
string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		// get the PLC name
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		iRes = dpDisconnect("gcsAnalogStatus_transition_PLC_CB", deviceName + ".ProcessInput.PosRSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
		}

}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogStatus_transition_PLC_disconnection
/**
Purpose: Animate Transition widget disconnection

Parameters:

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsAnalogStatus_transition_PLC_disconnection()
{
	line_transition.foreCol = "unDataNoAccess";
}

//@}
gcsAnalogStatus_WidgetLockCallBack(string sdpe, bool bLockState)
{
  
  //DebugN("gcsAnalogStatus_WidgetLockCallBack is called");
  
string deviceName, sPlcName, sSystemName;

 deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
	{
		deviceName = g_sDpName;
	}

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	
	int iRes;

	if(bLockState) { // lock set so disconnect if not already disconnected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
    //DebugN("g_sWidgetType: " +g_sWidgetType + "bLockState= "+bLockState);
		if(g_bDisconnectCB) {
			switch(g_sWidgetType) {
				case "AS":
					gcsAnalogStatus_WidgetDisconnect(g_sDpName, iRes);
					break;

				case "step_PLC":
					//DebugN("gcsAnalogStatus_step_PLC_WidgetDisconnect");
					gcsAnalogStatus_step_PLC_WidgetDisconnect(g_sDpName, iRes);
					break;
				case "transition_PLC":
					gcsAnalogStatus_transition_PLC_WidgetDisconnect(g_sDpName, iRes);
					break;
				default:
					break;
			}
			g_bDisconnectCB = false;
		}
		gcsAnalogStatus_WidgetDisconnection(g_sWidgetType);
	}
	else { // lock not set so connect if not already connected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		g_bDisconnectCB = true;
    //DebugN("g_sWidgetType: "+g_sWidgetType+ " is already connected..");
    
		switch(g_sWidgetType) {
			case "AS":
				gcsAnalogStatus_WidgetConnect(g_sDpName, sPlcName, sSystemName);
				break;
			case "step_PLC":
				gcsAnalogStatus_step_PLC_WidgetConnect(g_sDpName);
				break;
			case "transition_PLC":
				gcsAnalogStatus_transition_PLC_WidgetConnect(g_sDpName);
				break;
			default:
				break;
		}

	}
}


