/**@name LIBRARY: gcsConfigAnalogComputed.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains the function used to set Unicos AnalogComputed configuration.
Modifications History:
  $Log: gcsConfigAnalogComputed.ctl,v $
  Revision 1.17  2009/03/12 10:40:09  gthomas
  -Fixed variable typo in gcsConfigAnalogComputed_setConfig function

  Revision 1.16  2008/09/04 14:41:12  gthomas
  -moved back to version 1.13

  Revision 1.15  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.14  2008/06/09 14:54:26  aabadz
  Result after move some constants, only use use inside this file, from gcsObjects_declarations.

  Revision 1.13  2008/01/09 13:40:09  gamx
  reviewed:
  1) exception handling;
  2) naming;
  3) documenting

  Revision 1.12  2007/11/15 08:38:52  ikoblik
  gcsConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.11  2007/11/14 16:39:31  ikoblik
  gcsConfigGenericFunctions code review comments implementation

  Revision 1.10  2007/10/30 16:14:51  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
  gcsConfigAnalogComputed_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_AnalogComputed_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_AnalogComputed_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogComputed_setCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogComputed_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_AnalogComputed_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_AnalogComputed_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogComputed_checkCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
*/


// Constants
const string GCS_CONFIG_ANALOGCOMPUTED_DPT_NAME = "AnalogComputed";


//Constants indicate positions in the configuration line (for DB file)
//Specified position numbers are relative. First 10 positions are reserved for general Unicos fields.
const unsigned GCS_CONFIG_ANALOGCOMPUTED_LENGTH       = 15;

const unsigned GCS_CONFIG_ANALOGCOMPUTED_UNIT       = 1;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_FORMAT       = 2;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_RANGEMAX     = 3;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_RANGEMIN     = 4;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_DEADBAND     = 5;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_DEADBAND_TYPE    = 6;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_ARCHIVE_ACTIVE   = 7;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_ARCHIVE_TIME_FILTER = 8;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_STSREG01 = 9;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_EVSTSREG01 = 10;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_POSOV    = 11;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_AUPOSRST = 12;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_MANREG01 = 13;
const unsigned GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_MPOSR    = 14;

//Specified position for mask event is for case of no archives.
//In case of archiving, the position number should be incremented with UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH
const unsigned GCS_CONFIG_ANALOGCOMPUTED_MASKEVENT      = 15;

//const unsigned UN_CONFIG_ANALOGCOMPUTED_LENGTH        = 15;
//const unsigned UN_CONFIG_ANALOGCOMPUTED_ADDITIONAL_LENGTH = 0;
//const unsigned UN_CONFIG_ANALOGCOMPUTED_ARCHIVE_ACTIVE    = 7;





//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogComputed_setConfig
/**
  Function creates dp and sets configuration for the device.
  Configuration is taken from the DB line, corresponding to the object.

  @param[in]  dsConfigs   	Configs (see dyn_string indexes in constants defined)
    		dsConfigs is the concatination of the DB configuration line + additional UNICOS parameters. These additional parameters.
    		Values of additional UNICOS parameters are specified in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo 	All possible errors will be accumulated here.
*/
gcsConfigAnalogComputed_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;

  // 1. Initialization : checking + dpCreate
  //unConfigGenericFunctions_checkAll(dsConfigs, GCS_CONFIG_ANALOGCOMPUTED_DPT_NAME, exceptionInfo);


  if (dynlen(exceptionInfo) <= 0)
  {
    if (dynlen(dsConfigs)) sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, GCS_CONFIG_ANALOGCOMPUTED_DPT_NAME, exceptionInfo);

    if (dynlen(exceptionInfo) <= 0)
    {
        unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);

        //call the device set function
        if (dynlen(exceptionInfo) <= 0)
        {
        	unConfigGenericFunctions_setDeviceConfig(dsConfigs, GCS_CONFIG_ANALOGCOMPUTED_DPT_NAME, exceptionInfo);
        }
    }
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogComputed_setConfig
/**
  Function sets configuration for the processing device: units, formats, ranges, modbus addresses.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
_UnPlc_AnalogComputed_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogComputed_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogComputed_setConfig
/**
  Function sets configuration for the processing device: units, formats, ranges, modbus addresses in case of S7 plc.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
S7_PLC_AnalogComputed_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogComputed_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogComputed_setCommonConfig
/**
  Function sets configuration for the processing device: units, formats, ranges, modbus addresses.
  It is used in both cases: unPlc and S7Plc.

  @param[in]  sPlcType     	PLC type: UN_PLC_DPTYPE or S7_PLC_DPTYPE
  @param[in]  dsConfigs  	dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
    				These additional parameters are added during the import process.
  				Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	All possible errors will be returned here
*/
gcsConfigAnalogComputed_setCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDescription;
  string sDpName;
  string sMask;
  string sScript;
  int iMask = 0;



  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(makeDynString(),"ERROR","gcsConfigAnalogComputed_checkCommonConfig: Unknown PLC type - " + sPlcType,"");
    return;
  }


  if (dynlen(dsConfigs) >= UN_CONFIG_OBJECTGENERAL_DPNAME) sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

  //Gets archiving configs from object and puts them to the common archiving configs.
  gcsConfigGenericFunctions_setArchives(GCS_CONFIG_ANALOGCOMPUTED_LENGTH, dsConfigs);



  // 2.2. Registers
  if(sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setAllRegisters(dsConfigs, \n";
  else if(sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setAllRegistersS7(dsConfigs, \n";
  else
  {
  	dynAppend(exceptionInfo, "Specified PLC type is unknown: " + sPlcType);
  	return;
  }

  sScript += "      makeDynString(\"StsReg01\"), \n";
  sScript += "      makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_STSREG01), \n";
  sScript += "      makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_EVSTSREG01), \n";
  sScript += "      makeDynString(\"ManReg01\"), \n";
  sScript += "      makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_MANREG01), \n";
  sScript += "      makeDynString(), makeDynString(), makeDynInt(), exceptionInfo); \n";

  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + sScript + " return exceptionInfo;}", makeDynString(), dsConfigs);



  // 2.3. Float values without smoothing
  if (dynlen(exceptionInfo) <= 0)
  {
  	if(sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs,  PVSS_MODBUS_FLOAT, \n";
  	else if(sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7_FLOAT, \n";
  	else
  	{
  		dynAppend(exceptionInfo, "Specified PLC type is unknown: " + sPlcType);
  		return;
  	}

  	sScript += "      makeDynString(\"AuPosRSt\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_AUPOSRST), \n";
  	sScript += "      makeDynString(\"MPosR\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_MPOSR), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_UNIT, GCS_CONFIG_ANALOGCOMPUTED_FORMAT, GCS_CONFIG_ANALOGCOMPUTED_DEADBAND, GCS_CONFIG_ANALOGCOMPUTED_DEADBAND_TYPE), \n";
  	sScript += "      makeDynInt(), makeDynString(), makeDynInt(), exceptionInfo); \n";

  	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + sScript + " return exceptionInfo;}", makeDynString(), dsConfigs);
  }



  // 2.4. Float values with smoothing
  if (dynlen(exceptionInfo) <= 0)
  {
  	if(sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs,  PVSS_MODBUS_FLOAT, \n";
  	else if(sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7_FLOAT, \n";
  	else
  	{
  		dynAppend(exceptionInfo, "Specified PLC type is unknown: " + sPlcType);
  		return;
  	}

  	sScript += "      makeDynString(\"PosOV\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_POSOV), \n";
  	sScript += "      makeDynString(), makeDynInt(), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_UNIT, GCS_CONFIG_ANALOGCOMPUTED_FORMAT, GCS_CONFIG_ANALOGCOMPUTED_DEADBAND, GCS_CONFIG_ANALOGCOMPUTED_DEADBAND_TYPE), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_RANGEMIN, GCS_CONFIG_ANALOGCOMPUTED_RANGEMAX), \n";
  	sScript += "      makeDynString(\"PosOV\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ARCHIVE_ACTIVE, GCS_CONFIG_ANALOGCOMPUTED_ARCHIVE_TIME_FILTER), \n";
  	sScript += "      exceptionInfo); \n";

  	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + sScript + " return exceptionInfo;}", makeDynString(), dsConfigs);
  }

  if (dynlen(exceptionInfo) <= 0)
  {
  	sMask = dsConfigs[GCS_CONFIG_ANALOGCOMPUTED_MASKEVENT + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH];
  	//unConfigGenericFunctions_checkInt(sMask, iMask, dsExceptionInfo); -- to be checked why is not working.
  	//used instead of scanf
  	if (dynlen(exceptionInfo) <=0) unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
  }
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogComputed_checkConfig
/**
  Function checks if specified device configuration is correct.
  Configuration is taken from the DB line, corresponding to the object.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
gcsConfigAnalogComputed_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  unsigned uUnsigned;

  if(dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH)
  {
    unConfigGenericFunctions_checkParameters(dsConfigs, GCS_CONFIG_ANALOGCOMPUTED_DPT_NAME, exceptionInfo);
    if (dynlen(exceptionInfo) <= 0) unConfigGenericFunctions_checkDeviceConfig(dsConfigs, GCS_CONFIG_ANALOGCOMPUTED_DPT_NAME, exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogComputed_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogComputed_checkConfig
/**
  Checks specified configuration in case of unPlc front-end type

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
_UnPlc_AnalogComputed_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogComputed_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogComputed_checkConfig
/**
  Checks specified configuration in case of S7 front-end type

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
S7_PLC_AnalogComputed_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
    gcsConfigAnalogComputed_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogComputed_checkCommonConfig
/**
  Checks specified configuration for the device.

  @param[in]  sPlcType     	  PLC type: UN_PLC_DPTYPE or S7_PLC_DPTYPE.
  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
    				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of exceptions
*/
gcsConfigAnalogComputed_checkCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sScript;
  bool bHasArchive = false;


  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogComputed_checkCommonConfig: Unknowm PLC type - " + sPlcType,"");
  }

  if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_ANALOGCOMPUTED_LENGTH, dsConfigs, bHasArchive))
  {
    fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". AnalogComputed_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
  
  
  if (bHasArchive)
  {
    gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_ANALOGCOMPUTED_LENGTH, dsConfigs, exceptionInfo);
    
    if (dynlen(exceptionInfo) > 0)
    {
      fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". AnalogComputed_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    }
  }



  // 2. Registers
  if(sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_checkAllRegisters(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  else if(sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_checkAllRegistersS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  else
  {
  	fwException_raise(exceptionInfo, "ERROR","gcsConfigAnalogComputed_checkCommonConfig: unknown plc type: " + sPlcType, "");
  	return;
  }

  sScript += "    makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_STSREG01), \n";
  sScript += "    makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_EVSTSREG01), \n";
  sScript += "    makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_MANREG01), \n";
  sScript += "    makeDynInt(), exceptionInfo); \n";

  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + sScript + " return exceptionInfo;}", makeDynString(), dsConfigs);




  // 3. Float block
  if(sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  else if(sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  else
  {
  	fwException_raise(exceptionInfo, "ERROR","gcsConfigAnalogComputed_checkCommonConfig: unknown plc type: " + sPlcType, "");
  	return;  	
  }
  sScript += "    makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_POSOV, GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_AUPOSRST), \n";
  sScript += "    makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ADDRESS_MPOSR), \n";
  sScript += "    makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_UNIT, GCS_CONFIG_ANALOGCOMPUTED_FORMAT, GCS_CONFIG_ANALOGCOMPUTED_DEADBAND, GCS_CONFIG_ANALOGCOMPUTED_DEADBAND_TYPE), \n";
  sScript += "    makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_RANGEMIN, GCS_CONFIG_ANALOGCOMPUTED_RANGEMAX), \n";
  sScript += "    makeDynInt(GCS_CONFIG_ANALOGCOMPUTED_ARCHIVE_ACTIVE, GCS_CONFIG_ANALOGCOMPUTED_ARCHIVE_TIME_FILTER), \n";
  sScript += "    exceptionInfo); \n";

  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + sScript + " return exceptionInfo;}", makeDynString(), dsConfigs);
}


//@}
