/**@name LIBRARY: gcsConfigAnalogParameter.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains the function used to set AnalogParameter configuration.
Modifications History:
  $Log: gcsConfigAnalogParameter.ctl,v $
  Revision 1.24  2011/05/02 12:00:00  Herve
  IS-510: CPC 5 and CPC 6 compatibility of the export/import of the CPC device configuration with all the front-end types
  
  Revision 1.23  2008/07/28 12:46:31  gamx
  exception handling has been restored

  Revision 1.22  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.21  2008/01/21 15:32:02  aabadz
  Introduce constants instead of some strings, modified some variable names to follow the name convention and use some external functiosn instead of repeat the sme code

  Revision 1.20  2007/11/15 08:38:52  ikoblik
  gcsConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.19  2007/11/14 16:39:31  ikoblik
  gcsConfigGenericFunctions code review comments implementation

  Revision 1.18  2007/10/30 16:20:45  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

/*
Internal Functions :
  gcsConfigAnalogParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_AnalogParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_AnalogParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogParameter_setCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_AnalogParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_AnalogParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogParameter_checkCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
*/

// Constants
const string GCS_CONFIG_ANALOGPARAMETER_DPT_NAME = "AnalogParameter";

// Constants
const unsigned GCS_CONFIG_ANALOGPARAMETER_LENGTH = 12;

const unsigned GCS_CONFIG_ANALOGPARAMETER_UNIT = 1;
const unsigned GCS_CONFIG_ANALOGPARAMETER_FORMAT = 2;
const unsigned GCS_CONFIG_ANALOGPARAMETER_DEADBANDE = 3;
const unsigned GCS_CONFIG_ANALOGPARAMETER_DEADBANDE_TYPE = 4;
const unsigned GCS_CONFIG_ANALOGPARAMETER_RANGEMAX = 5;
const unsigned GCS_CONFIG_ANALOGPARAMETER_RANGEMIN = 6;
const unsigned GCS_CONFIG_ANALOGPARAMETER_ARCHIVE_ACTIVE = 7;
const unsigned GCS_CONFIG_ANALOGPARAMETER_ARCHIVE_TIME_FILTER = 8;
const unsigned GCS_CONFIG_ANALOGPARAMETER_ADDRESS_VALRQACK = 9;
const unsigned GCS_CONFIG_ANALOGPARAMETER_ADDRESS_CURVALST = 10;
const unsigned GCS_CONFIG_ANALOGPARAMETER_ADDRESS_VALRQ = 11;
const unsigned GCS_CONFIG_ANALOGPARAMETER_DEFAULT_VALUE = 12;

const unsigned UN_CONFIG_ANALOGPARAMETER_LENGTH = 12;
const unsigned UN_CONFIG_ANALOGPARAMETER_ADDITIONAL_LENGTH = 0;
const unsigned UN_CONFIG_ANALOGPARAMETER_ARCHIVE_ACTIVE = 7;

// constant declaration for AnalogParameter device used with SOFT_FE front-end
const unsigned UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH = 8;
const unsigned UN_CONFIG_SOFT_FE_ANALOGPARAMETER_UNIT = 1;
const unsigned UN_CONFIG_SOFT_FE_ANALOGPARAMETER_FORMAT = 2;
const unsigned UN_CONFIG_SOFT_FE_ANALOGPARAMETER_RANGEMAX = 3;
const unsigned UN_CONFIG_SOFT_FE_ANALOGPARAMETER_RANGEMIN = 4;
const unsigned UN_CONFIG_SOFT_FE_ANALOGPARAMETER_DEFAULT_VALUE = 5;
const unsigned UN_CONFIG_SOFT_FE_ANALOGPARAMETER_AUTONOMOUS = 6;
const unsigned UN_CONFIG_SOFT_FE_ANALOGPARAMETER_ARCHIVE_ACTIVE = 7;
const unsigned UN_CONFIG_SOFT_FE_ANALOGPARAMETER_ARCHIVE_TIME_FILTER = 8;


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogParameter_setConfig
/**
Set configuration of given AnalogParamater datapoint

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName, sDescription;

// 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, "AnalogParameter", exceptionInfo);

  if (dynlen(exceptionInfo) <= 0)   // Good inputs
  {
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, "AnalogParameter", exceptionInfo);
    if (dynlen(exceptionInfo) <= 0) // Creation ok or dp already exists
    {
// 2. Generate device
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
      unConfigGenericFunctions_setDeviceConfig(dsConfigs, "AnalogParameter", exceptionInfo);
    }
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogParameter_setConfig
/**
Set configuration of given AnalogParameter datapoint for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_AnalogParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogParameter_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogParameter_setConfig
/**
Set configuration of given AnalogParameter datapoint for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_AnalogParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogParameter_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogParameter_setCommonConfig
/**
Set configuration of given AnalogParameter datapoint for both unPLc and S7_PLC front-end type

  @param[in]  plcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogParameter_setCommonConfig(string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDescription, sDpName, str;
  if(!gcsConfigGenericFunctions_checkPlcType(plcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogParameter_setCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + plcType,"");
    return;
  }
  sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite setting from import panel.
  gcsConfigGenericFunctions_setArchives(GCS_CONFIG_ANALOGPARAMETER_LENGTH, dsConfigs);

//  Float values. No smoothing. No pv_range
  if(plcType == UN_PLC_DPTYPE)
    str = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs,  PVSS_MODBUS_FLOAT, \n";
  if(plcType == S7_PLC_DPTYPE)
    str = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7_FLOAT, \n";
  str += "      makeDynString(\"ValRqAck\"), \n";
  str += "      makeDynInt(GCS_CONFIG_ANALOGPARAMETER_ADDRESS_VALRQACK), \n";
  str += "      makeDynString(\"ValRq\"), \n";
  str += "      makeDynInt(GCS_CONFIG_ANALOGPARAMETER_ADDRESS_VALRQ), \n";
  str += "      makeDynInt(GCS_CONFIG_ANALOGPARAMETER_UNIT, GCS_CONFIG_ANALOGPARAMETER_FORMAT, GCS_CONFIG_ANALOGPARAMETER_DEADBANDE, GCS_CONFIG_ANALOGPARAMETER_DEADBANDE_TYPE), \n";
  str += "      makeDynInt(), makeDynString(), makeDynInt(), exceptionInfo); \n";
  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);

// can not use unConfigGenericFunctions_setCommonFloatIntBlock, besause it works ONLY with "processinput" objects
  unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.ValRq", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_ANALOGPARAMETER_UNIT], exceptionInfo);
  unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.ValRq", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_ANALOGPARAMETER_FORMAT], exceptionInfo);
  unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_ANALOGPARAMETER_UNIT], exceptionInfo);
  unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_ANALOGPARAMETER_FORMAT], exceptionInfo);
  dpSetWait(sDpName + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_ANALOGPARAMETER_DEFAULT_VALUE]);

// Float values. Smoothing. Pv_range
  if(plcType == UN_PLC_DPTYPE)
    str = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs,  PVSS_MODBUS_FLOAT, \n";
  if(plcType == S7_PLC_DPTYPE)
    str = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7_FLOAT, \n";
  str += "    makeDynString(\"CurValSt\"), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGPARAMETER_ADDRESS_CURVALST), \n";
  str += "    makeDynString(), makeDynInt(), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGPARAMETER_UNIT, GCS_CONFIG_ANALOGPARAMETER_FORMAT, GCS_CONFIG_ANALOGPARAMETER_DEADBANDE, GCS_CONFIG_ANALOGPARAMETER_DEADBANDE_TYPE), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGPARAMETER_RANGEMIN, GCS_CONFIG_ANALOGPARAMETER_RANGEMAX), \n";
  str += "    makeDynString(\"CurValSt\"), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGPARAMETER_ARCHIVE_ACTIVE, GCS_CONFIG_ANALOGPARAMETER_ARCHIVE_TIME_FILTER), \n";
  str += "    exceptionInfo); \n";
  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);

// Description for the Recipes
  sDescription = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DESCRIPTION];
  unConfigGenericFunctions_setDescription(sDpName + ".ProcessOutput.ValRq", sDescription, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogParameter_checkConfig
/**
Check data before configuration

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogParameter_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  if(dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH)
  {
  // 1. Common
    unConfigGenericFunctions_checkParameters(dsConfigs, "AnalogParameter", exceptionInfo);
  // 2. call the device check function
    unConfigGenericFunctions_checkDeviceConfig(dsConfigs, "AnalogParameter", exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogParameter_checkConfig: " + getCatStr("gcsGeneration","BADLENGTH"),"");
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogParameter_checkConfig
/**
Check data before configuration for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_AnalogParameter_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogParameter_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogParameter_checkConfig
/**
Check data before configuration for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_AnalogParameter_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
    gcsConfigAnalogParameter_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogParameter_checkCommonConfig
/**
Check configuration of given AnalogParameter datapoint for both unPLc and S7_PLC front-end type

  @param[in]  plcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogParameter_checkCommonConfig(string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string str;
  float fValue, fMin, fMax;

  if(!gcsConfigGenericFunctions_checkPlcType(plcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogParameter_checkCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + plcType,"");
    return;
  }
  bool bHasArchive = false;
  if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_ANALOGPARAMETER_LENGTH, dsConfigs, bHasArchive))
  {
    fwException_raise(exceptionInfo,"ERROR","with PLC=" + plcType + ". gcsConfigAnalogParameter_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    return;
  }
  if (bHasArchive){
    gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_ANALOGPARAMETER_LENGTH, dsConfigs, exceptionInfo);
    if (dynlen(exceptionInfo) != 0){
      fwException_raise(exceptionInfo,"ERROR","with PLC=" + plcType + ". gcsConfigAnalogParameter_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
      return;
    }
  }

  // 2. Float block
  if(plcType == UN_PLC_DPTYPE)
    str = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  if(plcType == S7_PLC_DPTYPE)
    str = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGPARAMETER_ADDRESS_VALRQACK, GCS_CONFIG_ANALOGPARAMETER_ADDRESS_CURVALST), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGPARAMETER_ADDRESS_VALRQ),  \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGPARAMETER_UNIT, GCS_CONFIG_ANALOGPARAMETER_FORMAT, GCS_CONFIG_ANALOGPARAMETER_DEADBANDE, GCS_CONFIG_ANALOGPARAMETER_DEADBANDE_TYPE),\n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGPARAMETER_RANGEMIN, GCS_CONFIG_ANALOGPARAMETER_RANGEMAX), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGPARAMETER_ARCHIVE_ACTIVE, GCS_CONFIG_ANALOGPARAMETER_ARCHIVE_TIME_FILTER), \n";
  str += "    exceptionInfo); \n";
  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);

  unConfigGenericFunctions_checkFloat(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGPARAMETER_DEFAULT_VALUE], fValue, exceptionInfo);
  unConfigGenericFunctions_checkFloat(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGPARAMETER_RANGEMIN], fMin, exceptionInfo);
  unConfigGenericFunctions_checkFloat(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGPARAMETER_RANGEMAX], fMax, exceptionInfo);
  if((fValue < fMin) || (fValue > fMax))
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogParameter_checkCommonConfig: " + getCatStr("gcsGeneration","BADRANGE"),"");
}

//------------------------------------------------------------------------------------------------------------------------

// SOFT_FE_AnalogParameter_setConfig
/**
Purpose: set configuration of given Analog Parameter datapoint for SOFT_FE front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.1 
	. operating system: wXP.
	. distributed system: yes.
*/
SOFT_FE_AnalogParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bit32 b32;
	bool bLLC;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnSOFT_FE_config);
// 2.3. Float values
	unConfigGenericFunctions_setCommonFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("CurValSt"),
					makeDynInt(),		
					makeDynInt(UN_CONFIG_SOFT_FE_ANALOGPARAMETER_RANGEMIN, UN_CONFIG_SOFT_FE_ANALOGPARAMETER_RANGEMAX),
					makeDynString("CurValSt"),
					makeDynInt(UN_CONFIG_SOFT_FE_ANALOGPARAMETER_ARCHIVE_ACTIVE, UN_CONFIG_SOFT_FE_ANALOGPARAMETER_ARCHIVE_TIME_FILTER),
					exceptionInfo);
	
	unConfigGenericFunctions_setCommonFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("ValRqAck"),
					makeDynInt(),		
					makeDynInt(UN_CONFIG_SOFT_FE_ANALOGPARAMETER_RANGEMIN, UN_CONFIG_SOFT_FE_ANALOGPARAMETER_RANGEMAX),
					makeDynString("ValRqAck"),
					makeDynInt(UN_CONFIG_SOFT_FE_ANALOGPARAMETER_ARCHIVE_ACTIVE, UN_CONFIG_SOFT_FE_ANALOGPARAMETER_ARCHIVE_TIME_FILTER),
					exceptionInfo);
  
  unConfigGenericFunctions_setUnit(sDpName + ".ProcessInput.CurValSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.CurValSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_FORMAT], exceptionInfo);
			
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessInput.ValRqAck", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.ValRqAck", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_FORMAT], exceptionInfo);
	
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.ValRq", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.ValRq", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_FORMAT], exceptionInfo);
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_FORMAT], exceptionInfo);
	dpSetWait(sDpName + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_DEFAULT_VALUE]);
	dpSetWait(sDpName + ".ProcessInput.CurValSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_DEFAULT_VALUE]);
	dpSetWait(sDpName + ".ProcessInput.ValRqAck", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_DEFAULT_VALUE]);
	dpSetWait(sDpName + ".ProcessOutput.ValRq", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_DEFAULT_VALUE]);

	// set the DpFunctions is the device is in autonomous mode:
	if (dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_AUTONOMOUS] == 1)
		{
			// dpe config
			// DebugN("dpe to sp_function: "+sDpName + ".ProcessInput.CurValSt");
			unGenericDpFunctions_createDpeDpFunction(sDpName + ".ProcessInput.CurValSt");
			unGenericDpFunctions_createDpeDpFunction(sDpName + ".ProcessInput.ValRqAck");
			
			// dpe assign
			unGenericDpFunctions_addDpeinDpFunction(sDpName + ".ProcessInput.CurValSt", sDpName + ".ProcessOutput.ValRq:_original.._value");
			unGenericDpFunctions_addDpeinDpFunction(sDpName + ".ProcessInput.ValRqAck", sDpName + ".ProcessOutput.ValRq:_original.._value");
		} 
	
}

//------------------------------------------------------------------------------------------------------------------------

// SOFT_FE_AnalogParameter_checkConfig
/**
Purpose: check data before configuration for SOFT_FE front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.1 
	. operating system: wXP.
	. distributed system: yes.
*/
SOFT_FE_AnalogParameter_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	float fValue, fMin, fMax;
	int iAuto;
	
	if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_SOFT_FE_ANALOGPARAMETER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;

	if(bOk) {
		unConfigGenericFunctions_checkUnitFormat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_UNIT], 
												 configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_FORMAT], exceptionInfo);
		unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_RANGEMIN], 
														configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_RANGEMAX], exceptionInfo);
		unConfigGenericFunctions_checkArchiveParameters(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_ARCHIVE_TIME_FILTER], 
														configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_ARCHIVE_ACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","SOFT_FE_AnalogParameter_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
	iAuto=configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_AUTONOMOUS];
	if((iAuto !=0) && (iAuto !=1))
		fwException_raise(exceptionInfo,"ERROR","AnalogParameter_checkConfig: " + getCatStr("unGeneration","BADRANGE (Autonomous)"),"");
		
	fValue=(float)configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_DEFAULT_VALUE];
	fMin=(float)configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_RANGEMIN];
	fMax=(float)configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGPARAMETER_RANGEMAX];
	if((fValue < fMin) || (fValue > fMax))
		fwException_raise(exceptionInfo,"ERROR","AnalogParameter_checkConfig: " + getCatStr("unGeneration","BADRANGE"),"");

}


//@}
