/**@name LIBRARY: gcsConfigAnalogStatus.ctl

@author: Artem Burmyakov (IT-CO-FE)
@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains the function used to set AnalogStatus configuration.
Modifications History:
  $Log: gcsConfigAnalogStatus.ctl,v $
  Revision 1.27  2011/05/02 12:00:00  Herve
  IS-510: CPC 5 and CPC 6 compatibility of the export/import of the CPC device configuration with all the front-end types
  
  Revision 1.26  2008/07/29 08:00:12  gamx
  fixes in "getArchiveSettings" function:
  archive parameters are handled correctly + the order of parameters is not fixed

  Revision 1.25  2008/06/13 12:29:47  aabadz
  Modifications asked by UNICOS for their version 4.X

  Revision 1.24  2008/01/11 14:10:55  gamx
  reviewed naming of variables

  Revision 1.23  2007/12/03 14:01:04  gamx
  fully checked version, fixed syntactic errors

  Revision 1.22  2007/12/03 13:20:37  gamx
  "dedband" is replaced with "DEADBAND"

  Revision 1.21  2007/11/29 11:32:53  gamx
  Functions to check and set WS as As have been moved from here to WS library.

  Revision 1.20  2007/11/28 16:16:21  gamx
  modifications after code review:
  changes to be consistent with naming conventions,
  introduced constants (to keep dpt name) and the set of external functions (to extract archive settings),
  exception checks are added.

  Revision 1.19  2007/11/15 08:38:52  ikoblik
  gcsConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.18  2007/11/14 16:39:31  ikoblik
  gcsConfigGenericFunctions code review comments implementation

  Revision 1.17  2007/10/30 16:20:52  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
  gcsConfigAnalogStatus_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_AnalogStatus_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_AnalogStatus_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogStatus_setCommonConfig (string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogStatus_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_AnalogStatus_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_AnalogStatus_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogStatus_checkCommonConfig (string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
*/


// Constants
const string GCS_CONFIG_ANALOGSTATUS_DPT_NAME = "AnalogStatus";

//Constants below indicate position number in the configuration line.
//Specified positions are relative, first 10 positions are reserved for standard UNICOS fields.
const unsigned GCS_CONFIG_ANALOGSTATUS_LENGTH       = 9;

const unsigned GCS_CONFIG_ANALOGSTATUS_UNIT         = 1;
const unsigned GCS_CONFIG_ANALOGSTATUS_FORMAT       = 2;
const unsigned GCS_CONFIG_ANALOGSTATUS_DEADBAND    = 3;
const unsigned GCS_CONFIG_ANALOGSTATUS_DEADBAND_TYPE = 4;
const unsigned GCS_CONFIG_ANALOGSTATUS_RANGEMAX       = 5;
const unsigned GCS_CONFIG_ANALOGSTATUS_RANGEMIN       = 6;
const unsigned GCS_CONFIG_ANALOGSTATUS_ARCHIVE_ACTIVE = 7;
const unsigned GCS_CONFIG_ANALOGSTATUS_ARCHIVE_TIME_FILTER = 8;
const unsigned GCS_CONFIG_ANALOGSTATUS_ADDRESS_AUPOSRST = 9;

const unsigned UN_CONFIG_ANALOGSTATUS_LENGTH       = 12;
const unsigned UN_CONFIG_ANALOGSTATUS_ADDITIONAL_LENGTH       = 0;
const unsigned UN_CONFIG_ANALOGSTATUS_ARCHIVE_ACTIVE = 7;

// constant declaration for AnalogStatus device used with CMW front-end
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_LENGTH 			= 17;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_UNIT 				= 1;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_FORMAT 			= 2;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_DEADBANDE 		= 3;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_DEADBANDE_TYPE = 4;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_RANGEMAX 			= 5;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_RANGEMIN 			= 6;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_ARCHIVE_ACTIVE = 7;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_ARCHIVE_TIME_FILTER = 8;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_TYPE = 9;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_TRANSFORMATION_TYPE = 10;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_GROUP = 11;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_CMWDEVICE = 12;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_CMWPROPERTY = 13;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_CMWTAG = 14;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_CMW_FILTERNAME = 15;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_CMW_FILTERVALUE = 16;
const unsigned UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_LLC = 17;

// constant declaration for AnalogStatus device used with SOFT_FE front-end
const unsigned UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH 			= 6;
const unsigned UN_CONFIG_SOFT_FE_ANALOGSTATUS_UNIT 				= 1;
const unsigned UN_CONFIG_SOFT_FE_ANALOGSTATUS_FORMAT 			= 2;
const unsigned UN_CONFIG_SOFT_FE_ANALOGSTATUS_RANGEMAX 			= 3;
const unsigned UN_CONFIG_SOFT_FE_ANALOGSTATUS_RANGEMIN 			= 4;
const unsigned UN_CONFIG_SOFT_FE_ANALOGSTATUS_ARCHIVE_ACTIVE = 5;
const unsigned UN_CONFIG_SOFT_FE_ANALOGSTATUS_ARCHIVE_TIME_FILTER = 6;

//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogStatus_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for AnalogStatus object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;

  unConfigGenericFunctions_checkAll(dsConfigs, GCS_CONFIG_ANALOGSTATUS_DPT_NAME, exceptionInfo);
  if (dynlen(exceptionInfo) <= 0)   // Good inputs
  {
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, GCS_CONFIG_ANALOGSTATUS_DPT_NAME, exceptionInfo);

    if (dynlen(exceptionInfo) <= 0)
    {
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
      if (dynlen(exceptionInfo) <= 0)
      {
      	unConfigGenericFunctions_setDeviceConfig(dsConfigs, GCS_CONFIG_ANALOGSTATUS_DPT_NAME, exceptionInfo);
      }
    }
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogStatus_setConfig
/**
Set configuration of given AnalogStatus datapoint for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogStatus_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogStatus_setConfig
/**
Set configuration of given AnalogStatus datapoint for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogStatus_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogStatus_setCommonConfig
/**
Set configuration of given AnalogStatus datapoint for both unPLc and S7_PLC front-end type

  @param[in]  sPlcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogStatus_setCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName, str, dataType;
  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogStatus_setCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + sPlcType,"");
    return;
  }

  //this check is required just to avoid any problems with PVSS software as itself
  if (dynlen(dsConfigs) >= UN_CONFIG_OBJECTGENERAL_DPNAME) sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

  //if archive name are in the config->overwrite setting from import panel.
  gcsConfigGenericFunctions_setArchives(GCS_CONFIG_ANALOGSTATUS_LENGTH, dsConfigs);

  // 2.2. Registers
  //if(sPlcType == UN_PLC_DPTYPE) str = " unConfigGenericFunctions_setAllRegisters(dsConfigs, \n";
  //if(sPlcType == S7_PLC_DPTYPE) str = " unConfigGenericFunctions_setAllRegistersS7(dsConfigs, \n";
  //str += "    makeDynString(), makeDynInt(), makeDynInt(), makeDynString(), \n";
  //str += "    makeDynInt(), makeDynString(), makeDynString(), \n";
  //str += "    makeDynInt(), \n";  //  GCS_CONFIG_ANALOGSTATUS_NORMAL_POSITION,
  //str += "    exceptionInfo); \n";
  //evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}", makeDynString(), dsConfigs);


// 2.3. Float values
  dataType = "_FLOAT";
  if(globalExists("bDoConvertion"))
    if(bDoConvertion) dataType = "_INT16";  //  "_UINT16";

  if(sPlcType == UN_PLC_DPTYPE) str = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs,  PVSS_MODBUS" + dataType + ", \n";
  else if(sPlcType == S7_PLC_DPTYPE) str = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7" + dataType + ", \n";
  else
  {
  	dynAppend(exceptionInfo, "gcsConfigAnalogStatus_setCommonConfig: plc type is unknown - " + sPlcType);
  	return;
  }

  str += "    makeDynString(\"AuPosRSt\"), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGSTATUS_ADDRESS_AUPOSRST), \n";
  str += "    makeDynString(), makeDynInt(), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGSTATUS_UNIT, GCS_CONFIG_ANALOGSTATUS_FORMAT, GCS_CONFIG_ANALOGSTATUS_DEADBAND, GCS_CONFIG_ANALOGSTATUS_DEADBAND_TYPE), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGSTATUS_RANGEMIN, GCS_CONFIG_ANALOGSTATUS_RANGEMAX), \n";
  str += "    makeDynString(\"AuPosRSt\"), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGSTATUS_ARCHIVE_ACTIVE, GCS_CONFIG_ANALOGSTATUS_ARCHIVE_TIME_FILTER), \n";
  str += "    exceptionInfo); \n";

  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);
}









//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogStatus_getArchiveSettings
/**
  Extracts archive scale, type and time sampling from the archive config.

  @param[in]  sArchiveConfig	Description of archive settings in this format: scale=<value1>|archive=<value2>|time=<value3>
  @param[out] sScale		Archive scale.
  @param[out] bNoScale		Indicates if scaling is set or not.
  @param[out] sType		Archive type.
  @param[out] sTime		Archive time sampling.
  @param[out] exceptionInfo	Exception if archive configuration is not correct.
*/
gcsConfigAnalogStatus_getArchiveSettings(string sArchiveConfig, string &sScale, bool &bNoScale, string &sType, string &sTime, dyn_string &exceptionInfo)
{
	dyn_string dsSettings, dsSettingValue;
	int iIndex;
	string sErrorMessage = "gcsConfigAnalogStatus_getArchiveSettings: archive settings are not correct.";


	dsSettings = strsplit(sArchiveConfig, "|");
	
	
	//default settings for not mandatory parameters
	bNoScale = true;
	sScale = "";
	sTime = 0;
	
	
	//the order of parameters should not be fixed
	//"archive", "scale", "time" parameters can be placed in different order
	string sParameterType;
	string sParameterValue;
	for (int i = 1; i <= dynlen(dsSettings); i++)
	{
	    if(strpos(dsSettings[i], "=") < 0)
	    {
	    	dynAppend(exceptionInfo, sErrorMessage);
	    	return;
	    }	
	
	    sParameterType = (strsplit(dsSettings[i], "="))[1];
	    sParameterValue = (strsplit(dsSettings[i], "="))[2];
	    
	    if ((sParameterType == "") || (sParameterValue == ""))
	    {
	    	dynAppend(exceptionInfo, sErrorMessage);
	    	return;	    	
	    }
	    
	    switch (sParameterType)
	    {
	    	case "scale":	sScale = sParameterValue;
	    			bNoScale = false;
	    			break;
	    			
	    	case "archive":	sType = sParameterValue;
	    			break;
	    			
	    	case "time":	sTime = sParameterValue;
	    			break;
	    }
	}
}














//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogStatus_checkConfig
/**
Check data before configuration

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogStatus_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  if(dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH)
  {
  // 1. Common
    unConfigGenericFunctions_checkParameters(dsConfigs, GCS_CONFIG_ANALOGSTATUS_DPT_NAME, exceptionInfo);
  // 2. call the device check function
    unConfigGenericFunctions_checkDeviceConfig(dsConfigs, GCS_CONFIG_ANALOGSTATUS_DPT_NAME, exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogStatus_checkConfig: " + getCatStr("gcsGeneration","BADLENGTH"),"");
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogStatus_checkConfig
/**
Check data before configuration for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_AnalogStatus_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogStatus_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogStatus_checkConfig
/**
Check data before configuration for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_AnalogStatus_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
    gcsConfigAnalogStatus_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogStatus_checkCommonConfig
/**
Check configuration of given AnalogStatus datapoint for both unPLc and S7_PLC front-end type

  @param[in]  sPlcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogStatus_checkCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string str;
  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogStatus_checkCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + sPlcType,"");
    return;
  }

  bool bHasArchive = false;
  if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_ANALOGSTATUS_LENGTH, dsConfigs, bHasArchive))
  {
    fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigAnalogStatus_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    return;
  }
  if (bHasArchive){
    gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_ANALOGSTATUS_LENGTH, dsConfigs, exceptionInfo);
    if (dynlen(exceptionInfo) != 0){
      fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigAnalogStatus_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
      return;
    }
  }

// 2. Registers
  //if(sPlcType == UN_PLC_DPTYPE)
  //  str = " unConfigGenericFunctions_checkAllRegisters(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  //if(sPlcType == S7_PLC_DPTYPE)
  //  str = " unConfigGenericFunctions_checkAllRegistersS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  //str += "    makeDynInt(), makeDynInt(), makeDynInt(), \n";
  //str += "    makeDynInt(), \n"; // GCS_CONFIG_ANALOGSTATUS_NORMAL_POSITION,
  //str += "    exceptionInfo); \n";
  //execScript("main(dyn_string dsConfigs, dyn_string &exceptionInfo) {\n" + str + "}",
  //  makeDynString(), dsConfigs, exceptionInfo); //return void

  // 3. Float block
  if(sPlcType == UN_PLC_DPTYPE) str = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  else if(sPlcType == S7_PLC_DPTYPE) str = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  else
  {
  	dynAppend(exceptionInfo, "gcsConfigAnalogStatus_checkCommonConfig: plc type is unknown - " + sPlcType);
  	return;
  }

  str += "    makeDynInt(GCS_CONFIG_ANALOGSTATUS_ADDRESS_AUPOSRST), \n";
  str += "    makeDynInt(),  \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGSTATUS_UNIT, GCS_CONFIG_ANALOGSTATUS_FORMAT, GCS_CONFIG_ANALOGSTATUS_DEADBAND, GCS_CONFIG_ANALOGSTATUS_DEADBAND_TYPE), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGSTATUS_RANGEMIN, GCS_CONFIG_ANALOGSTATUS_RANGEMAX), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGSTATUS_ARCHIVE_ACTIVE, GCS_CONFIG_ANALOGSTATUS_ARCHIVE_TIME_FILTER), \n";
  str += "    exceptionInfo); \n";
  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);
}

//------------------------------------------------------------------------------------------------------------------------

// CMW_AnalogStatus_checkConfig
/**
Purpose: check data before configuration for CMW front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CMW_AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;

	if(bOk) {
	// 3. Float block
		unConfigGenericFunctions_checkCommonFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CMW_ANALOGSTATUS_UNIT, UN_CONFIG_CMW_ANALOGSTATUS_FORMAT, UN_CONFIG_CMW_ANALOGSTATUS_DEADBANDE, 
											UN_CONFIG_CMW_ANALOGSTATUS_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_CMW_ANALOGSTATUS_RANGEMIN, UN_CONFIG_CMW_ANALOGSTATUS_RANGEMAX),
				makeDynInt(UN_CONFIG_CMW_ANALOGSTATUS_ARCHIVE_ACTIVE, UN_CONFIG_CMW_ANALOGSTATUS_ARCHIVE_TIME_FILTER),
				exceptionInfo);

	// check group
//DebugN(configLine, configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGSTATUS_TYPE], configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGSTATUS_GROUP]);
		unConfigCMW_checkType(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGSTATUS_TYPE], configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGSTATUS_GROUP], exceptionInfo);
	
	// check transformation
		unConfigCMW_checkTransformationType(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGSTATUS_TRANSFORMATION_TYPE], exceptionInfo);	
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CMW_AnalogStatus_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CMW_AnalogStatus_setConfig
/**
Purpose: set configuration of given Analog Status datapoint for CMW front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CMW_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bit32 b32;
	bool bLLC;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_CMW_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnCMW_config);
// 2.3. Float values
	unConfigGenericFunctions_setCommonFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("AuPosRSt"),
					makeDynInt(UN_CONFIG_CMW_ANALOGSTATUS_UNIT, UN_CONFIG_CMW_ANALOGSTATUS_FORMAT, 
											UN_CONFIG_CMW_ANALOGSTATUS_DEADBANDE, UN_CONFIG_CMW_ANALOGSTATUS_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_CMW_ANALOGSTATUS_RANGEMIN, UN_CONFIG_CMW_ANALOGSTATUS_RANGEMAX),
					makeDynString("AuPosRSt"),
					makeDynInt(UN_CONFIG_CMW_ANALOGSTATUS_ARCHIVE_ACTIVE, UN_CONFIG_CMW_ANALOGSTATUS_ARCHIVE_TIME_FILTER),
					exceptionInfo);
					
	if(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_LLC] == "TRUE")
		bLLC = true;
	unConfigCMW_setAddress(sDpName + ".ProcessInput.AuPosRSt", 
													dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_ANALOGSTATUS_TYPE], 
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_ANALOGSTATUS_TRANSFORMATION_TYPE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_ANALOGSTATUS_GROUP],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_CMW_FILTERNAME],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_CMW_FILTERVALUE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_CMWDEVICE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_CMWPROPERTY],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_ANALOGSTATUS_ADDRESS_CMWTAG],
													bLLC,
													exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// SOFT_FE_AnalogStatus_setConfig
/**
Purpose: set configuration of given Analog Status datapoint for SOFT_FE front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
SOFT_FE_AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bit32 b32;
	bool bLLC;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnSOFT_FE_config);
// 2.3. Float values
	unConfigGenericFunctions_setCommonFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("AuPosRSt"),
					makeDynInt(),		
					makeDynInt(UN_CONFIG_SOFT_FE_ANALOGSTATUS_RANGEMIN, UN_CONFIG_SOFT_FE_ANALOGSTATUS_RANGEMAX),
					makeDynString("AuPosRSt"),
					makeDynInt(UN_CONFIG_SOFT_FE_ANALOGSTATUS_ARCHIVE_ACTIVE, UN_CONFIG_SOFT_FE_ANALOGSTATUS_ARCHIVE_TIME_FILTER),
					exceptionInfo);
					
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessInput.AuPosRSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGSTATUS_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.AuPosRSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGSTATUS_FORMAT], exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// SOFT_FE_AnalogStatus_checkConfig
/**
Purpose: check data before configuration for SOFT_FE front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
SOFT_FE_AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_SOFT_FE_ANALOGSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;

	if(bOk) {
		unConfigGenericFunctions_checkUnitFormat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGSTATUS_UNIT], 
												 configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGSTATUS_FORMAT], exceptionInfo);
		unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGSTATUS_RANGEMIN], 
														configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGSTATUS_RANGEMAX], exceptionInfo);
		unConfigGenericFunctions_checkArchiveParameters(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGSTATUS_ARCHIVE_TIME_FILTER], 
														configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGSTATUS_ARCHIVE_ACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","SOFT_FE_AnalogStatus_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//@}
