/**@name LIBRARY: gcsConfigDigitalParameter.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 24.07.2003
@brief This library contains the function used to set DigitalParameter configuration.
Modifications History:
  $Log: gcsConfigDigitalParameter.ctl,v $
  Revision 1.30  2011/05/02 12:00:00  Herve
  IS-510: CPC 5 and CPC 6 compatibility of the export/import of the CPC device configuration with all the front-end types
  
  Revision 1.29  2010/10/08 14:59:04  amerezhi
  Setting GCS_CONFIG_DIGITALPARAMETER_DEFAULT_VALUE is fixed

  Revision 1.28  2009/04/14 13:35:04  gthomas
  commented Debug Statements

  Revision 1.27  2009/03/10 15:50:00  gthomas
  added UNICOS Changes made on 17/02/2009
  modifications in gcsConfigDigitalParameter_setCommonConfig function: added case to deal with strings "True" or "true"

  Revision 1.26  2008/07/28 12:49:35  gamx
  exception handling has been restored

  Revision 1.25  2008/06/19 12:23:17  gthomas
  Moved back variable in global lib as it used in faceplate pnl and library

  Revision 1.24  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.23  2008/06/09 14:54:26  aabadz
  Result after move some constants, only use use inside this file, from gcsObjects_declarations.

  Revision 1.22  2008/01/22 12:52:44  aabadz
  Fixed:
  - Naming
  - Call to other functions to reuse code
  - Some error check

  Revision 1.21  2007/11/15 08:38:52  ikoblik
  gcsConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.20  2007/11/14 16:39:31  ikoblik
  gcsConfigGenericFunctions code review comments implementation

  Revision 1.19  2007/10/30 16:21:05  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
  gcsConfigDigitalParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_DigitalParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_DigitalParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigDigitalParameter_setCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigDigitalParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_DigitalParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_DigitalParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigDigitalParameter_checkCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigDigitalParameter_setBoolBlock (string plcType, dyn_string dsConfigs, int dataType, dyn_string dsInputDps, dyn_int diInputAddresses, dyn_string dsOutputDps, dyn_int diOutputAddresses, dyn_int diCommonData, dyn_string &exceptionInfo)
*/

const string GCS_CONFIG_DIGITALPARAMETER_DPT_NAME = "DigitalParameter";
// Constants
const unsigned GCS_CONFIG_DIGITALPARAMETER_LENGTH = 5;

const unsigned GCS_CONFIG_DIGITALPARAMETER_ARCHIVE_ACTIVE = 1;
const unsigned GCS_CONFIG_DIGITALPARAMETER_ADDRESS_VALRQACK = 2;
const unsigned GCS_CONFIG_DIGITALPARAMETER_ADDRESS_CURVALST = 3;
const unsigned GCS_CONFIG_DIGITALPARAMETER_ADDRESS_VALRQ = 4;
const unsigned GCS_CONFIG_DIGITALPARAMETER_DEFAULT_VALUE = 5;

const unsigned UN_CONFIG_DIGITALPARAMETER_LENGTH = 5;
const unsigned UN_CONFIG_DIGITALPARAMETER_ADDITIONAL_LENGTH   = 0;
const unsigned UN_CONFIG_DIGITALPARAMETER_ARCHIVE_ACTIVE = 1;

// constant declaration for DigitalParameter device used with SOFT_FE front-end
const unsigned UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH = 3;
const unsigned UN_CONFIG_SOFT_FE_DIGITALPARAMETER_AUTONOMOUS = 1;
const unsigned UN_CONFIG_SOFT_FE_DIGITALPARAMETER_DEFAULT_VALUE = 2;
const unsigned UN_CONFIG_SOFT_FE_DIGITALPARAMETER_ARCHIVE_ACTIVE = 3;


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigDigitalParameter_setConfig
/**
Set configuration of given DigitalParameter datapoint

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigDigitalParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName, sDescription;
  bool archiveActive;
  int archiveTimeFilter, iArchiveType, iSmoothProcedure;

// 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, "DigitalParameter", exceptionInfo);
  if (dynlen(exceptionInfo) <= 0)   // Good inputs
  {
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, "DigitalParameter", exceptionInfo);
    if (dynlen(exceptionInfo) <= 0) // Creation ok or dp already exists
    {
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
      unConfigGenericFunctions_setDeviceConfig(dsConfigs, "DigitalParameter", exceptionInfo);
    }
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_DigitalParameter_setConfig
/**
Set configuration of given DigitalParameter datapoint for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_DigitalParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigDigitalParameter_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_DigitalParameter_setConfig
/**
Set configuration of given DigitalParameter datapoint for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_DigitalParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigDigitalParameter_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigDigitalParameter_setCommonConfig
/**
Set configuration of given DigitalParameter datapoint for both unPLc and S7_PLC front-end type

  @param[in]  plcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigDigitalParameter_setCommonConfig(string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName, sDescription, str, sDefaultValueConfigured;
  int dataType, archiveTimeFilter, iArchiveType, iSmoothProcedure;
  bool archiveActive, bDefaultValue;
  
  if(!gcsConfigGenericFunctions_checkPlcType(plcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigDigitalParameter_setCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + plcType,"");
    return;
  }
  sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite setting from import panel.
  gcsConfigGenericFunctions_setArchives(GCS_CONFIG_DIGITALPARAMETER_LENGTH, dsConfigs);


//  Bool values
  if(plcType == UN_PLC_DPTYPE) dataType = PVSS_MODBUS_BOOL;
  if(plcType == S7_PLC_DPTYPE) dataType = PVSS_S7_BOOL;

// Registers
  gcsConfigDigitalParameter_setBoolBlock(plcType, dsConfigs, dataType,
    makeDynString("ValRqAck", "CurValSt"),
    makeDynInt(GCS_CONFIG_DIGITALPARAMETER_ADDRESS_VALRQACK, GCS_CONFIG_DIGITALPARAMETER_ADDRESS_CURVALST),
    makeDynString("ValRq"),
    makeDynInt(GCS_CONFIG_DIGITALPARAMETER_ADDRESS_VALRQ),
    makeDynInt(), exceptionInfo);

//DebugN("sDpName="+sDpName);
//DebugN("gcsConfigDigitalParameter_setCommonConfig()");
//DebugN("UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ALLCOMMON_LENGTH",UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ALLCOMMON_LENGTH);
//DebugN("dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_DIGITALPARAMETER_DEFAULT_VALUE]=",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_DIGITALPARAMETER_DEFAULT_VALUE]);		
        
	sDefaultValueConfigured=dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_DIGITALPARAMETER_DEFAULT_VALUE];
	gcsConfigGenericFunctions_checkBool(sDefaultValueConfigured, bDefaultValue, exceptionInfo);
	dpSetWait(sDpName + ".ProcessOutput.DfltVal", bDefaultValue);

// Particular dpe : CurValSt : Archive
  unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_DIGITALPARAMETER_ARCHIVE_ACTIVE],
    archiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
  if (archiveActive)      // Archive
  {
    fwArchive_set(sDpName + ".ProcessInput.CurValSt",
            dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
            DPATTR_ARCH_PROC_SIMPLESM,
            DPATTR_COMPARE_OLD_NEW,
            0, 0, exceptionInfo);
  }
  else            // No Archive
  {
    fwArchive_delete(sDpName + ".ProcessInput.CurValSt", exceptionInfo);
  }

// Description for the Recipes
  sDescription = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DESCRIPTION];
    unConfigGenericFunctions_setDescription(sDpName + ".ProcessOutput.ValRq", sDescription, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigDigitalParameter_checkConfig
/**
Check data before configuration

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigDigitalParameter_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  if(dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH)
  {
// 1. Common
    unConfigGenericFunctions_checkParameters(dsConfigs, "DigitalParameter", exceptionInfo);
// Call the device check function
    unConfigGenericFunctions_checkDeviceConfig(dsConfigs, "DigitalParameter", exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigDigitalParameter_checkConfig: " + getCatStr("gcsGeneration","BADLENGTH"),"");
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_DigitalParameter_checkConfig
/**
Check data before configuration for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_DigitalParameter_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigDigitalParameter_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);

  unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH,
    makeDynInt(GCS_CONFIG_DIGITALPARAMETER_ADDRESS_VALRQACK, GCS_CONFIG_DIGITALPARAMETER_ADDRESS_CURVALST),
    makeDynInt(GCS_CONFIG_DIGITALPARAMETER_ADDRESS_VALRQ),
    makeDynInt(), makeDynInt(), makeDynInt(), exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_DigitalParameter_checkConfig
/**
Check data before configuration for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_DigitalParameter_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigDigitalParameter_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);

///@attention Do not call unConfigGenericFunctions_checkFloatIntBlockS7 function here! words are not treated by Unicos!!!

  unConfigGenericFunctions_checkAddressS7(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_DIGITALPARAMETER_ADDRESS_VALRQACK], UN_CONFIG_S7_PLC_DATATYPE_WORD, exceptionInfo);
  unConfigGenericFunctions_checkAddressS7(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_DIGITALPARAMETER_ADDRESS_CURVALST], UN_CONFIG_S7_PLC_DATATYPE_WORD, exceptionInfo);
  unConfigGenericFunctions_checkAddressS7(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_DIGITALPARAMETER_ADDRESS_VALRQ], UN_CONFIG_S7_PLC_DATATYPE_WORD, exceptionInfo);

  unConfigGenericFunctions_checkCommonFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH,
      makeDynInt(), makeDynInt(), makeDynInt(), exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigDigitalParameter_checkCommonConfig
/**
Check configuration of given DigitalParameter datapoint for both unPLc and S7_PLC front-end type

  @param[in]  plcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigDigitalParameter_checkCommonConfig(string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string str;
  bool bValue;
  if(!gcsConfigGenericFunctions_checkPlcType(plcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigDigitalParameter_checkCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + plcType,"");
    return;
  }
  bool bHasArchive = false;
  if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_DIGITALPARAMETER_LENGTH, dsConfigs, bHasArchive))
  {
    fwException_raise(exceptionInfo,"ERROR","with PLC=" + plcType + ". gcsConfigDigitalParameter_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    return;
  }
  if (bHasArchive){
    gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_DIGITALPARAMETER_LENGTH, dsConfigs, exceptionInfo);
    if (dynlen(exceptionInfo) != 0){
      fwException_raise(exceptionInfo,"ERROR","with PLC=" + plcType + ". gcsConfigDigitalParameter_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
      return;
    }
  }

  unConfigGenericFunctions_checkArchiveParameters("", dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_DIGITALPARAMETER_ARCHIVE_ACTIVE], exceptionInfo);
  gcsConfigGenericFunctions_checkBool(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_DIGITALPARAMETER_DEFAULT_VALUE], bValue, exceptionInfo);
}


// gcsConfigDigitalParameter_setBoolBlock
/**
Set a block of float or int dpe config
@note same as unConfigGenericFunctions_setFloatIntBlock , but we can't set format for the BOOL!!!!!

  @param[in]  plcType       PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs     Full config line
  @param[in]  dataType      Boolean type
  @param[in]  dsInputDps      Names of available Dps (in ProcessInput)
  @param[in]  diInputAddresses  Index of addresses in dsConfigs
  @param[in]  dsOutputDps     Names of available Dps (in ProcessOutput)
  @param[in]  diOutputAddresses Index of addresses in dsConfigs
  @param[in]  diCommonData    Index of common data (4 parameters : 1-unit, 2-format 3-deadband 4-deadband type)
  @param[out] exceptionInfo   All possible errors will be returned here

@attention Each given index is "relative", i.e. UN_CONFIG_ALLCOMMON_LENGTH must be added to have absolute index
*/
gcsConfigDigitalParameter_setBoolBlock(string plcType, dyn_string dsConfigs, int dataType, dyn_string dsInputDps, dyn_int diInputAddresses,
          dyn_string dsOutputDps, dyn_int diOutputAddresses, dyn_int diCommonData, dyn_string &exceptionInfo)
{
  int i, length, iPLCNumber, driverNum;
  string str, addressReference, sDpName, sPLCType;
  bool archiveActive;

  // Initialization
  sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
  driverNum = dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER];
  iPLCNumber = dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_NUMBER];
  sPLCType = dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_TYPE];

  // 1. Input common configs
  length = dynlen(dsInputDps);
  for(i = 1; i <= length; i++)
  {
    if(plcType == UN_PLC_DPTYPE)
      unConfigGenericFunctions_getUnicosAddressReference(
        makeDynString(DPATTR_ADDR_MODE_INPUT_SPONT, sPLCType, iPLCNumber, false,
        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diInputAddresses[i]],
        dsConfigs[UN_CONFIG_OBJECTGENERAL_EVENT16]),
        addressReference);
    if(plcType == S7_PLC_DPTYPE)
      unConfigGenericFunctions_getUnicosAddressReferenceS7(
        makeDynString(DPATTR_ADDR_MODE_INPUT_SPONT,
        dsConfigs[UN_CONFIG_OBJECTGENERAL_PLCNAME],
        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diInputAddresses[i]]),
        addressReference);

    if(plcType == UN_PLC_DPTYPE) str = "MODBUS";
    if(plcType == S7_PLC_DPTYPE) str = fwPeriphAddress_TYPE_S7;
    if (addressReference != "")
    {
      fwPeriphAddress_set(sDpName + ".ProcessInput." + dsInputDps[i],
        makeDynString(str, driverNum, addressReference,
            DPATTR_ADDR_MODE_INPUT_SPONT,
            dataType,
            UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
            "", "", "", "",
            UN_CONFIG_LOWLEVEL_ANALOG_PROCESS_INPUT,
            0,
            UN_CONFIG_DEFAULT_ADDRESS_STARTING_TIME,
            UN_CONFIG_DEFAULT_ADDRESS_INTERVAL_TIME,
            ""),
        exceptionInfo);
      }
  }

  // 2. Output common config
  length = dynlen(dsOutputDps);
  for(i = 1; i <= length; i++)
  {
    if(plcType == UN_PLC_DPTYPE)
      unConfigGenericFunctions_getUnicosAddressReference(
        makeDynString(DPATTR_ADDR_MODE_OUTPUT, sPLCType, iPLCNumber, false,
        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diOutputAddresses[i]],
        dsConfigs[UN_CONFIG_OBJECTGENERAL_EVENT16]),
        addressReference);
    if(plcType == S7_PLC_DPTYPE)
      unConfigGenericFunctions_getUnicosAddressReferenceS7(
        makeDynString(DPATTR_ADDR_MODE_OUTPUT,
        dsConfigs[UN_CONFIG_OBJECTGENERAL_PLCNAME],
        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diOutputAddresses[i]]),
        addressReference);

    if(plcType == UN_PLC_DPTYPE) str = "MODBUS";
    if(plcType == S7_PLC_DPTYPE) str = fwPeriphAddress_TYPE_S7;
    if (addressReference != "")
    {
      fwPeriphAddress_set(sDpName + ".ProcessOutput." + dsOutputDps[i],
        makeDynString(str, driverNum, addressReference,
            DPATTR_ADDR_MODE_OUTPUT,
            dataType,
            UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
            "", "", "", "",
            UN_CONFIG_LOWLEVEL_ANALOG_PROCESS_OUTPUT,
            0,
            UN_CONFIG_DEFAULT_ADDRESS_STARTING_TIME,
            UN_CONFIG_DEFAULT_ADDRESS_INTERVAL_TIME,
            ""),
        exceptionInfo);
    }
    else
    {
      fwException_raise(exceptionInfo,"ERROR","gcsConfigDigitalParameter_setBoolBlock: " + sDpName + ".ProcessInput." + dsOutputDps[i] + getCatStr("unGeneration","ADDRESSFAILED"),"");
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

// SOFT_FE_DigitalParameter_setConfig
/**
Purpose: set configuration of given Digital Parameter datapoint for SOFT_FE front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.1 
	. operating system: wXP.
	. distributed system: yes.
*/
SOFT_FE_DigitalParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bit32 b32;
	bool bLLC;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnSOFT_FE_config);
        //Register
//unConfigGenericFunctions_setCommonAllRegisters(dsConfigs,
//					 makeDynString("CurValSt"),
//					 makeDynString("ValRqAck"),
//					 exceptionInfo);
  
	dpSetWait(sDpName + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALPARAMETER_DEFAULT_VALUE]);
	dpSetWait(sDpName + ".ProcessInput.CurValSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALPARAMETER_DEFAULT_VALUE]);
	dpSetWait(sDpName + ".ProcessInput.ValRqAck", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALPARAMETER_DEFAULT_VALUE]);
	dpSetWait(sDpName + ".ProcessOutput.ValRq", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALPARAMETER_DEFAULT_VALUE]);

	// set the DpFunctions is the device is in autonomous mode:
	if (dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALPARAMETER_AUTONOMOUS] == 1)
		{
			// dpe config
			// DebugN("dpe to sp_function: "+sDpName + ".ProcessInput.CurValSt");
			unGenericDpFunctions_createDpeDpFunction(sDpName + ".ProcessInput.CurValSt");
			unGenericDpFunctions_createDpeDpFunction(sDpName + ".ProcessInput.ValRqAck");
			
			// dpe assign
			unGenericDpFunctions_addDpeinDpFunction(sDpName + ".ProcessInput.CurValSt", sDpName + ".ProcessOutput.ValRq:_original.._value");
			unGenericDpFunctions_addDpeinDpFunction(sDpName + ".ProcessInput.ValRqAck", sDpName + ".ProcessOutput.ValRq:_original.._value");
		} 
	
}

//------------------------------------------------------------------------------------------------------------------------

// SOFT_FE_DigitalParameter_checkConfig
/**
Purpose: check data before configuration for SOFT_FE front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.1 
	. operating system: wXP.
	. distributed system: yes.
*/
SOFT_FE_DigitalParameter_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	float fValue, fMin, fMax;
	int iAuto;
	
	if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_SOFT_FE_DIGITALPARAMETER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;

	if(bOk) {
		unConfigGenericFunctions_checkArchiveParameters(0, 
														configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALPARAMETER_ARCHIVE_ACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","SOFT_FE_DigitalParameter_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
	iAuto=configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALPARAMETER_AUTONOMOUS];
	if((iAuto !=0) && (iAuto !=1))
		fwException_raise(exceptionInfo,"ERROR","DigitalParameter_checkConfig: " + getCatStr("unGeneration","BADRANGE (Autonomous)"),"");
		
	fValue=(float)configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALPARAMETER_DEFAULT_VALUE];

}

//@}
