/**@name LIBRARY: gcsConfigGenericFunctions.ctl

@author: Ivan Koblik (IT-CO-FE)
@author: Nikolay Kulman (IT-CO-FE)
@author: Geraldine Thomas (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 14.11.2003
@brief This library contains functions called from other GCS scripts.
Modifications History:
  $Log: gcsConfigGenericFunctions.ctl,v $
  Revision 1.20  2010/09/17 16:33:00  amerezhi
  The gcsConfigGenericFunctions_checkIntVariablesPairs is fixed

  Revision 1.19  2008/09/04 14:48:54  gthomas
  -moved back to version 1.17

  Revision 1.18  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.17  2007/11/27 10:47:11  ikoblik
  Tested + bug fixes according to the November, 21st code review.

  Revision 1.16  2007/11/26 16:09:49  ikoblik
  Small bug fixes to getPlcDpName

  Revision 1.15  2007/11/22 10:04:33  gamx
  gcsConfigGenericFunctions_getPlcDpName is added

  Revision 1.14  2007/11/15 08:38:52  ikoblik
  gcsConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.13  2007/11/14 15:33:07  ikoblik
  gcsConfigGenericFunctions code review comments implementation

  Revision 1.12  2007/10/30 16:19:51  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/

/*
Internal Functions :
gcsConfigGenericFunctions_checkIntVariablesPairs(dyn_string dsConfigs, int iStringToCheck, dyn_string &exceptionInfo)
gcsConfigGenericFunctions_getPlcDpName(string sSystemName, string sPlcName, string &sPlcDpName, dyn_string &exceptionInfo)
gcsConfigGenericFunctions_setDigitalAlert(dyn_string dsConfigs, string sDpName, string sDescription, bool bOkRange,
  bool bActive, bool bSMS, string sClass, dyn_string &exceptionInfo)
gcsConfigGenericFunctions_setArchives(int iToSkip, dyn_string& dsConfigs)
bool gcsConfigGenericFunctions_checkConfigLength(int iToSkip, dyn_string& dsConfigs, bool &bHasArchive)
void gcsConfigGenericFunctions_checkArchiveConfig(int iToSkip, dyn_string& dsConfigs, dyn_string& exceptionInfo)
bool gcsConfigGenericFunctions_checkPlcType(string plcType)
void gcsConfigGenericFunctions_checkBool(string bSource, bool& bRes, dyn_string &exceptionInfo)
*/

//------------------------------------------------------------------------------------------------------------------------
// gcsConfigGenericFunctions_checkIntVariablesPairs
/**
Reads commar seperated values string from datapoint configs and verifies that each of the
values is a pair of an identifier on the left side, assignment sign, and an integer on the right side.
For example: "value1=23,value2=10"

  @param[in]  dsConfigs       Full config line
  @param[in]  iStringToCheck  Commar seperated values string's index in the datapoint configs array.
  @param[out] exceptionInfo   All possible errors will be returned here
*/
gcsConfigGenericFunctions_checkIntVariablesPairs(dyn_string dsConfigs, int iStringToCheck, dyn_string &exceptionInfo)
{
  string sLabels;
  dyn_string dsLabels, dsValue;
  int iCorrectValue;

  sLabels = dsConfigs[iStringToCheck];

  // if the string is empty, no problem. Next step should be setConfig
  // if the string is not empty, check that each element is: anInt=aString
  if ( sLabels != GCS_CONFIG_EMPTY_STRING)
  {
    // Split the configuration string in sub string
    dsLabels = strsplit(sLabels, ",");

    // Check that each of the sub string is "n=string"
    int iLenght = dynlen(dsLabels);
    for (int i = 1; i <= iLenght; i++)
    {
      dsValue = strsplit(dsLabels[i], "=");
	  if (dynlen(dsValue) != 2) {
        fwException_raise(exceptionInfo,"ERROR","gcsConfigGenericFunctions_checkIntVariablesPairs:" + getCatStr("unGeneration","BADDATA"), "");
		return;
	  } else {
        //Here dsValue FIRST element is the number.
        unConfigGenericFunctions_checkInt(strrtrim(strltrim(dsValue[1])), iCorrectValue, exceptionInfo);
	  }
    }
  }
}



//------------------------------------------------------------------------------------------------------------------------
// gcsConfigGenericFunctions_getPlcDpName
/**
Returns the full name of the Plc data point. Checks that Plc name is not empty and that Plc dp exists inside DB.

  @param[in]  sSystemName The name of the distributed system (can be empty if not set).
  @param[in]  sPlcName    The name of the plc.
  @param[out] sPlcDpName  The full name of the plc dp.
  @param[out] exceptionInfo The list of exceptions.
*/

gcsConfigGenericFunctions_getPlcDpName(string sSystemName, string sPlcName, string &sPlcDpName, dyn_string &exceptionInfo)
{
  sPlcDpName = "";

  if ("" == sPlcName){
    dynAppend(exceptionInfo, "Plc name is not specified");
    return;
  }

  string sTemp = sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName;
  if (!dpExists(sTemp)) {
    dynAppend(exceptionInfo, "Dp for specified Plc does not exist. Plc name: " + sPlcName + ", system name: " + sSystemName);
    return;
  }
  sPlcDpName = sTemp;
}




//----------------------------------------------------------------------------------------------------------------------------------
// gcsConfigGenericFunctions_setDigitalAlert
/**
Set digital alert of Analog Alarm and Digital Alarm

  @param[in]  dsConfigs   Full config line
  @param[in]  sDpName     Datapoint name
  @param[in]  description   Datapoint description
  @param[in]  bOkRange    False if ok range = false, true if Ok range = true
  @param[in]  active      True if alert must be set to active
  @param[in]  bSMS      True if we need to set SMS configuration for this alarm
  @param[in]  class     True if alert must be set to active
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigGenericFunctions_setDigitalAlert(dyn_string dsConfigs, string sDpName, string sDescription, bool bOkRange,
  bool bActive, bool bSMS, string sClass, dyn_string &exceptionInfo)
{
  dyn_string dsAlertTexts, dsTemp;
  bool bError = false;

  if(!dpExists(sDpName))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigGenericFunctions_setDigitalAlert: " + sDpName + getCatStr("unGeneration","DPDOESNOTEXIST"),"");
  }
  dsTemp = makeDynString(
    dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DOMAIN],
    dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_NATURE],
    dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS],
    sDescription,
    dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DESCRIPTION],
    dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DEFAULTPANEL]
  );

  unConfigGenericFunctions_setDigitalAlert(sDpName, dsTemp, bOkRange, false /*not active*/, bSMS, exceptionInfo);

  if (dynlen(exceptionInfo) != 0)
    return;

  bError |= 0 < dpSet(sDpName + ":_alert_hdl.._text0", bOkRange ? sDescription : "");
  bError |= 0 < dpSet(sDpName + ":_alert_hdl.._text1", bOkRange ? "" : sDescription);
  bError |= 0 < dpSet(sDpName + ":_alert_hdl.._class", sClass);
  if (bError)
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigGenericFunctions_setDigitalAlert: " + sDpName + getCatStr("unGeneration","ERRORSETTINGALERTCONFIG"),"");
    return;
  }
  if(bActive)
    fwAlertConfig_activate(sDpName, exceptionInfo);
}

//----------------------------------------------------------------------------------------------------------------------------------
// gcsConfigGenericFunctions_setArchives
/**
Gets archiving configs from object and puts them to the common archiving configs.

  @param[in]  iToSkip   Num elements in dsConfigs to skip before common part. (ex. GCS_CONFIG_ANALOGPAR_LENGTH)
  @param[out] dsConfigs Configs
*/
gcsConfigGenericFunctions_setArchives(int iToSkip, dyn_string& dsConfigs)
{
  int iIdx = 0;
  // if archive name are in the config->overwrite setting from import panel.
  if(dynlen(dsConfigs) == (iToSkip + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH))
  {
    iIdx = iToSkip + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL;
    if(dsConfigs[iIdx] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[iIdx];

    iIdx = iToSkip + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG;
    if(dsConfigs[iIdx] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[iIdx];

    iIdx = iToSkip + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT;
    if(dsConfigs[iIdx] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[iIdx];
  }
}


//----------------------------------------------------------------------------------------------------------------------------------
// gcsConfigGenericFunctions_checkConfigLength
/**
Check if the configs lenght is correct. Also reports if config has archiving in it.

  @return   Result of checking, true if OK, else false.
  @param[in]  iToSkip   Num elements in dsConfigs to skip before common part. (ex. GCS_CONFIG_ANALOGPAR_LENGTH)
  @param[out] dsConfigs Configs
  @param[out] bHasArchive returns true if passed configs have settings for archiving
*/
bool gcsConfigGenericFunctions_checkConfigLength(int iToSkip, dyn_string& dsConfigs, bool &bHasArchive)
{
  if (dynlen(dsConfigs) == (iToSkip + UN_CONFIG_COMMON_LENGTH)){
    bHasArchive = false;
    return true;
  }
  else if (dynlen(dsConfigs) == (iToSkip + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH))
  {
    bHasArchive = true;
    return true;
  }
  return false;
}


//----------------------------------------------------------------------------------------------------------------------------------
// gcsConfigGenericFunctions_checkArchiveConfig
/**
Check if the configs lenght is correct for archives. Check if archive configs are correct

  @return   Result of checking, true if OK, else false.
  @param[in]  iToSkip   Num elements in dsConfigs to skip before common part. (ex. GCS_CONFIG_ANALOGPAR_LENGTH)
  @param[out] dsConfigs Configs
  @param[out] exceptionInfo All possible errors will be returned here
*/
void gcsConfigGenericFunctions_checkArchiveConfig(int iToSkip, dyn_string& dsConfigs, dyn_string& exceptionInfo)
{
  if (dynlen(dsConfigs) == (iToSkip + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH))
  {
    int iCommon = iToSkip + UN_CONFIG_COMMON_LENGTH;
    unConfigGenericFunctions_checkArchive(dsConfigs[iCommon + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
      dsConfigs[iCommon + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
      dsConfigs[iCommon + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
      exceptionInfo);
  }
  else 
    fwException_raise(exceptionInfo,"ERROR","gcsConfigGenericFunctions_checkArchiveConfig:" + getCatStr("unGeneration","BADLENGTH"), "");
}


//----------------------------------------------------------------------------------------------------------------------------------
// gcsConfigGenericFunctions_checkPlcType
/**
Check if we have correct PLC type. (must be UN_PLC_DPTYPE or S7_PLC_DPTYPE)

  @return   Result of checking, true if OK, sles false.
  @param[in]  plcType   PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE at the moment
*/
bool gcsConfigGenericFunctions_checkPlcType(string plcType)
{
  return (plcType == UN_PLC_DPTYPE || plcType == S7_PLC_DPTYPE);
}


//----------------------------------------------------------------------------------------------------------------------------------
// gcsConfigGenericFunctions_checkBool
/**
Check if we have correct boolean value in input string and return it in bRes

  @param[in]  bSource       Input string. To check inside if it is correct boolean value.
  @param[out] bRes          Flag to indicate if we have correct value it will be returned here.
  @param[out] exceptionInfo All possible errors will be returned here
*/
void gcsConfigGenericFunctions_checkBool(string bSource, bool& bRes, dyn_string &exceptionInfo)
{
  bSource = strtolower(strrtrim(strltrim(bSource)));
  if(bSource == "true" || bSource == "1" || bSource == "y" || bSource == "yes")
  {
    bRes = true;
    return;
  }
  else if(bSource == "false" || bSource == "0" || bSource == "n" || bSource == "no")
  {
    bRes = false;
    return;
  }
  fwException_raise(exceptionInfo,"ERROR","gcsConfigGenericFunctions_checkBool:" + getCatStr("unGeneration","BADDATA") + bSource, "");
}
//@}