/**@name LIBRARY: gcsConfigMassFlowController.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 24.07.2003
@brief This library contains the function used to set MassFlowController configuration.
Modifications History:
	$Log: gcsConfigMassFlowController.ctl,v $
	Revision 1.20  2010/09/17 11:45:00  amerezhi
	Checks fixed
	Changed "fwException_raise(makeDynString(), ..." to "fwException_raise(exceptionInfo, ..."

	Revision 1.18  2008/09/04 14:35:34  gthomas
	-Moved back generic constant to gcsObjects_declarations lib
	-fixed exception handling
	
	Revision 1.17  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.16  2008/06/09 14:54:26  aabadz
	Result after move some constants, only use use inside this file, from gcsObjects_declarations.

	Revision 1.15  2007/12/11 09:10:11  aabadz
	Used unicos function (unConfigGenericFunctions_checkInt) instead sscanf

	Revision 1.14  2007/12/10 15:47:17  aabadz
	Corrections following the document from code review. Version after testing

	Revision 1.13  2007/12/09 14:02:57  gamx
	constant to keep dpt name for MFC is introduced

	Revision 1.12  2007/12/05 10:49:55  aabadz
	Corrections following the document from code review. Version before testing

	Revision 1.9  2007/10/30 16:21:13  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
	gcsConfigMassFlowController_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
	_UnPlc_MassFlowController_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
	S7_PLC_MassFlowController_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
	gcsConfigMassFlowController_setCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
	gcsConfigMassFlowController_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
	_UnPlc_MassFlowController_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
	S7_PLC_MassFlowController_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
	gcsConfigMassFlowController_checkCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
*/

const string GCS_CONFIG_MFC_DPT_NAME = "MassFlowController";

const unsigned GCS_CONFIG_MFC_LENGTH 							= 39;
// Flow and SetPoint Parameters
const unsigned GCS_CONFIG_MFC_FLOW_SETPOINT_UNIT 				= 1;
const unsigned GCS_CONFIG_MFC_FLOW_SETPOINT_FORMAT 				= 2;
const unsigned GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND 			= 3;
const unsigned GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND_TYPE		= 4;
const unsigned GCS_CONFIG_MFC_FLOW_SETPOINT_RANGEMAX 			= 5;
const unsigned GCS_CONFIG_MFC_FLOW_SETPOINT_RANGEMIN 			= 6;
const unsigned GCS_CONFIG_MFC_FLOW_SETPOINT_ARCHIVE_ACTIVE		= 7;
const unsigned GCS_CONFIG_MFC_FLOW_SETPOINT_ARCHIVE_TIME_FILTER	= 8;
// TOTALIZER Parameters
const unsigned GCS_CONFIG_MFC_TOTALIZER_UNIT 					= 9;
const unsigned GCS_CONFIG_MFC_TOTALIZER_FORMAT 					= 10;
const unsigned GCS_CONFIG_MFC_TOTALIZER_DEADBAND	 			= 11;
const unsigned GCS_CONFIG_MFC_TOTALIZER_DEADBAND_TYPE			= 12;
const unsigned GCS_CONFIG_MFC_TOTALIZER_ARCHIVE_ACTIVE			= 13;
const unsigned GCS_CONFIG_MFC_TOTALIZER_ARCHIVE_TIME_FILTER		= 14;
// Normal position
const unsigned GCS_CONFIG_MFC_NORMAL_POSITION 					= 15;
//Addresses Process Input
const unsigned GCS_CONFIG_MFC_ADDRESS_STSREG01 					= 16;
const unsigned GCS_CONFIG_MFC_ADDRESS_EVSTSREG01				= 17;
const unsigned GCS_CONFIG_MFC_ADDRESS_APSPST 					= 18;
const unsigned GCS_CONFIG_MFC_ADDRESS_AUSPRST 					= 19;
const unsigned GCS_CONFIG_MFC_ADDRESS_SPOV	 					= 20;
const unsigned GCS_CONFIG_MFC_ADDRESS_DRMOST 					= 21;
const unsigned GCS_CONFIG_MFC_ADDRESS_AUDRMRST	 				= 22;
const unsigned GCS_CONFIG_MFC_ADDRESS_DRMOOV 					= 23;
const unsigned GCS_CONFIG_MFC_ADDRESS_CCST 						= 24;
const unsigned GCS_CONFIG_MFC_ADDRESS_AUCCRST 					= 25;
const unsigned GCS_CONFIG_MFC_ADDRESS_CCOV						= 26;
const unsigned GCS_CONFIG_MFC_ADDRESS_TOTMOST					= 27;
const unsigned GCS_CONFIG_MFC_ADDRESS_AUTOTMORST				= 28;
const unsigned GCS_CONFIG_MFC_ADDRESS_TOTMOOV					= 29;
const unsigned GCS_CONFIG_MFC_ADDRESS_MAXFLST 					= 30;
const unsigned GCS_CONFIG_MFC_ADDRESS_FLST 						= 31;
const unsigned GCS_CONFIG_MFC_ADDRESS_TOTST						= 32;
const unsigned GCS_CONFIG_MFC_ADDRESS_MANREG01 					= 33;
const unsigned GCS_CONFIG_MFC_ADDRESS_MSPR 						= 34;
const unsigned GCS_CONFIG_MFC_ADDRESS_MDRMOR 					= 35;
const unsigned GCS_CONFIG_MFC_ADDRESS_MCCR 						= 36;
const unsigned GCS_CONFIG_MFC_ADDRESS_MTOTMOR 					= 37;
const unsigned GCS_CONFIG_MFC_CALIBRATION_CURVE 				= 38;
const unsigned GCS_CONFIG_MFC_MASKEVENT 						= 39;


//------------------------------------------------------------------------------------------------------------------------

// gcsConfigMassFlowController_setConfig
/**
Create MassFlowController datapoint, check it and set configuration

	@param[in]	dsConfigs		Configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
	@param[out]	exceptionInfo	All possible errors will be returned here
*/
gcsConfigMassFlowController_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;

// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, GCS_CONFIG_MFC_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		if (dynlen(dsConfigs)>=UN_CONFIG_OBJECTGENERAL_DPNAME)
			sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Bad configuration array size","");
			return;
		}
		unConfigGenericFunctions_createDp(sDpName, GCS_CONFIG_MFC_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			if (dynlen(exceptionInfo)!=0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Setting common","");
				return;
			}

			// call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, GCS_CONFIG_MFC_DPT_NAME, exceptionInfo);
			if (dynlen(exceptionInfo)!=0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Setting device configuration","");
				return;
			}
		}
	}
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_MassFlowController_setConfig
/**
Set configuration of given MassFlowController datapoint for _UnPlc front-end type

	@param[in]	dsConfigs		Configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
	@param[out]	exceptionInfo	All possible errors will be returned here
*/
_UnPlc_MassFlowController_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	gcsConfigMassFlowController_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_MassFlowController_setConfig
/**
Set configuration of given MassFlowController datapoint for S7_PLC front-end type

	@param[in]	dsConfigs		Configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
	@param[out]	exceptionInfo	All possible errors will be returned here
*/
S7_PLC_MassFlowController_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	gcsConfigMassFlowController_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigMassFlowController_setCommonConfig
/**
Set configuration of given MassFlowController datapoint for both unPLc and S7_PLC front-end type

	@param[in]	sPlcType			PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
	@param[in]	dsConfigs		Configs (see dyn_string indexes in constants defined)
	@param[out]	exceptionInfo	All possible errors will be returned here
*/
gcsConfigMassFlowController_setCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string stringValue, sDeviceName, sDpName, sMask, str;
	int iMask = 0;
	bool bCheckType;

	bCheckType = gcsConfigGenericFunctions_checkPlcType(sPlcType);
	if(!bCheckType)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknown PLC type - " + sPlcType,"");
		return;
	}
	if (dynlen(dsConfigs)>=UN_CONFIG_OBJECTGENERAL_DPNAME)
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	else
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Bad configuration array size - ","");
		return;
	}

// if archive name are in the config->overwrite setting from import panel.
	gcsConfigGenericFunctions_setArchives(GCS_CONFIG_MFC_LENGTH, dsConfigs);

	if (dynlen(dsConfigs)>= GCS_CONFIG_MFC_MASKEVENT + UN_CONFIG_ALLCOMMON_LENGTH)
		sMask = dsConfigs[GCS_CONFIG_MFC_MASKEVENT + UN_CONFIG_ALLCOMMON_LENGTH];
	else
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Bad configuration array size - ","");
		return;
	}

	//Call this function instead of sscanf()
	unConfigGenericFunctions_checkInt(sMask, iMask, exceptionInfo);
	if (dynlen(exceptionInfo)>0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Bad int " + sPlcType,"");
		return;
	}

// 2.2. Registers
	if(sPlcType == UN_PLC_DPTYPE)
		str = "	unConfigGenericFunctions_setAllRegisters(dsConfigs, \n";
	else if(sPlcType == S7_PLC_DPTYPE)
		str = "	unConfigGenericFunctions_setAllRegistersS7(dsConfigs, \n";
	     else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for regiters- " + sPlcType,"");
			return;
		}

	str += "	makeDynString(\"StsReg01\"), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_STSREG01), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_EVSTSREG01), \n";
	str += "	makeDynString(\"ManReg01\"), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_MANREG01), \n";
	str += "	makeDynString(\"StartISt;StopISt\"), \n";
	str += "	makeDynString(UN_STSREG01_STARTIST + \";\" + UN_STSREG01_STOPIST), \n";
	str += "	GCS_CONFIG_MFC_NORMAL_POSITION, exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Executing the script (for registers) - " + sPlcType,"");
		return;
	}

// 2.3. Float values
	if(sPlcType == UN_PLC_DPTYPE)
		str = "	unConfigGenericFunctions_setFloatIntBlock(dsConfigs, 	PVSS_MODBUS_FLOAT, \n";
	else if(sPlcType == S7_PLC_DPTYPE)
		str = "	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs, 	PVSS_S7_FLOAT, \n";
	     else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for floats- " + sPlcType,"");
			return;
		}

	str += "	makeDynString(\"ApSpSt\", \"AuSpRSt\", \"SpOV\", \"FlSt\"), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_APSPST, GCS_CONFIG_MFC_ADDRESS_AUSPRST, GCS_CONFIG_MFC_ADDRESS_SPOV, GCS_CONFIG_MFC_ADDRESS_FLST), \n";
	str += "	makeDynString(\"MSpR\"), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_MSPR), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_UNIT, GCS_CONFIG_MFC_FLOW_SETPOINT_FORMAT, GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND, GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND_TYPE), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_RANGEMIN, GCS_CONFIG_MFC_FLOW_SETPOINT_RANGEMAX), \n";
	str += "	makeDynString(\"SpOV\", \"FlSt\"), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_ARCHIVE_ACTIVE, GCS_CONFIG_MFC_FLOW_SETPOINT_ARCHIVE_TIME_FILTER), \n";
	str += "	exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Executing the script (for floats) - " + sPlcType,"");
		return;
	}


// 2.4. Float values
	if(sPlcType == UN_PLC_DPTYPE)
		str = "	unConfigGenericFunctions_setFloatIntBlock(dsConfigs, 	PVSS_MODBUS_FLOAT, \n";
	else if(sPlcType == S7_PLC_DPTYPE)
		str = "	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs, 	PVSS_S7_FLOAT, \n";
	     else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for floats- " + sPlcType,"");
			return;
		}

	str += "	makeDynString(\"MaxFlSt\"), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_MAXFLST), \n";
	str += "	makeDynString(), makeDynInt(), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_UNIT, GCS_CONFIG_MFC_FLOW_SETPOINT_FORMAT, GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND, GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND_TYPE), \n";
	str += "	makeDynInt(), makeDynString(\"MaxFlSt\"), makeDynInt(), exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Executing the script (for floats) - " + sPlcType,"");
		return;
	}

	if(sPlcType == UN_PLC_DPTYPE)
		str = "	unConfigGenericFunctions_setFloatIntBlock(dsConfigs, 	PVSS_MODBUS_FLOAT, \n";
	else if(sPlcType == S7_PLC_DPTYPE)
		str = "	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs, 	PVSS_S7_FLOAT, \n";
	     else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for floats- " + sPlcType,"");
			return;
		}

	str += "	makeDynString(\"TotSt\"), makeDynInt(GCS_CONFIG_MFC_ADDRESS_TOTST), \n";
	str += "	makeDynString(), makeDynInt(), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_TOTALIZER_UNIT, GCS_CONFIG_MFC_TOTALIZER_FORMAT, GCS_CONFIG_MFC_TOTALIZER_DEADBAND, GCS_CONFIG_MFC_TOTALIZER_DEADBAND_TYPE), \n";
	str += "	makeDynInt(), makeDynString(\"TotSt\"), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_TOTALIZER_ARCHIVE_ACTIVE, GCS_CONFIG_MFC_TOTALIZER_ARCHIVE_TIME_FILTER), \n";
	str += "	exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Executing the script (for floats) - " + sPlcType,"");
		return;
	}

	if(sPlcType == UN_PLC_DPTYPE)
		str = "	unConfigGenericFunctions_setFloatIntBlock(dsConfigs, 	PVSS_MODBUS_UINT16, \n";
	else if(sPlcType == S7_PLC_DPTYPE)
		str = "	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs, 	PVSS_S7_UINT16, \n";
	     else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for floats- " + sPlcType,"");
			return;
		}

	str += "	makeDynString(\"DrMoSt\",\"AuDrMRSt\",\"DrMoOV\", \"TotMoSt\",\"AuTotMoRSt\",\"TotMoOV\", \"CcSt\",\"AuCcRSt\",\"CcOV\"), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_DRMOST, GCS_CONFIG_MFC_ADDRESS_AUDRMRST, \n";
	str += "		   GCS_CONFIG_MFC_ADDRESS_DRMOOV, GCS_CONFIG_MFC_ADDRESS_TOTMOST, \n";
	str += "		   GCS_CONFIG_MFC_ADDRESS_AUTOTMORST, GCS_CONFIG_MFC_ADDRESS_TOTMOOV, \n";
	str += "		   GCS_CONFIG_MFC_ADDRESS_CCST, GCS_CONFIG_MFC_ADDRESS_AUCCRST, \n";
	str += "		   GCS_CONFIG_MFC_ADDRESS_CCOV), \n";
	str += "	makeDynString(\"MDrMoR\", \"MTotMoR\", \"MCcR\"), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_MDRMOR, GCS_CONFIG_MFC_ADDRESS_MTOTMOR, GCS_CONFIG_MFC_ADDRESS_MCCR), \n";
	str += "	makeDynInt(), makeDynInt(), makeDynString(), makeDynInt(), exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Executing the script (for floats) - " + sPlcType,"");
		return;
	}

	// 2.5. Get the description and set it in the common of CcSt
	if (dynlen(dsConfigs)>=UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_MFC_CALIBRATION_CURVE)
		stringValue = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_MFC_CALIBRATION_CURVE];
	else
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Bad configuration array size","");
		return;
	}

	sDeviceName = unGenericDpFunctions_getDpName(sDpName);
	if ( stringValue == "" || stringValue == GCS_CONFIG_EMPTY_STRING)
	{
		stringValue = GCS_CONFIG_DEFAULT_CALIBRATION_CURVE;
	}

	unConfigGenericFunctions_setDescription(sDeviceName + ".ProcessInput.CcSt", stringValue, exceptionInfo);
	if (dynlen(exceptionInfo)!=0)
	{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Set new description (CcSt)- ","");
			return;
	}

	unConfigGenericFunctions_setDescription(sDeviceName + ".ProcessInput.DrMoSt",
		"1=Close,2=Open,3=Set to min,4=Set to max,5=Regulated", exceptionInfo);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Set new description (DrMoSt)- ","");
		return;
	}

	unConfigGenericFunctions_setDescription(sDeviceName + ".ProcessInput.TotMoSt",
		"1=Stop,2=Start, 3=Reset", exceptionInfo);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Set new description (TotMoSt)- ","");
		return;
	}

	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask,
		makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_setConfig: Set new mask event - ","");
		return;
	}
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigMassFlowController_checkConfig
/**
Check data before configuration

	@param[in]	dsConfigs		Configs (see dyn_string indexes in constants defined)
	@param[out]	exceptionInfo	All possible errors will be returned here
*/
gcsConfigMassFlowController_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	unsigned uUnsignedStatus;
	if (dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH) {
		// 1. Common
		unConfigGenericFunctions_checkParameters(dsConfigs, "MassFlowController", exceptionInfo);
		if (dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_MFC_MASKEVENT) {
			unConfigGenericFunctions_checkUnsigned(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_MFC_MASKEVENT], uUnsignedStatus, exceptionInfo);
		}
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(dsConfigs, "MassFlowController", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_MassFlowController_checkConfig
/**
Check data before configuration for _UnPlc front-end type

	@param[in]	dsConfigs		Configs (see dyn_string indexes in constants defined)
	@param[out]	exceptionInfo	All possible errors will be returned here
*/
_UnPlc_MassFlowController_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	gcsConfigMassFlowController_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_MassFlowController_checkConfig
/**
Check data before configuration for S7_PLC front-end type

	@param[in]	dsConfigs		Configs (see dyn_string indexes in constants defined)
	@param[out]	exceptionInfo	All possible errors will be returned here
*/
S7_PLC_MassFlowController_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
		gcsConfigMassFlowController_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigMassFlowController_checkCommonConfig
/**
Check configuration of given MassFlowController datapoint for both unPLc and S7_PLC front-end type

	@param[in]	sPlcType			PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
	@param[in]	dsConfigs		Configs (see dyn_string indexes in constants defined)
	@param[out]	exceptionInfo	All possible errors will be returned here
*/
gcsConfigMassFlowController_checkCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string str;
	bool bCheckType;

	bCheckType = gcsConfigGenericFunctions_checkPlcType(sPlcType);
	if(!bCheckType)
	{
		fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType +  "gcsConfigMassFlowController_checkCommonConfig: Unknowm PLC type - " + sPlcType,"");
		return;
	}

  	bool bHasArchive = false;
  	if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_MFC_LENGTH, dsConfigs, bHasArchive))
  	{
  	  fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigMassFlowController_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  	  return;
  	}
  	if (bHasArchive){
  	  gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_MFC_LENGTH, dsConfigs, exceptionInfo);
  	  if (dynlen(exceptionInfo) != 0){
  	    fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigMassFlowController_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  	    return;
  	  }
  	}

	// 2. Registers
	if(sPlcType == UN_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkAllRegisters(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	else if(sPlcType == S7_PLC_DPTYPE)
			str = " unConfigGenericFunctions_checkAllRegistersS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	     else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for registers- " + sPlcType,"");
			return;
		}

	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_STSREG01), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_EVSTSREG01), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_MANREG01), \n";
	str += "	GCS_CONFIG_MFC_NORMAL_POSITION,	exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Executing the script (for registers) - " + sPlcType,"");
		return;
	}

	// 3. Float block
	if(sPlcType == UN_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	else if(sPlcType == S7_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	     else
	 	{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for floats- " + sPlcType,"");
			return;
		}

	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_DRMOST, GCS_CONFIG_MFC_ADDRESS_AUDRMRST,  \n";
	str += "		   GCS_CONFIG_MFC_ADDRESS_DRMOOV, GCS_CONFIG_MFC_ADDRESS_TOTMOST,  \n";
	str += "		   GCS_CONFIG_MFC_ADDRESS_AUTOTMORST, GCS_CONFIG_MFC_ADDRESS_TOTMOOV, \n";
	str += "		   GCS_CONFIG_MFC_ADDRESS_CCST, GCS_CONFIG_MFC_ADDRESS_AUCCRST,  \n";
	str += "		   GCS_CONFIG_MFC_ADDRESS_CCOV), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_MSPR, GCS_CONFIG_MFC_ADDRESS_MDRMOR, \n";
	str += "		   GCS_CONFIG_MFC_ADDRESS_MTOTMOR, GCS_CONFIG_MFC_ADDRESS_MCCR), \n";
	str += "	makeDynInt(), makeDynInt(), makeDynInt(), exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Executing the script (for float) - " + sPlcType,"");
		return;
	}

	if(sPlcType == UN_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	else if(sPlcType == S7_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	     else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for floats- " + sPlcType,"");
			return;
		}

	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_APSPST, GCS_CONFIG_MFC_ADDRESS_AUSPRST, GCS_CONFIG_MFC_ADDRESS_SPOV), \n";
	str += "	makeDynInt(),  \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_UNIT, GCS_CONFIG_MFC_FLOW_SETPOINT_FORMAT, GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND, GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND_TYPE), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_RANGEMIN, GCS_CONFIG_MFC_FLOW_SETPOINT_RANGEMAX), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_ARCHIVE_ACTIVE, GCS_CONFIG_MFC_FLOW_SETPOINT_ARCHIVE_TIME_FILTER), \n";
	str += "	exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Executing the script (for float) - " + sPlcType,"");
		return;
	}

	if(sPlcType == UN_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	else  if(sPlcType == S7_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	      else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for floats- " + sPlcType,"");
			return;
		}

	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_FLST), makeDynInt(),  \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_UNIT, GCS_CONFIG_MFC_FLOW_SETPOINT_FORMAT, GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND, GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND_TYPE), \n";
	str += "	makeDynInt(), makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_ARCHIVE_ACTIVE, GCS_CONFIG_MFC_FLOW_SETPOINT_ARCHIVE_TIME_FILTER), \n";
	str += "	exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Executing the script (for float) - " + sPlcType,"");
		return;
	}

	if(sPlcType == UN_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	else if(sPlcType == S7_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	     else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for floats- " + sPlcType,"");
			return;
		}

	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_MAXFLST), \n";
	str += "	makeDynInt(),  \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_UNIT, GCS_CONFIG_MFC_FLOW_SETPOINT_FORMAT, GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND, GCS_CONFIG_MFC_FLOW_SETPOINT_DEADBAND_TYPE), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_RANGEMIN, GCS_CONFIG_MFC_FLOW_SETPOINT_RANGEMAX), \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_FLOW_SETPOINT_ARCHIVE_ACTIVE, GCS_CONFIG_MFC_FLOW_SETPOINT_ARCHIVE_TIME_FILTER), \n";
	str += "	exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Executing the script (for float) - " + sPlcType,"");
		return;
	}

	if(sPlcType == UN_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	else if(sPlcType == S7_PLC_DPTYPE)
		str = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
	     else
		{
			fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Unknow PLC TYPE for floats- " + sPlcType,"");
			return;
		}

	str += "	makeDynInt(GCS_CONFIG_MFC_ADDRESS_TOTST), makeDynInt(),  \n";
	str += "	makeDynInt(GCS_CONFIG_MFC_TOTALIZER_UNIT, GCS_CONFIG_MFC_TOTALIZER_FORMAT, GCS_CONFIG_MFC_TOTALIZER_DEADBAND, GCS_CONFIG_MFC_TOTALIZER_DEADBAND_TYPE), \n";
	str += "	makeDynInt(), makeDynInt(GCS_CONFIG_MFC_TOTALIZER_ARCHIVE_ACTIVE, GCS_CONFIG_MFC_TOTALIZER_ARCHIVE_TIME_FILTER), \n";
	str += "	exceptionInfo); \n";
	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
		makeDynString(), dsConfigs);
	if (dynlen(exceptionInfo)!=0)
	{
		fwException_raise(exceptionInfo,"ERROR","gcsConfigMassFlowController_checkCommonConfig: Executing the script (for float) - " + sPlcType,"");
		return;
	}

	// 8. Parameter to display int or string values
	gcsConfigGenericFunctions_checkIntVariablesPairs(dsConfigs, UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_MFC_CALIBRATION_CURVE, exceptionInfo);

}


//@}
