/**@name LIBRARY: gcsConfigPump.ctl

@author: Artem Burmyakov (IT-CO-FE)
@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains the function used to set Unicos Pump configuration.
Modifications History:
  $Log: gcsConfigPump.ctl,v $
  Revision 1.20  2010/09/17 11:15:00  amerezhi
  The check for GCS_CONFIG_PUMP_MASKEVENT is added
  MaskEvent set is fixed

  Revision 1.19  2008/09/04 14:43:19  gthomas
  -moved back to version 1.16

  Revision 1.18  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.17  2008/06/09 14:54:26  aabadz
  Result after move some constants, only use use inside this file, from gcsObjects_declarations.

  Revision 1.16  2008/01/09 13:31:56  gamx
  changed the way of exception report

  Revision 1.15  2007/12/03 15:54:22  gamx
  instead of "sscanf" standard unicos functions are used, but there is a problem with event mask position. it is under investigation.

  Revision 1.14  2007/11/21 15:24:44  gamx
  if/else if/else instructions added

  Revision 1.13  2007/11/21 14:37:54  gamx
  remove unused variables

  Revision 1.12  2007/11/20 11:20:44  gamx
  if/if constructions are replaced with if/else where possible.

  Revision 1.11  2007/11/20 10:53:27  gamx
  sintactic bugs are fixed

  Revision 1.10  2007/11/16 15:03:59  gamx
  reviewed version (but "scanf" is not replaced with unicos function yet)

  Revision 1.9  2007/11/15 08:38:52  ikoblik
  gcsConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.8  2007/11/14 16:39:31  ikoblik
  gcsConfigGenericFunctions code review comments implementation

  Revision 1.7  2007/10/30 16:21:19  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
External Functions :
  gcsConfigPump_setConfig (dyn_string dsConfigs, dyn_string &dsExceptionInfo)
  _UnPlc_Pump_setConfig (dyn_string dsConfigs, dyn_string &dsExceptionInfo)
  S7_PLC_Pump_setConfig (dyn_string dsConfigs, dyn_string &dsExceptionInfo)
  gcsConfigPump_setCommonConfig (string sPlcType, dyn_string dsConfigs, dyn_string &dsExceptionInfo)
  gcsConfigPump_checkConfig (dyn_string dsConfigs, dyn_string &dsExceptionInfo)
  _UnPlc_Pump_checkConfig (dyn_string dsConfigs, dyn_string &dsExceptionInfo)
  S7_PLC_Pump_checkConfig (dyn_string dsConfigs, dyn_string &dsExceptionInfo)
  gcsConfigPump_checkCommonConfig (string sPlcType, dyn_string dsConfigs, dyn_string &dsExceptionInfo)
*/




const string GCS_CONFIG_PUMP_DPT_NAME = "Pump";


//Constants indicate positions in the configuration line (for DB file)
//Specified position numbers are relative. First 10 positions are reserved for general Unicos fields.
const unsigned GCS_CONFIG_PUMP_LENGTH            	= 28;

const unsigned GCS_CONFIG_PUMP_SETPOINT_UNIT     	= 1;
const unsigned GCS_CONFIG_PUMP_SETPOINT_FORMAT    	= 2;
const unsigned GCS_CONFIG_PUMP_SETPOINT_DEADBAND  	= 3;
const unsigned GCS_CONFIG_PUMP_SETPOINT_DEADBAND_TYPE   = 4;
const unsigned GCS_CONFIG_PUMP_SETPOINT_RANGEMAX        = 5;
const unsigned GCS_CONFIG_PUMP_SETPOINT_RANGEMIN        = 6;
const unsigned GCS_CONFIG_PUMP_SETPOINT_ARCHIVE_ACTIVE  = 7;
const unsigned GCS_CONFIG_PUMP_SETPOINT_ARCHIVE_TIME_FILTER  = 8;
const unsigned GCS_CONFIG_PUMP_LOAD_UNIT          	= 9;
const unsigned GCS_CONFIG_PUMP_LOAD_FORMAT        	= 10;
const unsigned GCS_CONFIG_PUMP_LOAD_DEADBAND         	= 11;
const unsigned GCS_CONFIG_PUMP_LOAD_DEADBAND_TYPE    	= 12;
const unsigned GCS_CONFIG_PUMP_LOAD_RANGEMAX         	= 13;
const unsigned GCS_CONFIG_PUMP_LOAD_RANGEMIN         	= 14;
const unsigned GCS_CONFIG_PUMP_LOAD_ARCHIVE_ACTIVE      = 15;
const unsigned GCS_CONFIG_PUMP_LOAD_ARCHIVE_TIME_FILTER = 16;
const unsigned GCS_CONFIG_PUMP_NORMAL_POSITION         	= 17;
const unsigned GCS_CONFIG_PUMP_ADDRESS_STSREG01        	= 18;
const unsigned GCS_CONFIG_PUMP_ADDRESS_STSREG02        	= 19;
const unsigned GCS_CONFIG_PUMP_ADDRESS_EVSTSREG01      	= 20;
const unsigned GCS_CONFIG_PUMP_ADDRESS_EVSTSREG02      	= 21;
const unsigned GCS_CONFIG_PUMP_ADDRESS_POSST         	= 22;
const unsigned GCS_CONFIG_PUMP_ADDRESS_LOADST        	= 23;
const unsigned GCS_CONFIG_PUMP_ADDRESS_AUPOSRST        	= 24;
const unsigned GCS_CONFIG_PUMP_ADDRESS_AOUTOVST        	= 25;
const unsigned GCS_CONFIG_PUMP_ADDRESS_MANREG01        	= 26;
const unsigned GCS_CONFIG_PUMP_ADDRESS_MPOSR         	= 27;

//Specified position for mask event is for case of no archives.
//In case of archiving, the position number should be incremented with UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH
const unsigned GCS_CONFIG_PUMP_MASKEVENT           	= 28;








//------------------------------------------------------------------------------------------------------------------------
// gcsConfigPump_setConfig
/**
  Function creates dp and sets configuration for the device.
  Configuration is taken from the DB line, corresponding to the object.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
gcsConfigPump_setConfig(dyn_string dsConfigs, dyn_string &dsExceptionInfo)
{
  string sDpName;


  //unConfigGenericFunctions_checkAll(dsConfigs, GCS_CONFIG_PUMP_DPT_NAME, dsExceptionInfo);

  if (dynlen(dsExceptionInfo) <= 0)
  {
    	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    	unConfigGenericFunctions_createDp(sDpName, GCS_CONFIG_PUMP_DPT_NAME, dsExceptionInfo);

    	//DP is created successfully or existed before
    	if (dynlen(dsExceptionInfo) <= 0)
    	{
      	   unConfigGenericFunctions_setCommon(dsConfigs, dsExceptionInfo);

      	   //Call the device set function
      	   if (dynlen(dsExceptionInfo) <= 0)
      	   {
      	   	unConfigGenericFunctions_setDeviceConfig(dsConfigs, GCS_CONFIG_PUMP_DPT_NAME, dsExceptionInfo);
      	   }
    	}
  }
}






//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_Pump_setConfig
/**
  Function sets configuration for the processing device: units, formats, ranges, modbus addresses.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
_UnPlc_Pump_setConfig(dyn_string dsConfigs, dyn_string &dsExceptionInfo)
{
  gcsConfigPump_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, dsExceptionInfo);
}








//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_Pump_setConfig
/**
  Function sets configuration for the processing device: units, formats, ranges, modbus addresses in case of S7 plc.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
S7_PLC_Pump_setConfig(dyn_string dsConfigs, dyn_string &dsExceptionInfo)
{
  gcsConfigPump_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, dsExceptionInfo);
}







//------------------------------------------------------------------------------------------------------------------------
// gcsConfigPump_setCommonConfig
/**
  Function sets configuration for the processing device: units, formats, ranges, modbus addresses.
  It is used in both cases: unPlc and S7Plc.

  @param[in]  sPlcType     	PLC type: UN_PLC_DPTYPE or S7_PLC_DPTYPE
  @param[in]  dsConfigs  	dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
    				These additional parameters are added during the import process.
  				Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	All possible errors will be returned here
*/
gcsConfigPump_setCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &dsExceptionInfo)
{
  string sDpName;
  string sMask;
  string sScript;

  int iMask;


  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(makeDynString(),"ERROR","gcsConfigPump_checkCommonConfig: Unknown PLC type - " + sPlcType,"");
    return;
  }
  sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

  //Gets archiving configs from object and puts them to the common archiving configs.
  gcsConfigGenericFunctions_setArchives(GCS_CONFIG_PUMP_LENGTH, dsConfigs);







  //2.2. Registers
  if (sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setAllRegisters(dsConfigs, \n";
  else if (sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setAllRegistersS7(dsConfigs, \n";
  else
  {
  	dynAppend(dsExceptionInfo, "Specified PLC type is unknown: " + sPlcType);
  	return;
  }
  sScript += "      makeDynString(\"StsReg01\",\"StsReg02\"), \n";
  sScript += "      makeDynInt(GCS_CONFIG_PUMP_ADDRESS_STSREG01, GCS_CONFIG_PUMP_ADDRESS_STSREG02), \n";
  sScript += "      makeDynInt(GCS_CONFIG_PUMP_ADDRESS_EVSTSREG01, GCS_CONFIG_PUMP_ADDRESS_EVSTSREG02), \n";
  sScript += "      makeDynString(\"ManReg01\"), \n";
  sScript += "      makeDynInt(GCS_CONFIG_PUMP_ADDRESS_MANREG01), \n";
  sScript += "      makeDynString(\"StartISt;StopISt\"), \n";
  sScript += "      makeDynString(UN_STSREG01_STARTIST + \";\" + UN_STSREG01_STOPIST), \n";
  sScript += "      GCS_CONFIG_PUMP_NORMAL_POSITION, dsExceptionInfo); \n";

  evalScript(dsExceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string dsExceptionInfo; " + sScript + " return dsExceptionInfo;}", makeDynString(), dsConfigs);




  // 2.3. Float values
  if (dynlen(dsExceptionInfo) <=0 )
  {
  	if (sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs,  PVSS_MODBUS_FLOAT, \n";
  	else if (sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7_FLOAT, \n";
        else
        {
		dsExceptionInfo = "Specified PLC type is unknown: " + sPlcType;
		return;
  	}
  	sScript += "      makeDynString(\"PosSt\", \"AuPosRSt\", \"AOutOVSt\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_PUMP_ADDRESS_POSST, GCS_CONFIG_PUMP_ADDRESS_AUPOSRST, GCS_CONFIG_PUMP_ADDRESS_AOUTOVST), \n";
  	sScript += "      makeDynString(\"MPosR\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_PUMP_ADDRESS_MPOSR), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_PUMP_SETPOINT_UNIT, GCS_CONFIG_PUMP_SETPOINT_FORMAT, GCS_CONFIG_PUMP_SETPOINT_DEADBAND, GCS_CONFIG_PUMP_SETPOINT_DEADBAND_TYPE), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_PUMP_SETPOINT_RANGEMIN, GCS_CONFIG_PUMP_SETPOINT_RANGEMAX), \n";
  	sScript += "      makeDynString(\"PosSt\", \"AOutOVSt\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_PUMP_SETPOINT_ARCHIVE_ACTIVE, GCS_CONFIG_PUMP_SETPOINT_ARCHIVE_TIME_FILTER), \n";
  	sScript += "      dsExceptionInfo); \n";

  	evalScript(dsExceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string dsExceptionInfo; " + sScript + " return dsExceptionInfo;}", makeDynString(), dsConfigs);
  }



  if (dynlen(dsExceptionInfo) <= 0)
  {
  	if (sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs,  PVSS_MODBUS_FLOAT, \n";
  	else if (sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7_FLOAT, \n";
  	else
	{
		dsExceptionInfo = "Specified PLC type is unknown: " + sPlcType;
	  	return;
  	}
  	sScript += "      makeDynString(\"LoadSt\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_PUMP_ADDRESS_LOADST), \n";
  	sScript += "      makeDynString(), makeDynInt(), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_PUMP_LOAD_UNIT, GCS_CONFIG_PUMP_LOAD_FORMAT, GCS_CONFIG_PUMP_LOAD_DEADBAND, GCS_CONFIG_PUMP_LOAD_DEADBAND_TYPE), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_PUMP_LOAD_RANGEMIN, GCS_CONFIG_PUMP_LOAD_RANGEMAX), \n";
  	sScript += "      makeDynString(\"LoadSt\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_PUMP_LOAD_ARCHIVE_ACTIVE, GCS_CONFIG_PUMP_LOAD_ARCHIVE_TIME_FILTER), \n";
  	sScript += "      dsExceptionInfo); \n";

  	evalScript(dsExceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string dsExceptionInfo; " + sScript + " return dsExceptionInfo;}", makeDynString(), dsConfigs);
  }


  if (dynlen(dsExceptionInfo) <= 0)
  {
  	sMask = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_PUMP_MASKEVENT];
  	unConfigGenericFunctions_checkInt(sMask, iMask, dsExceptionInfo);
  	if (dynlen(dsExceptionInfo)<=0) unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, makeDynString(".ProcessInput.evStsReg01"), dsExceptionInfo);
  }
}







//------------------------------------------------------------------------------------------------------------------------
// gcsConfigPump_checkConfig
/**
  Function checks if specified device configuration is correct.
  Configuration is taken from the DB line, corresponding to the object.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
gcsConfigPump_checkConfig(dyn_string dsConfigs, dyn_string &dsExceptionInfo)
{
  unsigned uUnsigned;
  //int iMaskPosition = GCS_CONFIG_PUMP_MASKEVENT + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH;


  if(dynlen(dsConfigs) >= GCS_CONFIG_PUMP_LENGTH)
  {
	unConfigGenericFunctions_checkParameters(dsConfigs, GCS_CONFIG_PUMP_DPT_NAME, dsExceptionInfo);
    	//if (dynlen(dsExceptionInfo) <= 0) unConfigGenericFunctions_checkUnsigned(dsConfigs[iMaskPosition], uUnsigned, dsExceptionInfo);
    if (dynlen(dsExceptionInfo) <= 0) unConfigGenericFunctions_checkDeviceConfig(dsConfigs, GCS_CONFIG_PUMP_DPT_NAME, dsExceptionInfo);
  }
  else
  {
    	fwException_raise(dsExceptionInfo,"ERROR","gcsConfigPump_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}









//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_Pump_checkConfig
/**
  Checks specified configuration in case of unPlc front-end type

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
_UnPlc_Pump_checkConfig(dyn_string dsConfigs, dyn_string &dsExceptionInfo)
{
  gcsConfigPump_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, dsExceptionInfo);
}







//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_Pump_checkConfig
/**
  Checks specified configuration in case of S7 front-end type

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
S7_PLC_Pump_checkConfig(dyn_string dsConfigs, dyn_string &dsExceptionInfo)
{
    gcsConfigPump_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, dsExceptionInfo);
}







//------------------------------------------------------------------------------------------------------------------------
// gcsConfigPump_checkCommonConfig
/**
  Checks specified configuration for the device.

  @param[in]  plcType     	  PLC type: UN_PLC_DPTYPE or S7_PLC_DPTYPE.
  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
    				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of exceptions
*/
gcsConfigPump_checkCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &dsExceptionInfo)
{
  string sScript;
  bool bHasArchive = false;


  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
  	fwException_raise(dsExceptionInfo, "ERROR","gcsConfigPump_checkCommonConfig: Unknowm PLC type - " + sPlcType,"");
  }
  
  if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_PUMP_LENGTH, dsConfigs, bHasArchive))
  {
    	fwException_raise(dsExceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigPump_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
  
  
  if (bHasArchive)
  {
    	gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_PUMP_LENGTH, dsConfigs, dsExceptionInfo);
    
    	if (dynlen(dsExceptionInfo) > 0)
    	{
    	  fwException_raise(dsExceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigPump_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    	}
  }

  // 2. Registers
  if (dynlen(dsExceptionInfo) <= 0)
  {
  	if (sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_checkAllRegisters(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  	else if (sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_checkAllRegistersS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  	else
  	{
  		dynAppend(dsExceptionInfo, "Specified PLC type is unknown: " + sPlcType);
  		return;
  	}

  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_ADDRESS_STSREG01, GCS_CONFIG_PUMP_ADDRESS_STSREG02), \n";
  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_ADDRESS_EVSTSREG01, GCS_CONFIG_PUMP_ADDRESS_EVSTSREG02), \n";
  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_ADDRESS_MANREG01), \n";
  	sScript += "    GCS_CONFIG_PUMP_NORMAL_POSITION, dsExceptionInfo); \n";

  	evalScript(dsExceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string dsExceptionInfo; " + sScript + " return dsExceptionInfo;}", makeDynString(), dsConfigs);
  }



  // 3. Float block
  if (dynlen(dsExceptionInfo) <= 0)
  {
  	if (sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  	else if (sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  	else
  	{
  		dynAppend(dsExceptionInfo, "Specified PLC type is unknown: " + sPlcType);
  		return;
  	}

  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_ADDRESS_POSST, GCS_CONFIG_PUMP_ADDRESS_AUPOSRST, GCS_CONFIG_PUMP_ADDRESS_AOUTOVST), \n";
  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_ADDRESS_MPOSR),  \n";
  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_SETPOINT_UNIT, GCS_CONFIG_PUMP_SETPOINT_FORMAT, GCS_CONFIG_PUMP_SETPOINT_DEADBAND, GCS_CONFIG_PUMP_SETPOINT_DEADBAND_TYPE), \n";
  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_SETPOINT_RANGEMIN, GCS_CONFIG_PUMP_SETPOINT_RANGEMAX), \n";
  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_SETPOINT_ARCHIVE_ACTIVE, GCS_CONFIG_PUMP_SETPOINT_ARCHIVE_TIME_FILTER), \n";
  	sScript += "    dsExceptionInfo); \n";

  	evalScript(dsExceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string dsExceptionInfo; " + sScript + " return dsExceptionInfo;}", makeDynString(), dsConfigs);
  }



  if (dynlen(dsExceptionInfo) <= 0)
  {
  	if (sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  	else if (sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  	else
  	{
  		dynAppend(dsExceptionInfo, "Specified PLC type is unknown: " + sPlcType);
  		return;
  	}

  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_ADDRESS_LOADST), makeDynInt(),  \n";
  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_LOAD_UNIT, GCS_CONFIG_PUMP_LOAD_FORMAT, GCS_CONFIG_PUMP_LOAD_DEADBAND, GCS_CONFIG_PUMP_LOAD_DEADBAND_TYPE), \n";
  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_LOAD_RANGEMIN, GCS_CONFIG_PUMP_LOAD_RANGEMAX), \n";
  	sScript += "    makeDynInt(GCS_CONFIG_PUMP_LOAD_ARCHIVE_ACTIVE, GCS_CONFIG_PUMP_LOAD_ARCHIVE_TIME_FILTER), \n";
  	sScript += "    dsExceptionInfo); \n";

  	evalScript(dsExceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string dsExceptionInfo; " + sScript + " return dsExceptionInfo;}", makeDynString(), dsConfigs);
  }

  if (dynlen(dsExceptionInfo) <= 0)
  {
    unsigned uUnsigned;
    unConfigGenericFunctions_checkUnsigned(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_PUMP_MASKEVENT], uUnsigned, dsExceptionInfo);
  }
}


//@}
