/**@name LIBRARY: gcsConfigWordComputed.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains the function used to set WordComputed configuration.
Modifications History:
  $Log: gcsConfigWordComputed.ctl,v $
  Revision 1.20  2010/10/01 14:50:10  amerezhi
  Removed PV_RANGE and DEADBAND deletion.

  Revision 1.19  2010/09/08 16:53:00  amerezhi
  The check for RangeMaxMix fields is added
  The check for registers is fixed
  The check for StringList (GCS_CONFIG_WORDCOMPUTED_DESCRIPTION_MPOSR) is fixed

  Revision 1.18  2008/09/04 14:38:02  gthomas
  -moved back to version 1.15

  Revision 1.17  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.16  2008/06/09 14:54:26  aabadz
  Result after move some constants, only use use inside this file, from gcsObjects_declarations.

  Revision 1.15  2008/01/23 15:59:18  gamx
  removed useless comments,
  the version after the final test

  Revision 1.14  2008/01/16 15:57:13  aabadz
  Some sintactic errors fixed

  Revision 1.13  2008/01/09 16:43:10  gamx
  reviewed:
  1) exception handling;
  2) naming;
  3) documenting

  Revision 1.12  2007/11/15 08:38:52  ikoblik
  gcsConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.11  2007/11/14 16:39:31  ikoblik
  gcsConfigGenericFunctions code review comments implementation

  Revision 1.10  2007/10/30 16:21:24  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
  gcsConfigWordComputed_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_WordComputed_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_WordComputed_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordComputed_setCommonConfig (string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordComputed_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_WordComputed_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_WordComputed_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordComputed_checkCommonConfig (string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
*/


// Constants
const string GCS_CONFIG_WORDCOMPUTED_DPT_NAME = "WordComputed";

//Constants indicate positions in the configuration line (for DB file)
//Specified position numbers are relative. First 10 positions are reserved for general Unicos fields.
const unsigned GCS_CONFIG_WORDCOMPUTED_LENGTH       = 15;

const unsigned GCS_CONFIG_WORDCOMPUTED_UNIT         = 1;
const unsigned GCS_CONFIG_WORDCOMPUTED_FORMAT       = 2;
const unsigned GCS_CONFIG_WORDCOMPUTED_RANGEMAX       = 3;
const unsigned GCS_CONFIG_WORDCOMPUTED_RANGEMIN       = 4;
const unsigned GCS_CONFIG_WORDCOMPUTED_DEADBAND       = 5;
const unsigned GCS_CONFIG_WORDCOMPUTED_DEADBAND_TYPE    = 6;
const unsigned GCS_CONFIG_WORDCOMPUTED_ARCHIVE_ACTIVE   = 7;
const unsigned GCS_CONFIG_WORDCOMPUTED_ADDRESS_STSREG01   = 8;
const unsigned GCS_CONFIG_WORDCOMPUTED_ADDRESS_EVSTSREG01 = 9;
const unsigned GCS_CONFIG_WORDCOMPUTED_ADDRESS_POSOV    = 10;
const unsigned GCS_CONFIG_WORDCOMPUTED_ADDRESS_AUPOSRST   = 11;
const unsigned GCS_CONFIG_WORDCOMPUTED_ADDRESS_MANREG01   = 12;
const unsigned GCS_CONFIG_WORDCOMPUTED_ADDRESS_MPOSR    = 13;
const unsigned GCS_CONFIG_WORDCOMPUTED_DESCRIPTION_MPOSR  = 14;

//Specified position for mask event is for case of no archives.
//In case of archiving, the position number should be incremented with UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH
const unsigned GCS_CONFIG_WORDCOMPUTED_MASKEVENT      = 15;

//const unsigned UN_CONFIG_WORDCOMPUTED_LENGTH        = 15;
//const unsigned UN_CONFIG_WORDCOMPUTED_ADDITIONAL_LENGTH   = 0;
//const unsigned UN_CONFIG_WORDCOMPUTED_ARCHIVE_ACTIVE    = 7;





//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordComputed_setConfig
/**
  Function creates dp and sets configuration for the device.
  Configuration is taken from the DB line, corresponding to the object.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
gcsConfigWordComputed_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;


  //unConfigGenericFunctions_checkAll(dsConfigs, GCS_CONFIG_WORDCOMPUTED_DPT_NAME, exceptionInfo);

  if (dynlen(exceptionInfo) <= 0)
  {
    if (dynlen(dsConfigs) >= UN_CONFIG_OBJECTGENERAL_DPNAME) sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    	unConfigGenericFunctions_createDp(sDpName, GCS_CONFIG_WORDCOMPUTED_DPT_NAME, exceptionInfo);

    //exceptions are accumulated inside exceptionInfo using dynAppend
    if (dynlen(exceptionInfo) <= 0)
    	unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
    if (dynlen(exceptionInfo) <= 0)
    	unConfigGenericFunctions_setDeviceConfig(dsConfigs, GCS_CONFIG_WORDCOMPUTED_DPT_NAME, exceptionInfo);
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_WordComputed_setConfig
/**
  Function sets configuration for the processing device: units, formats, ranges, modbus addresses.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
_UnPlc_WordComputed_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordComputed_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_WordComputed_setConfig
/**
  Function sets configuration for the processing device: units, formats, ranges, modbus addresses in case of S7 plc.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
S7_PLC_WordComputed_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordComputed_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordComputed_setCommonConfig
/**
  Function sets configuration for the processing device: units, formats, ranges, modbus addresses.
  It is used in both cases: unPlc and S7Plc.

  @param[in]  sPlcType     	PLC type: UN_PLC_DPTYPE or S7_PLC_DPTYPE
  @param[in]  dsConfigs  	dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
    				These additional parameters are added during the import process.
  				Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	All possible errors will be returned here
*/
gcsConfigWordComputed_setCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;
  string sPositionStatus;
  string sDeviceName;
  string sDescription;
  string sMask;
  string sScript;
  bool bArchiveActive;
  int bArchiveTimeFilter;
  int iArchiveType;
  int iSmoothProcedure;
  int iMask;



  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWordComputed_checkCommonConfig: Unknown PLC type - " + sPlcType,"");
    return;
  }


  sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

  //Gets archiving configs from object and puts them to the common archiving configs.
  gcsConfigGenericFunctions_setArchives(GCS_CONFIG_WORDCOMPUTED_LENGTH, dsConfigs);



  // Registers
  if(sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setAllRegisters(dsConfigs, \n";
  else if(sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setAllRegistersS7(dsConfigs, \n";
  else
  {
  	dynAppend(exceptionInfo, "Specified PLC type is unknown: " + sPlcType);
  	return;
  }

  sScript += "      makeDynString(\"StsReg01\"), \n";
  sScript += "      makeDynInt(GCS_CONFIG_WORDCOMPUTED_ADDRESS_STSREG01), \n";
  sScript += "      makeDynInt(GCS_CONFIG_WORDCOMPUTED_ADDRESS_EVSTSREG01), \n";
  sScript += "      makeDynString(\"ManReg01\"), \n";
  sScript += "      makeDynInt(GCS_CONFIG_WORDCOMPUTED_ADDRESS_MANREG01), \n";
  sScript += "      makeDynString(), makeDynString(), \n";
  sScript += "      0, exceptionInfo); \n";

  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + sScript + " return exceptionInfo;}", makeDynString(), dsConfigs);


  // 2.3. Int values
  // for some reasons unicos set pv_range only for the first DPE in the list, so we need to set it manually for others.
  if (dynlen(exceptionInfo) <= 0)
  {
  	if(sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs,  PVSS_MODBUS_UINT16, \n";
  	else if(sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7_UINT16, \n";
  	else
  	{
  		dynAppend(exceptionInfo, "Specified PLC type is unknown: " + sPlcType);
  		return;
  	}

  	sScript += "      makeDynString(\"PosOV\", \"AuPosRSt\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_WORDCOMPUTED_ADDRESS_POSOV, GCS_CONFIG_WORDCOMPUTED_ADDRESS_AUPOSRST), \n";
  	sScript += "      makeDynString(\"MPosR\"), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_WORDCOMPUTED_ADDRESS_MPOSR), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_WORDCOMPUTED_UNIT, GCS_CONFIG_WORDCOMPUTED_FORMAT, GCS_CONFIG_WORDCOMPUTED_DEADBAND, GCS_CONFIG_WORDCOMPUTED_DEADBAND_TYPE), \n";
  	sScript += "      makeDynInt(GCS_CONFIG_WORDCOMPUTED_RANGEMIN, GCS_CONFIG_WORDCOMPUTED_RANGEMAX), \n";
  	sScript += "      makeDynString(), makeDynInt(), exceptionInfo); \n";

  	evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + sScript + " return exceptionInfo;}", makeDynString(), dsConfigs);
  }


  if (dynlen(exceptionInfo) <= 0)
  {
  	if (dynlen(dsConfigs) >= (GCS_CONFIG_WORDCOMPUTED_MASKEVENT + UN_CONFIG_ALLCOMMON_LENGTH))
  			sMask = dsConfigs[GCS_CONFIG_WORDCOMPUTED_MASKEVENT + UN_CONFIG_ALLCOMMON_LENGTH];
  	sscanf(sMask, "%d", iMask);
  	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask,  makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
  }


  //2.4. Archive for PosOV
  if (dynlen(exceptionInfo) <= 0)
  {
  	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_WORDCOMPUTED_ARCHIVE_ACTIVE], bArchiveActive, bArchiveTimeFilter, iArchiveType, iSmoothProcedure);
  	if(bArchiveActive)
  	{
  	  fwArchive_set(sDpName + ".ProcessInput.PosOV",
  	      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
  	      DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW,
  	      0, 0, exceptionInfo);
  	}
  	else
  	{
  	  fwArchive_delete(sDpName + ".ProcessInput.PosOV", exceptionInfo);
  	}
  }


  //2.5. Displaying a string or int value
  if (dynlen(exceptionInfo) <= 0)
  {
  	int iMposrDescription = UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDCOMPUTED_DESCRIPTION_MPOSR;
  	if (dynlen(dsConfigs) >= iMposrDescription) sPositionStatus = dsConfigs[iMposrDescription];

  	sDeviceName = (unGenericDpFunctions_getDpName(sDpName) + ".ProcessInput.PosOV");

  	if (sPositionStatus == "") sPositionStatus = GCS_CONFIG_EMPTY_STRING;

  	unConfigGenericFunctions_setDescription(sDeviceName, sPositionStatus, exceptionInfo);
  }
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordComputed_checkConfig
/**
  Function checks if specified device configuration is correct.
  Configuration is taken from the DB line, corresponding to the object.

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
gcsConfigWordComputed_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  unsigned uUnsigned;
  if(dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH)
  {
    //exceptions are accumulated in exceptionInfo
    unConfigGenericFunctions_checkParameters(dsConfigs, GCS_CONFIG_WORDCOMPUTED_DPT_NAME, exceptionInfo);
	if (dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDCOMPUTED_MASKEVENT) {
		unConfigGenericFunctions_checkUnsigned(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDCOMPUTED_MASKEVENT], uUnsigned, exceptionInfo);
	}
    unConfigGenericFunctions_checkDeviceConfig(dsConfigs, GCS_CONFIG_WORDCOMPUTED_DPT_NAME, exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWordComputed_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_WordComputed_checkConfig
/**
  Checks specified configuration in case of unPlc front-end type

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
_UnPlc_WordComputed_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordComputed_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_WordComputed_checkConfig
/**
  Checks specified configuration in case of S7 front-end type

  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
  				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of errors
*/
S7_PLC_WordComputed_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
    gcsConfigWordComputed_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordComputed_checkCommonConfig
/**
  Checks specified configuration for the device.

  @param[in]  sPlcType     	  PLC type: UN_PLC_DPTYPE or S7_PLC_DPTYPE.
  @param[in]  dsConfigs  	  dsConfigs is a device configuration line (from DB)+ additional parameters needed for PVSS.
    				  These additional parameters are added during the import process.
  				  Positions of these additional parameters are specified in unConfigGenericFunctions.ctl.
  @param[out] dsExceptionInfo 	  list of exceptions
*/
gcsConfigWordComputed_checkCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sScript;
  bool bHasArchive = false;

  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(makeDynString(),"ERROR","gcsConfigWordComputed_checkCommonConfig: Unknowm PLC type - " + sPlcType,"");
    return;
  }

  if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_WORDCOMPUTED_LENGTH, dsConfigs, bHasArchive))
  {
    fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigWordComputed_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    return;
  }

  if (bHasArchive)
  {
    gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_WORDCOMPUTED_LENGTH, dsConfigs, exceptionInfo);
    if (dynlen(exceptionInfo) > 0)
    {
      fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigWordComputed_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
      return;
    }
  }


  // Registers
  if(sPlcType == UN_PLC_DPTYPE)
	sScript = " unConfigGenericFunctions_checkAllRegisters(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  if(sPlcType == S7_PLC_DPTYPE)
	sScript = " unConfigGenericFunctions_checkAllRegistersS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  sScript += "    makeDynInt(GCS_CONFIG_WORDCOMPUTED_ADDRESS_STSREG01), \n";
  sScript += "    makeDynInt(GCS_CONFIG_WORDCOMPUTED_ADDRESS_EVSTSREG01), \n";
  sScript += "    makeDynInt(GCS_CONFIG_WORDCOMPUTED_ADDRESS_MANREG01), \n";
  sScript += "    0, exceptionInfo); \n";
  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + sScript + " return exceptionInfo;}", makeDynString(), dsConfigs);

  // Float block
  dyn_string exceptionInfoTemp;
  if(sPlcType == UN_PLC_DPTYPE)
	sScript = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  if(sPlcType == S7_PLC_DPTYPE)
	sScript = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  sScript += "    makeDynInt(GCS_CONFIG_WORDCOMPUTED_ADDRESS_POSOV, GCS_CONFIG_WORDCOMPUTED_ADDRESS_AUPOSRST), \n";
  sScript += "    makeDynInt(GCS_CONFIG_WORDCOMPUTED_ADDRESS_MPOSR), \n";
  sScript += "    makeDynInt(GCS_CONFIG_WORDCOMPUTED_UNIT, GCS_CONFIG_WORDCOMPUTED_FORMAT, GCS_CONFIG_WORDCOMPUTED_DEADBAND, GCS_CONFIG_WORDCOMPUTED_DEADBAND_TYPE), \n";
  sScript += "    makeDynInt(GCS_CONFIG_WORDCOMPUTED_RANGEMIN, GCS_CONFIG_WORDCOMPUTED_RANGEMAX), \n";
  sScript += "    makeDynInt(), exceptionInfoTemp); \n";

  evalScript(exceptionInfoTemp, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfoTemp; " + sScript + " return exceptionInfoTemp;}", makeDynString(), dsConfigs);
  dynAppend(exceptionInfo, exceptionInfoTemp);


  // 4. Particular dpe : PosOV
  unConfigGenericFunctions_checkArchiveParameters("", dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDCOMPUTED_ARCHIVE_ACTIVE], exceptionInfo);
  // 5. Parameter to display int or string values
  gcsConfigGenericFunctions_checkIntVariablesPairs(dsConfigs, UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDCOMPUTED_DESCRIPTION_MPOSR, exceptionInfo);
}


//@}
