/**@name LIBRARY: gcsConfigWordParameter.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains the function used to set WordParameter configuration.
Modifications History:
  $Log: gcsConfigWordParameter.ctl,v $
  Revision 1.24  2010/10/08 12:52:37  amerezhi
  The unConfigGenericFunctions_checkFloatIntBlock is added to gcsConfigWordParameter_checkCommonConfig.

  Revision 1.23  2008/07/28 12:52:37  gamx
  exception handling has been restored

  Revision 1.22  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.21  2008/01/22 12:52:44  aabadz
  Fixed:
  - Naming
  - Call to other functions to reuse code
  - Some error check

  Revision 1.20  2007/12/10 15:42:45  aabadz
  Fixed error in variable name

  Revision 1.19  2007/11/15 08:38:52  ikoblik
  gcsConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.18  2007/11/14 16:39:31  ikoblik
  gcsConfigGenericFunctions code review comments implementation

  Revision 1.17  2007/10/30 16:21:29  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
  gcsConfigWordParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_WordParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_WordParameter_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordParameter_setCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_WordParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_WordParameter_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordParameter_checkCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
*/

const string GCS_CONFIG_WORDPARAMETER_DPT_NAME = "WordParameter";
// Constants
const unsigned GCS_CONFIG_WORDPARAMETER_LENGTH = 11;

const unsigned GCS_CONFIG_WORDPARAMETER_UNIT = 1;
const unsigned GCS_CONFIG_WORDPARAMETER_FORMAT = 2;
const unsigned GCS_CONFIG_WORDPARAMETER_DEADBANDE = 3;
const unsigned GCS_CONFIG_WORDPARAMETER_DEADBANDE_TYPE = 4;
const unsigned GCS_CONFIG_WORDPARAMETER_RANGEMAX = 5;
const unsigned GCS_CONFIG_WORDPARAMETER_RANGEMIN = 6;
const unsigned GCS_CONFIG_WORDPARAMETER_ARCHIVE_ACTIVE = 7;
const unsigned GCS_CONFIG_WORDPARAMETER_ADDRESS_VALRQACK = 8;
const unsigned GCS_CONFIG_WORDPARAMETER_ADDRESS_CURVALST = 9;
const unsigned GCS_CONFIG_WORDPARAMETER_ADDRESS_VALRQ = 10;
const unsigned GCS_CONFIG_WORDPARAMETER_DEFAULT_VALUE = 11;

const unsigned UN_CONFIG_WORDPARAMETER_LENGTH = 11;
const unsigned UN_CONFIG_WORDPARAMETER_ADDITIONAL_LENGTH  = 0;
const unsigned UN_CONFIG_WORDPARAMETER_ARCHIVE_ACTIVE = 7;


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordParameter_setConfig
/**
Set configuration of given WordParameter datapoint

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWordParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName, sDescription;
  bool archiveActive;
  int archiveTimeFilter, iArchiveType, iSmoothProcedure;

  // 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, "WordParameter", exceptionInfo);

  if (dynlen(exceptionInfo) <= 0)   // Good inputs
  {
// 2. Generate device
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, "WordParameter", exceptionInfo);
    if (dynlen(exceptionInfo) <= 0) // Creation ok or dp already exists
    {
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
      unConfigGenericFunctions_setDeviceConfig(dsConfigs, "WordParameter", exceptionInfo);
    }
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_WordParameter_setConfig
/**
Set configuration of given WordParameter datapoint for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_WordParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordParameter_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_WordParameter_setConfig
/**
Set configuration of given WordParameter datapoint for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_WordParameter_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordParameter_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordParameter_setCommonConfig
/**
Set configuration of given WordParameter datapoint for both unPLc and S7_PLC front-end type

  @param[in]  plcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWordParameter_setCommonConfig(string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName, sDescription, str;
  int archiveTimeFilter, iArchiveType, iSmoothProcedure;
  bool archiveActive;

  if(!gcsConfigGenericFunctions_checkPlcType(plcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWordParameter_setCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + plcType,"");
    return;
  }
  sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite setting from import panel.
  gcsConfigGenericFunctions_setArchives(GCS_CONFIG_WORDPARAMETER_LENGTH, dsConfigs);

  // 2.2. Int values
  if(plcType == UN_PLC_DPTYPE)
    str = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs, PVSS_MODBUS_UINT16, \n";
  if(plcType == S7_PLC_DPTYPE)
    str = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7_UINT16, \n";
  str += "      makeDynString(\"ValRqAck\"), \n";
  str += "      makeDynInt(GCS_CONFIG_WORDPARAMETER_ADDRESS_VALRQACK), \n";
  str += "      makeDynString(\"ValRq\"), \n";
  str += "      makeDynInt(GCS_CONFIG_WORDPARAMETER_ADDRESS_VALRQ), \n";
  str += "      makeDynInt(GCS_CONFIG_WORDPARAMETER_UNIT, GCS_CONFIG_WORDPARAMETER_FORMAT, GCS_CONFIG_WORDPARAMETER_DEADBANDE, GCS_CONFIG_WORDPARAMETER_DEADBANDE_TYPE), \n";
  str += "      makeDynInt(), \n";
  str += "      makeDynString(), makeDynInt(), exceptionInfo); \n";
  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);

  if(plcType == UN_PLC_DPTYPE)
    str = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs, PVSS_MODBUS_UINT16, \n";
  if(plcType == S7_PLC_DPTYPE)
    str = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7_UINT16, \n";
  str += "      makeDynString(\"CurValSt\"), \n";
  str += "      makeDynInt(GCS_CONFIG_WORDPARAMETER_ADDRESS_CURVALST), \n";
  str += "      makeDynString(), \n";
  str += "      makeDynInt(), \n";
  str += "      makeDynInt(GCS_CONFIG_WORDPARAMETER_UNIT, GCS_CONFIG_WORDPARAMETER_FORMAT, GCS_CONFIG_WORDPARAMETER_DEADBANDE, GCS_CONFIG_WORDPARAMETER_DEADBANDE_TYPE), \n";
  str += "      makeDynInt(GCS_CONFIG_WORDPARAMETER_RANGEMIN, GCS_CONFIG_WORDPARAMETER_RANGEMAX), \n";
  str += "      makeDynString(\"CurValSt\"), \n";
  str += "      makeDynInt(GCS_CONFIG_WORDPARAMETER_ARCHIVE_ACTIVE), exceptionInfo); \n";
  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);


// can not use unConfigGenericFunctions_setCommonFloatIntBlock, besause it works ONLY with "processinput" objects
  unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.ValRq", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_UNIT], exceptionInfo);
  unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.ValRq", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_FORMAT], exceptionInfo);
  unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_UNIT], exceptionInfo);
  unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_FORMAT], exceptionInfo);
  dpSetWait(sDpName + ".ProcessOutput.DfltVal", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_DEFAULT_VALUE]);

  // 2.3. Particular dpe : CurValSt : Archive
  unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_ARCHIVE_ACTIVE],
      archiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
  if (archiveActive)      // Archive
  {
    fwArchive_set(sDpName + ".ProcessInput.CurValSt",
        dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
        DPATTR_ARCH_PROC_SIMPLESM,
        DPATTR_COMPARE_OLD_NEW,
        0, 0, exceptionInfo);
  }
  else            // No Archive
  {
    fwArchive_delete(sDpName + ".ProcessInput.CurValSt", exceptionInfo);
  }

  // 2.4. Description for the Recipes
  sDescription = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DESCRIPTION];
  unConfigGenericFunctions_setDescription(sDpName + ".ProcessOutput.ValRq", sDescription, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordParameter_checkConfig
/**
Check data before configuration

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWordParameter_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	bool bHasArchive = false;

	if (dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH) {
		unConfigGenericFunctions_checkParameters(dsConfigs, "WordParameter", exceptionInfo);
		// Call the device check function
		unConfigGenericFunctions_checkDeviceConfig(dsConfigs, "WordParameter", exceptionInfo);
	} else {
		fwException_raise(exceptionInfo,"ERROR","gcsConfigWordParameter_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_WordParameter_checkConfig
/**
Check data before configuration for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_WordParameter_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordParameter_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);

  unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH,
      makeDynInt(GCS_CONFIG_WORDPARAMETER_ADDRESS_VALRQACK, GCS_CONFIG_WORDPARAMETER_ADDRESS_CURVALST),
      makeDynInt(GCS_CONFIG_WORDPARAMETER_ADDRESS_VALRQ),
      makeDynInt(GCS_CONFIG_WORDPARAMETER_UNIT, GCS_CONFIG_WORDPARAMETER_FORMAT, GCS_CONFIG_WORDPARAMETER_DEADBANDE, GCS_CONFIG_WORDPARAMETER_DEADBANDE_TYPE),
      makeDynInt(GCS_CONFIG_WORDPARAMETER_RANGEMIN, GCS_CONFIG_WORDPARAMETER_RANGEMAX),
      makeDynInt(), exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_WordParameter_checkConfig
/**
Check data before configuration for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_WordParameter_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordParameter_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);

///@attention Do not call unConfigGenericFunctions_checkFloatIntBlockS7 function here! words are not treated by Unicos!!!
  unConfigGenericFunctions_checkAddressS7(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_ADDRESS_VALRQACK], UN_CONFIG_S7_PLC_DATATYPE_WORD, exceptionInfo);
  unConfigGenericFunctions_checkAddressS7(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_ADDRESS_CURVALST], UN_CONFIG_S7_PLC_DATATYPE_WORD, exceptionInfo);
  unConfigGenericFunctions_checkAddressS7(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_ADDRESS_VALRQ], UN_CONFIG_S7_PLC_DATATYPE_WORD, exceptionInfo);

// 2. Common data & Range & Archive
  unConfigGenericFunctions_checkCommonFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH,
      makeDynInt(GCS_CONFIG_WORDPARAMETER_UNIT, GCS_CONFIG_WORDPARAMETER_FORMAT, GCS_CONFIG_WORDPARAMETER_DEADBANDE, GCS_CONFIG_WORDPARAMETER_DEADBANDE_TYPE),
      makeDynInt(GCS_CONFIG_WORDPARAMETER_RANGEMIN, GCS_CONFIG_WORDPARAMETER_RANGEMAX),
      makeDynInt(), exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordParameter_checkCommonConfig
/**
Check configuration of given WordParameter datapoint for both unPLc and S7_PLC front-end type

  @param[in]  plcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWordParameter_checkCommonConfig(string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string str;
  int iValue, iMin, iMax;
  if(!gcsConfigGenericFunctions_checkPlcType(plcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWordParameter_checkCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + plcType,"");
    return;
  }
  bool bHasArchive = false;
  if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_WORDPARAMETER_LENGTH, dsConfigs, bHasArchive))
  {
    fwException_raise(exceptionInfo,"ERROR","with PLC=" + plcType + ". gcsConfigWordParameter_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    return;
  }
  if (bHasArchive){
    gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_WORDPARAMETER_LENGTH, dsConfigs, exceptionInfo);
    if (dynlen(exceptionInfo) != 0){
      fwException_raise(exceptionInfo,"ERROR","with PLC=" + plcType + ". gcsConfigWordParameter_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
      return;
    }
  }

  unConfigGenericFunctions_checkArchiveParameters("", dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_ARCHIVE_ACTIVE], exceptionInfo);
  unConfigGenericFunctions_checkInt(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_DEFAULT_VALUE], iValue, exceptionInfo);
  unConfigGenericFunctions_checkFloat(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_RANGEMIN], iMin, exceptionInfo);
  unConfigGenericFunctions_checkFloat(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDPARAMETER_RANGEMAX], iMax, exceptionInfo);
  if((iValue < iMin) || (iValue > iMax))
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWordParameter_checkCommonConfig: " + getCatStr("gcsGeneration","BADRANGE"),"");
}


//@}
