/**@name LIBRARY: gcsConfigWordStatus.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 23.07.2003
@brief This library contains the function used to set WordStatus configuration.
Modifications History:
  $Log: gcsConfigWordStatus.ctl,v $
  Revision 1.20  2011/05/02 12:00:00  Herve
  IS-510: CPC 5 and CPC 6 compatibility of the export/import of the CPC device configuration with all the front-end types
  
  Revision 1.19  2010/09/08 16:53:00  amerezhi
  The check for StringList (GCS_CONFIG_WORDSTATUS_DESCRIPTION_AUPOSRST) is fixed.
  Setting empty description is fixed.

  Revision 1.24  2008/07/28 09:23:40  gamx
  exception handling has been restored.

  Revision 1.22  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.21  2008/05/26 12:18:18  gthomas
  Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
  to be used with PVSS 3.6 QT

  22/04/08    Enrique (AB/CO) Archive set in UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG instead of BOOLEAN
  Revision 1.20  2008/02/12 15:32:48  gamx
  an extra function is included to remove spaces from stepper description.
  [the format of description is "1=state1,2=state2,...", spaces are not allowed]

  Revision 1.19  2007/12/03 14:01:04  gamx
  fully checked version, fixed syntactic errors

  Revision 1.18  2007/11/29 11:32:24  gamx
  modifications after code review:
  changes to be consistent with naming conventions,
  introduced constants (to keep dpt name) and the set of external functions (to extract archive settings),
  exception checks are added.
  Functions to check and set WS as As have been moved here from AS library.

  Revision 1.17  2007/11/15 08:38:52  ikoblik
  gcsConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.16  2007/11/14 16:39:31  ikoblik
  gcsConfigGenericFunctions code review comments implementation

  Revision 1.15  2007/10/30 16:21:35  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
  gcsConfigWordStatus_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_WordStatus_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_WordStatus_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordStatus_setCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordStatus_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordStatus_WSAScheckConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordStatus_WSASsetConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_WordStatus_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_WordStatus_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWordStatus_checkCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
*/


// Constants
const string GCS_CONFIG_WORDSTATUS_DPT_NAME = "WordStatus";

//Constants indicate position numbers in the config line of the WordStatus object.
//Each position is relative as first 10 are reserved for Unicos information.
const unsigned GCS_CONFIG_WORDSTATUS_LENGTH       = 9;

const unsigned GCS_CONFIG_WORDSTATUS_UNIT         = 1;
const unsigned GCS_CONFIG_WORDSTATUS_FORMAT       = 2;
const unsigned GCS_CONFIG_WORDSTATUS_DEADBAND    = 3;
const unsigned GCS_CONFIG_WORDSTATUS_DEADBAND_TYPE = 4;
const unsigned GCS_CONFIG_WORDSTATUS_RANGEMAX     = 5;
const unsigned GCS_CONFIG_WORDSTATUS_RANGEMIN     = 6;
const unsigned GCS_CONFIG_WORDSTATUS_ARCHIVE_ACTIVE = 7;
const unsigned GCS_CONFIG_WORDSTATUS_ADDRESS_AUPOSRST = 8;
const unsigned GCS_CONFIG_WORDSTATUS_DESCRIPTION_AUPOSRST = 9;

const unsigned UN_CONFIG_WORDSTATUS_LENGTH        = 9;
const unsigned UN_CONFIG_WORDSTATUS_ADDITIONAL_LENGTH = 0;
const unsigned UN_CONFIG_WORDSTATUS_ARCHIVE_ACTIVE = 7;

// constant declaration for WordStatus device used with CMW front-end
const unsigned UN_CONFIG_CMW_WORDSTATUS_LENGTH 			= 17;
const unsigned UN_CONFIG_CMW_WORDSTATUS_UNIT 				= 1;
const unsigned UN_CONFIG_CMW_WORDSTATUS_FORMAT 			= 2;
const unsigned UN_CONFIG_CMW_WORDSTATUS_DEADBANDE 		= 3;
const unsigned UN_CONFIG_CMW_WORDSTATUS_DEADBANDE_TYPE = 4;
const unsigned UN_CONFIG_CMW_WORDSTATUS_RANGEMAX 		= 5;
const unsigned UN_CONFIG_CMW_WORDSTATUS_RANGEMIN 		= 6;
const unsigned UN_CONFIG_CMW_WORDSTATUS_ARCHIVE_ACTIVE = 7;
const unsigned UN_CONFIG_CMW_WORDSTATUS_ARCHIVE_TIME_FILTER = 8;
const unsigned UN_CONFIG_CMW_WORDSTATUS_TYPE = 9;
const unsigned UN_CONFIG_CMW_WORDSTATUS_TRANSFORMATION_TYPE = 10;
const unsigned UN_CONFIG_CMW_WORDSTATUS_GROUP = 11;
const unsigned UN_CONFIG_CMW_WORDSTATUS_ADDRESS_CMWDEVICE = 12;
const unsigned UN_CONFIG_CMW_WORDSTATUS_ADDRESS_CMWPROPERTY = 13;
const unsigned UN_CONFIG_CMW_WORDSTATUS_ADDRESS_CMWTAG = 14;
const unsigned UN_CONFIG_CMW_WORDSTATUS_ADDRESS_CMW_FILTERNAME = 15;
const unsigned UN_CONFIG_CMW_WORDSTATUS_ADDRESS_CMW_FILTERVALUE = 16;
const unsigned UN_CONFIG_CMW_WORDSTATUS_ADDRESS_LLC = 17;

// constant declaration for WordStatus device used with SOFT_FE front-end
const unsigned UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH 			= 7;
const unsigned UN_CONFIG_SOFT_FE_WORDSTATUS_UNIT 			= 1;
const unsigned UN_CONFIG_SOFT_FE_WORDSTATUS_FORMAT 			= 2;
const unsigned UN_CONFIG_SOFT_FE_WORDSTATUS_RANGEMAX 		= 3;
const unsigned UN_CONFIG_SOFT_FE_WORDSTATUS_RANGEMIN 		= 4;
const unsigned UN_CONFIG_SOFT_FE_WORDSTATUS_ARCHIVE_ACTIVE = 5;
const unsigned UN_CONFIG_SOFT_FE_WORDSTATUS_ARCHIVE_TIME_FILTER = 6;
const unsigned UN_CONFIG_SOFT_FE_WORDSTATUS_DESCRIPTION = 7;


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordStatus_setConfig
/**
Creates dp (or checks if created) of corresponding type and sets its configuration.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWordStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;


  if(strpos(dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_WIDGET], GCS_CONFIG_ANALOGSTATUS_DPT_NAME) >= 0)    // we have WS->AS
  {
    gcsConfigWordStatus_WSASsetConfig(dsConfigs, exceptionInfo);
    return;
  }

  // 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, GCS_CONFIG_WORDSTATUS_DPT_NAME, exceptionInfo);
  if (dynlen(exceptionInfo) <= 0)   // Good inputs
  {
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, GCS_CONFIG_WORDSTATUS_DPT_NAME, exceptionInfo);
    if (dynlen(exceptionInfo) <= 0) // Creation ok or dp already exists
    {
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);

      // Call the device set function in case common config is set correctly
      if (dynlen(exceptionInfo) <= 0)
      {
      	unConfigGenericFunctions_setDeviceConfig(dsConfigs, GCS_CONFIG_WORDSTATUS_DPT_NAME, exceptionInfo);
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordStatus_WSASsetConfig
/**
  Analyses configuration of WordStatus object and creates corresponding AnalogStatus object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWordStatus_WSASsetConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sArchiveSettings;
  int iAuPosRStPosition = UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_DESCRIPTION_AUPOSRST;
  string sScale = 1.0, sArchiveType, sArchiveTime;
  int iRes;
  bool bNoScale = false;


  if (dynlen(dsConfigs) >= iAuPosRStPosition) sArchiveSettings = dsConfigs[iAuPosRStPosition];

  //Example <scale=0.1|archive=VR,2.0,A|time=20> 2% relative value smoothing and 20s time smoothing <scale=0.1|archive=VR,3.0,N|time=0> 3% relative value smoothing no time smoothing
  //if(strpos(description, "scale") < 0 || strpos(description, "archive") < 0 || strpos(description, "time") < 0 || strpos(description, "|") < 0)
  if(strpos(sArchiveSettings, "archive") < 0)
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWordStatus_WSASsetConfig: WS->AS " + getCatStr("unGeneration","BADINPUTS"),"");
    return;
  }



  gcsConfigAnalogStatus_getArchiveSettings(sArchiveSettings, sScale, bNoScale, sArchiveType, sArchiveTime, exceptionInfo);
  if (dynlen(exceptionInfo) > 0) return; //error is already placed in exception info



  //as corresponding positions of WS object can be different from position in AS config,
  //conversion is required.
  dsConfigs[UN_CONFIG_OBJECTGENERAL_OBJECT] = GCS_CONFIG_ANALOGSTATUS_DPT_NAME;
  dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGSTATUS_ADDRESS_AUPOSRST]
    = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_ADDRESS_AUPOSRST];
  dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGSTATUS_ARCHIVE_ACTIVE] = sArchiveType;
  dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGSTATUS_ARCHIVE_TIME_FILTER] = sArchiveTime;
  dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGSTATUS_FORMAT] = "###.##";
  //strreplace(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], "WordStatus", GCS_CONFIG_ANALOGSTATUS_DPT_NAME); -- TBChecked

  addGlobal("bDoConvertion", BOOL_VAR);
  bDoConvertion = true;
  gcsConfigAnalogStatus_setConfig(dsConfigs, exceptionInfo);
  if (dynlen(exceptionInfo) > 0)
  {
      fwException_raise(exceptionInfo,"ERROR","gcsConfigWordStatus_WSASsetConfig: the following config cannot be set:" + dsConfigs,"");
      return;
  }
  removeGlobal("bDoConvertion");


  if(bNoScale == false)
  {
    string dpe = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.AuPosRSt";
    iRes = dpSetWait(dpe + ":_msg_conv.._type", DPCONFIG_CONVERSION_RAW_TO_ENG_MAIN,
      dpe + ":_msg_conv.1._type", DPDETAIL_CONV_POLY,
      dpe + ":_msg_conv.1._poly_grade", 1,
      dpe + ":_msg_conv.1._poly_a", 0,
      dpe + ":_msg_conv.1._poly_b", sScale);
    if(iRes != 0)
    {
      fwException_raise(exceptionInfo,"ERROR","gcsConfigWordStatus_WSASsetConfig: WS->AS convertion" + getCatStr("unGeneration","BADINPUTS"),"");
      return;
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_WordStatus_setConfig
/**
Set configuration of given WordStatus datapoint for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_WordStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordStatus_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_WordStatus_setConfig
/**
Set configuration of given WordStatus datapoint for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_WordStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordStatus_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordStatus_setCommonConfig
/**
Set configuration of given WordStatus datapoint for both unPLc and S7_PLC front-end type

  @param[in]  sPlcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWordStatus_setCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDeviceName, sDescription, sDpName, sScript;
  bool bArchiveActive;
  int iArchiveTimeFilter, iArchiveType, iSmoothProcedure;
  int iRes = 0;

  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWordStatus_setCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + sPlcType,"");
    return;
  }
  if (dynlen(dsConfigs) >= UN_CONFIG_OBJECTGENERAL_DPNAME) sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

  //if archive name are in the config->overwrite setting from import panel.
  gcsConfigGenericFunctions_setArchives(GCS_CONFIG_WORDSTATUS_LENGTH, dsConfigs);


  if(sPlcType == UN_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs,  PVSS_MODBUS_UINT16, \n";
  else if(sPlcType == S7_PLC_DPTYPE) sScript = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7_UINT16, \n";
  else
  {
  	dynAppend(exceptionInfo, "gcsConfigWordStatus_setCommonConfig: unknown plc type: " + sPlcType);
  	return;
  }
  sScript += "    makeDynString(\"AuPosRSt\"), \n";
  sScript += "    makeDynInt(GCS_CONFIG_WORDSTATUS_ADDRESS_AUPOSRST), \n";
  sScript += "    makeDynString(), makeDynInt(), \n";
  sScript += "    makeDynInt(GCS_CONFIG_WORDSTATUS_UNIT, GCS_CONFIG_WORDSTATUS_FORMAT, GCS_CONFIG_WORDSTATUS_DEADBAND, GCS_CONFIG_WORDSTATUS_DEADBAND_TYPE), \n";
  sScript += "    makeDynInt(GCS_CONFIG_WORDSTATUS_RANGEMIN, GCS_CONFIG_WORDSTATUS_RANGEMAX), \n";
  sScript += "    makeDynString(), makeDynInt(), exceptionInfo); \n";
  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + sScript + " return exceptionInfo;}",
    makeDynString(), dsConfigs);

  if (dynlen(exceptionInfo) <= 0)
  {
  	//Particular dpe : AuPosRSt : Archive
  	if (dynlen(dsConfigs) >= (UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_WORDSTATUS_ARCHIVE_ACTIVE))
  		unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_WORDSTATUS_ARCHIVE_ACTIVE], bArchiveActive, iArchiveTimeFilter, iArchiveType, iSmoothProcedure);

  	if (bArchiveActive)
  	{
  	  fwArchive_set(sDpName + ".ProcessInput.AuPosRSt",
  	          dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
  	          DPATTR_ARCH_PROC_SIMPLESM,
  	          DPATTR_COMPARE_OLD_NEW,
  	          0,
  	          0,
  	          exceptionInfo);
  	}
  	else
  	{
  	  fwArchive_delete(sDpName + ".ProcessInput.AuPosRSt", exceptionInfo);
  	}

  	//Set description
  	if (dynlen(dsConfigs) >= (UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_DESCRIPTION_AUPOSRST))
  		sDescription = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_DESCRIPTION_AUPOSRST];

  	if (sDescription == "")
  	{
	  sDescription = GCS_CONFIG_EMPTY_STRING;
  	}

	//all spaces from WS description should be removed otherwise there is a problem to handle steppers
	iRes = strreplace(sDescription, " ", "");
	if (iRes >= 0)
	{
  		sDeviceName = (unGenericDpFunctions_getDpName(sDpName) + ".ProcessInput.AuPosRSt");
  		unConfigGenericFunctions_setDescription(sDeviceName, sDescription, exceptionInfo);
  	}
  	else dynAppend(exceptionInfo, "Failed to set the description for WS object: " + sDeviceName);
  }
}

//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordStatus_checkConfig
/**
Check data before configuration

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWordStatus_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  if(strpos(dsConfigs[UN_CONFIG_COMMON_WIDGET], GCS_CONFIG_ANALOGSTATUS_DPT_NAME) >= 0)     // we have WS->AS
  {
    gcsConfigWordStatus_WSAScheckConfig(dsConfigs, exceptionInfo);
    return;
  }

  if(dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH)
  {
  // 1. Common
    unConfigGenericFunctions_checkParameters(dsConfigs, GCS_CONFIG_WORDSTATUS_DPT_NAME, exceptionInfo);
  // 2. call the device check function
    unConfigGenericFunctions_checkDeviceConfig(dsConfigs, GCS_CONFIG_WORDSTATUS_DPT_NAME, exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWordStatus_checkConfig: " + getCatStr("gcsGeneration","BADLENGTH"),"");
  }
}

//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordStatus_WSAScheckConfig
/**
Function prepare WS config string to be able to export as AS

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWordStatus_WSAScheckConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string description;
  int iAuPosRStPosition = UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_DESCRIPTION_AUPOSRST;


  if (dynlen(dsConfigs) >= iAuPosRStPosition) description = dsConfigs[iAuPosRStPosition];

  if(strpos(description, "archive") < 0)
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWordStatus_WSAScheckConfig: WS->AS " + getCatStr("unGeneration","BADINPUTS"),"");
    return;
  }
  dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGSTATUS_ADDRESS_AUPOSRST] = dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_ADDRESS_AUPOSRST];
  dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGSTATUS_ARCHIVE_TIME_FILTER] = 30;  //  dummy value
  gcsConfigAnalogStatus_checkConfig(dsConfigs, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_WordStatus_checkConfig
/**
Check data before configuration for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_WordStatus_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordStatus_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);

  unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH,
    makeDynInt(GCS_CONFIG_WORDSTATUS_ADDRESS_AUPOSRST),
    makeDynInt(),
    makeDynInt(GCS_CONFIG_WORDSTATUS_UNIT, GCS_CONFIG_WORDSTATUS_FORMAT, GCS_CONFIG_WORDSTATUS_DEADBAND, GCS_CONFIG_WORDSTATUS_DEADBAND_TYPE),
    makeDynInt(GCS_CONFIG_WORDSTATUS_RANGEMIN, GCS_CONFIG_WORDSTATUS_RANGEMAX),
    makeDynInt(), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_WordStatus_checkConfig
/**
Check data before configuration for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_WordStatus_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigWordStatus_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);

///@attention Do not call unConfigGenericFunctions_checkFloatIntBlockS7 function here! words are not treated by Unicos!!!
  unConfigGenericFunctions_checkAddressS7(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_ADDRESS_AUPOSRST], UN_CONFIG_S7_PLC_DATATYPE_WORD, exceptionInfo);

// 2. Common data & Range & Archive
  unConfigGenericFunctions_checkCommonFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH,
      makeDynInt(GCS_CONFIG_WORDSTATUS_UNIT, GCS_CONFIG_WORDSTATUS_FORMAT, GCS_CONFIG_WORDSTATUS_DEADBAND, GCS_CONFIG_WORDSTATUS_DEADBAND_TYPE),
      makeDynInt(GCS_CONFIG_WORDSTATUS_RANGEMIN, GCS_CONFIG_WORDSTATUS_RANGEMAX),
      makeDynInt(), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWordStatus_checkCommonConfig
/**
Check configuration of given WordStatus datapoint for both unPLc and S7_PLC front-end type

  @param[in]  sPlcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWordStatus_checkCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string str;
  string sDescription;
  int iRes = 0;

  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWordStatus_checkCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + sPlcType,"");
    return;
  }

  bool bHasArchive = false;
  if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_WORDSTATUS_LENGTH, dsConfigs, bHasArchive))
  {
    fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigWordStatus_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    return;
  }

  if (bHasArchive){
    gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_WORDSTATUS_LENGTH, dsConfigs, exceptionInfo);
    if (dynlen(exceptionInfo) != 0){
      fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigWordStatus_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
      return;
    }
  }

  if (dynlen(dsConfigs) >= (UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_ARCHIVE_ACTIVE))
  	unConfigGenericFunctions_checkArchiveParameters("", dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_ARCHIVE_ACTIVE], exceptionInfo);

  //Check the labels if any
  gcsConfigGenericFunctions_checkIntVariablesPairs(dsConfigs, UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_DESCRIPTION_AUPOSRST, exceptionInfo);

  //Description of WS contains Steppers descriptions in the following format:
  //	1=state1,2=state2,...
  //without any spaces.
  //In case there are spaces, they should be removed.
  sDescription = dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORDSTATUS_DESCRIPTION_AUPOSRST];
  iRes = strpos(sDescription, " ");
  if (iRes < -1) dynAppend(exceptionInfo, "Check of WS description has failed.");
}

//------------------------------------------------------------------------------------------------------------------------

// CMW_WordStatus_checkConfig
/**
Purpose: check data before configuration for CMW front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CMW_WordStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
	// 3. Float block
		unConfigGenericFunctions_checkCommonFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CMW_WORDSTATUS_UNIT, UN_CONFIG_CMW_WORDSTATUS_FORMAT, UN_CONFIG_CMW_WORDSTATUS_DEADBANDE, 
											UN_CONFIG_CMW_WORDSTATUS_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_CMW_WORDSTATUS_RANGEMIN, UN_CONFIG_CMW_WORDSTATUS_RANGEMAX),
				makeDynInt(UN_CONFIG_CMW_WORDSTATUS_ARCHIVE_ACTIVE, UN_CONFIG_CMW_WORDSTATUS_ARCHIVE_TIME_FILTER),
				exceptionInfo);

	// check group
//DebugN(configLine, configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_WORDSTATUS_TYPE], configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_WORDSTATUS_GROUP]);
		unConfigCMW_checkType(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_WORDSTATUS_TYPE], configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_WORDSTATUS_GROUP], exceptionInfo);
	
	// check transformation
		unConfigCMW_checkTransformationType(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_WORDSTATUS_TRANSFORMATION_TYPE], exceptionInfo);	
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CMW_WordStatus_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CMW_WordStatus_setConfig
/**
Purpose: set configuration of given Word Status datapoint for CMW front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CMW_WordStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bit32 b32;
	bool bLLC;
		
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_CMW_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnCMW_config);
// 2.3. Float values
	unConfigGenericFunctions_setCommonFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("AuPosRSt"),
					makeDynInt(UN_CONFIG_CMW_WORDSTATUS_UNIT, UN_CONFIG_CMW_WORDSTATUS_FORMAT, 
											UN_CONFIG_CMW_WORDSTATUS_DEADBANDE, UN_CONFIG_CMW_WORDSTATUS_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_CMW_WORDSTATUS_RANGEMIN, UN_CONFIG_CMW_WORDSTATUS_RANGEMAX),
					makeDynString("AuPosRSt"),
					makeDynInt(UN_CONFIG_CMW_WORDSTATUS_ARCHIVE_ACTIVE, UN_CONFIG_CMW_WORDSTATUS_ARCHIVE_TIME_FILTER),
					exceptionInfo);
	
	if(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_WORDSTATUS_ADDRESS_LLC] == "TRUE")
		bLLC = true;
	unConfigCMW_setAddress(sDpName + ".ProcessInput.AuPosRSt", 
													dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_WORDSTATUS_TYPE], 
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_WORDSTATUS_TRANSFORMATION_TYPE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_WORDSTATUS_GROUP],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_WORDSTATUS_ADDRESS_CMW_FILTERNAME],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_WORDSTATUS_ADDRESS_CMW_FILTERVALUE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_WORDSTATUS_ADDRESS_CMWDEVICE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_WORDSTATUS_ADDRESS_CMWPROPERTY],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_WORDSTATUS_ADDRESS_CMWTAG],
													bLLC,
													exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// SOFT_FE_WordStatus_setConfig
/**
Purpose: set configuration of given Word Status datapoint for SOFT_FE front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
SOFT_FE_WordStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName, sDPE, sDescription;
	bit32 b32;
	bool bLLC;
		
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnSOFT_FE_config);
// 2.3. Float values
	unConfigGenericFunctions_setCommonFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("AuPosRSt"),
					makeDynInt(),		
					makeDynInt(UN_CONFIG_SOFT_FE_WORDSTATUS_RANGEMIN, UN_CONFIG_SOFT_FE_WORDSTATUS_RANGEMAX),
					makeDynString("AuPosRSt"),
					makeDynInt(UN_CONFIG_SOFT_FE_WORDSTATUS_ARCHIVE_ACTIVE, UN_CONFIG_SOFT_FE_WORDSTATUS_ARCHIVE_TIME_FILTER),
					exceptionInfo);
	
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessInput.AuPosRSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_WORDSTATUS_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.AuPosRSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_WORDSTATUS_FORMAT], exceptionInfo);
  sDescription = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_WORDSTATUS_DESCRIPTION];
  sDPE = unGenericDpFunctions_getDpName(sDpName) + ".ProcessInput.AuPosRSt";
  if(sDescription == "")
  {
    sDescription = GCS_CONFIG_EMPTY_STRING;
  }
  unConfigGenericFunctions_setDescription(sDPE, sDescription, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// SOFT_FE_WordStatus_checkConfig
/**
Purpose: check data before configuration for SOFT_FE front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
SOFT_FE_WordStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_SOFT_FE_WORDSTATUS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
		unConfigGenericFunctions_checkUnitFormat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_WORDSTATUS_UNIT], 
												 configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_WORDSTATUS_FORMAT], exceptionInfo);
		unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_WORDSTATUS_RANGEMIN], 
														configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_WORDSTATUS_RANGEMAX], exceptionInfo);
		unConfigGenericFunctions_checkArchiveParameters(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_WORDSTATUS_ARCHIVE_TIME_FILTER], 
														configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_WORDSTATUS_ARCHIVE_ACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","SOFT_FE_WordStatus_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//@}
