/**@name LIBRARY: unConfigAlarm.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002


Modification History: 
  16/11/2008: Quique
       - Include the fields Master (PCO or field object) and the Type of alarm in the "Normal position" container.
         They are separated by commas: e.g.:  
         Alarm;$Record Number$;$Equipment$_$Location$_$Name;$Description$;$Diagnostics$;$Html Info$;$Synoptic$;$Domain$;$Nature$;$Instance Type$;0,$Master$,$Type$;
         
  02/06/2008: Herve
    - add archive setting 

	06/05/08:	 Quique
		- Include the range in the new alarm faceplate using: unGenericDpFunctions_setFaceplateRange
		
	18/04/2006: Herve
		- implementation of mask event

	14/06/2005: Enrique
		- S7 adaptation, new functions:
			. S7_PLC_Alarm_setConfig
			. S7_PLC_Alarm_checkConfig
		
		
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_Alarm_checkConfig, _UnPlc_Alarm_setConfig

version 1.0

External Function : 
	. unConfigAlarm_setConfig :  set configuration of given Alarm datapoint
	. unConfigAlarm_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos Alarm configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const unsigned UN_CONFIG_ALARM_LENGTH = 4;

const unsigned UN_CONFIG_ALARM_NORMAL_POSITION = 1;
const unsigned UN_CONFIG_ALARM_ADDRESS_STSREG01 = 2;
const unsigned UN_CONFIG_ALARM_ADDRESS_EVSTSREG01 = 3;
const unsigned UN_CONFIG_ALARM_ADDRESS_MANREG01 = 4;

const unsigned UN_CONFIG_ALARM_ADDITIONAL_LENGTH = 1;
const unsigned UN_CONFIG_ALARM_MASKEVENT = 1;

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigAlarm_setConfig
/**
Purpose:  set configuration of given Alarm datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigAlarm_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "Alarm", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "Alarm", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "Alarm", exceptionInfo);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigAlarm_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigAlarm_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	// 1. Common
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH) {
		unConfigGenericFunctions_checkParameters(configLine, "Alarm", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "Alarm", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigAlarm_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_Alarm_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_Alarm_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
//DebugN("_UnPlc_Alarm_checkConfig");
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ALARM_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ALARM_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ALARM_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ALARM_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ALARM_ADDRESS_STSREG01),
				makeDynInt(UN_CONFIG_ALARM_ADDRESS_EVSTSREG01),
				makeDynInt(UN_CONFIG_ALARM_ADDRESS_MANREG01),
				UN_CONFIG_ALARM_NORMAL_POSITION,
				exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_Alarm_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_Alarm_setConfig
/**
Purpose:  set configuration of given Alarm datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_Alarm_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sMask = "0";
	int iMask;
	string sDpName;
	string sTypeAlarm, sMasterAlarm,sNormalPosition; 
        dyn_string dsFullNormalPosition;
        
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

//DebugN("_UnPlc_Alarm_setConfig");
// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ALARM_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ALARM_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ALARM_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ALARM_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ALARM_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// Normal position is composed by: (1) Normal position definition, (2) Master, (3) Type of alarm separated by Commas.
// Recuperate Master and Type and let only Normal position definition:
   if (strpos(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ALARM_NORMAL_POSITION],",")>=0)
        {
           dsFullNormalPosition=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ALARM_NORMAL_POSITION],",");
           sNormalPosition=dsFullNormalPosition[1];
           sMasterAlarm=dsFullNormalPosition[2];
           // in case of empty fields the third field must be created manually (empty)
           if (dynlen(dsFullNormalPosition)==2)
               sTypeAlarm="";
           else
               sTypeAlarm= dsFullNormalPosition[3];
           
           // set back the normal position
           dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ALARM_NORMAL_POSITION]=sNormalPosition;
           
           // Set the additional information
           unGenericDpFunctions_changeDpParameter(sDpName, UN_PARAMETER_NAME, 
                                                  sMasterAlarm+","+sTypeAlarm, 
                                                  exceptionInfo);
        }
   // else : former case without additional information           
        
// 2.2. Registers
	unConfigGenericFunctions_setAllRegisters(dsConfigs,
			 makeDynString("StsReg01"),
			 makeDynInt(UN_CONFIG_ALARM_ADDRESS_STSREG01),
			 makeDynInt(UN_CONFIG_ALARM_ADDRESS_EVSTSREG01),
			 makeDynString("ManReg01"),
			 makeDynInt(UN_CONFIG_ALARM_ADDRESS_MANREG01),
			 makeDynString("PosAl"),
			 makeDynString(UN_STSREG01_POSST),
			 UN_CONFIG_ALARM_NORMAL_POSITION,
			 exceptionInfo);
        
	fwArchive_set(sDpName + ".ProcessInput.PosAl",
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
			DPATTR_ARCH_PROC_SIMPLESM,
			DPATTR_COMPARE_OLD_NEW,
			0,
			0,
			exceptionInfo);

	// Set faceplate trend range (equally to Alarm)
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString("PosAl"), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MAX), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MIN), exceptionInfo);
	
// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosAl"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_Alarm_checkConfig
/**
Purpose: check data before configuration for S7_PLC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: Win XP.
	. distributed system: yes.
*/

S7_PLC_Alarm_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	// DebugN("FUNCTION: S7_PLC_Alarm_checkConfig(configLine="+configLine);
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ALARM_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ALARM_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ALARM_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ALARM_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ALARM_ADDRESS_STSREG01),
				makeDynInt(UN_CONFIG_ALARM_ADDRESS_EVSTSREG01),
				makeDynInt(UN_CONFIG_ALARM_ADDRESS_MANREG01),
				UN_CONFIG_ALARM_NORMAL_POSITION,
				exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_Alarm_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_Alarm_setConfig
/**
Purpose:  set configuration of given Alarm datapoint for S7_PLC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP.
	. distributed system: yes.
*/

S7_PLC_Alarm_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sMask = "0";
	int iMask;
	string sDpName;
	string sTypeAlarm, sMasterAlarm,sNormalPosition; 
        dyn_string dsFullNormalPosition;
            
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
        // DebugN("FUNCTIONL::S7_PLC_Alarm_setConfig(dsConfigs=)",dsConfigs);
        // DebugN("UN_CONFIG_ALLCOMMON_LENGTH="+UN_CONFIG_ALLCOMMON_LENGTH);
        // DebugN("UN_CONFIG_ALARM_NORMAL_POSITION="+UN_CONFIG_ALARM_NORMAL_POSITION);
        // DebugN("dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ALARM_NORMAL_POSITION]=",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ALARM_NORMAL_POSITION]);
// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ALARM_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ALARM_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ALARM_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ALARM_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ALARM_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ALARM_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// Normal position is composed by: (1) Normal position definition, (2) Master, (3) Type of alarm separated by Commas.
// Recuperate Master and Type and let only Normal position definition:
   if (strpos(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ALARM_NORMAL_POSITION],",")>=0)
        {
           dsFullNormalPosition=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ALARM_NORMAL_POSITION],",");
           sNormalPosition=dsFullNormalPosition[1];
           sMasterAlarm=dsFullNormalPosition[2];
           // in case of empty fields the third field must be created manually (empty)
           if (dynlen(dsFullNormalPosition)==2)
               sTypeAlarm="";
           else
               sTypeAlarm= dsFullNormalPosition[3];
           
           // set back the normal position
           dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ALARM_NORMAL_POSITION]=sNormalPosition;
           
           // Set the additional information
           unGenericDpFunctions_changeDpParameter(sDpName, UN_PARAMETER_NAME, 
                                                  sMasterAlarm+","+sTypeAlarm, 
                                                  exceptionInfo);
        }
   // else : former case without additional information                  
        
// 2.2. Registers
	unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
			 makeDynString("StsReg01"),
			 makeDynInt(UN_CONFIG_ALARM_ADDRESS_STSREG01),
			 makeDynInt(UN_CONFIG_ALARM_ADDRESS_EVSTSREG01),
			 makeDynString("ManReg01"),
			 makeDynInt(UN_CONFIG_ALARM_ADDRESS_MANREG01),
			 makeDynString("PosAl"),
			 makeDynString(UN_STSREG01_POSST),
			 UN_CONFIG_ALARM_NORMAL_POSITION,
			 exceptionInfo);

	fwArchive_set(sDpName + ".ProcessInput.PosAl",
			  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
			  DPATTR_ARCH_PROC_SIMPLESM,
			  DPATTR_COMPARE_OLD_NEW,
			  0,
			  0,
			  exceptionInfo);

	// Set faceplate trend range (equally to Alarm)
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString("PosAl"), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MAX), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MIN), exceptionInfo);
	
// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosAl"), exceptionInfo);
}

//@}
