/**@name LIBRARY: unConfigAnaDig.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002

Modification History: 
	18/04/2006: Herve
		- implementation of mask event

	15/05/2005: Enrique
		- S7 adaptation, new functions:
			. S7_PLC_AnaDig_setConfig
			. S7_PLC_AnaDig_checkConfig
			
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_AnaDig_checkConfig, _UnPlc_AnaDig_setConfig

version 1.0

External Function : 
	. unConfigAnaDig_setConfig :  set configuration of given AnaDig datapoint
	. unConfigAnaDig_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos AnaDig configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const unsigned UN_CONFIG_ANADIG_LENGTH = 23;

const unsigned UN_CONFIG_ANADIG_UNIT = 1;
const unsigned UN_CONFIG_ANADIG_FORMAT = 2;
const unsigned UN_CONFIG_ANADIG_RANGEMAX = 3;
const unsigned UN_CONFIG_ANADIG_RANGEMIN = 4;
const unsigned UN_CONFIG_ANADIG_LIMITON = 5;
const unsigned UN_CONFIG_ANADIG_LIMITOFF = 6;
const unsigned UN_CONFIG_ANADIG_DEADBANDE = 7;
const unsigned UN_CONFIG_ANADIG_DEADBANDE_TYPE = 8;
const unsigned UN_CONFIG_ANADIG_ARCHIVE_ACTIVE = 9;
const unsigned UN_CONFIG_ANADIG_ARCHIVE_TIME_FILTER = 10;
const unsigned UN_CONFIG_ANADIG_NORMAL_POSITION = 11;
const unsigned UN_CONFIG_ANADIG_ADDRESS_STSREG01 = 12;
const unsigned UN_CONFIG_ANADIG_ADDRESS_STSREG02 = 13;
const unsigned UN_CONFIG_ANADIG_ADDRESS_EVSTSREG01 = 14;
const unsigned UN_CONFIG_ANADIG_ADDRESS_EVSTSREG02 = 15;
const unsigned UN_CONFIG_ANADIG_ADDRESS_POSST = 16;
const unsigned UN_CONFIG_ANADIG_ADDRESS_AUPOSRST = 17;
const unsigned UN_CONFIG_ANADIG_ADDRESS_MPOSRST = 18;
const unsigned UN_CONFIG_ANADIG_ADDRESS_OUTOVST = 19;
const unsigned UN_CONFIG_ANADIG_ADDRESS_MANREG01 = 20;
const unsigned UN_CONFIG_ANADIG_ADDRESS_MPOSR = 21;
const unsigned UN_CONFIG_ANADIG_ADDRESS_PLION = 22;
const unsigned UN_CONFIG_ANADIG_ADDRESS_PLIOFF = 23;

const unsigned UN_CONFIG_ANADIG_ADDITIONAL_LENGTH = 1;
const unsigned UN_CONFIG_ANADIG_MASKEVENT = 1;

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigAnaDig_setConfig
/**
Purpose:  set configuration of given AnaDig datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigAnaDig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "AnaDig", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "AnaDig", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "AnaDig", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigAnaDig_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigAnaDig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "AnaDig", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "AnaDig", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigAnaDig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_AnaDig_setConfig
/**
Purpose:  set configuration of given AnaDig datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_AnaDig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANADIG_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANADIG_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANADIG_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegisters(dsConfigs,
			 makeDynString("StsReg01","StsReg02"),
			 makeDynInt(UN_CONFIG_ANADIG_ADDRESS_STSREG01, UN_CONFIG_ANADIG_ADDRESS_STSREG02),
			 makeDynInt(UN_CONFIG_ANADIG_ADDRESS_EVSTSREG01, UN_CONFIG_ANADIG_ADDRESS_EVSTSREG02),
			 makeDynString("ManReg01"),
			 makeDynInt(UN_CONFIG_ANADIG_ADDRESS_MANREG01),
			 makeDynString("StartISt;StopISt"),
			 makeDynString(UN_STSREG01_STARTIST + ";" + UN_STSREG01_STOPIST),
			 UN_CONFIG_ANADIG_NORMAL_POSITION,
			 exceptionInfo);
// 2.3. Float values
	unConfigGenericFunctions_setFloatIntBlock(dsConfigs,
			PVSS_MODBUS_FLOAT,
			makeDynString("PosSt", "OutOVSt", "AuPosRSt", "MPosRSt"),
			makeDynInt(UN_CONFIG_ANADIG_ADDRESS_POSST, UN_CONFIG_ANADIG_ADDRESS_OUTOVST, UN_CONFIG_ANADIG_ADDRESS_AUPOSRST, UN_CONFIG_ANADIG_ADDRESS_MPOSRST),
			makeDynString("MPosR", "PLiOn", "PLiOff"),
			makeDynInt(UN_CONFIG_ANADIG_ADDRESS_MPOSR, UN_CONFIG_ANADIG_ADDRESS_PLION, UN_CONFIG_ANADIG_ADDRESS_PLIOFF),
			makeDynInt(UN_CONFIG_ANADIG_UNIT, UN_CONFIG_ANADIG_FORMAT, UN_CONFIG_ANADIG_DEADBANDE, UN_CONFIG_ANADIG_DEADBANDE_TYPE),		
			makeDynInt(UN_CONFIG_ANADIG_RANGEMIN, UN_CONFIG_ANADIG_RANGEMAX),
			makeDynString("PosSt","OutOVSt"),
			makeDynInt(UN_CONFIG_ANADIG_ARCHIVE_ACTIVE, UN_CONFIG_ANADIG_ARCHIVE_TIME_FILTER),
			exceptionInfo);
// 2.4. Particular dpes : PLiOn & PLiOff -> unit + format + value for ProcessOutput values
	dpSet(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_LIMITON]);
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_FORMAT], exceptionInfo);
	dpSet(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_LIMITOFF]);	
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_FORMAT], exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																								".ProcessInput.StartISt", ".ProcessInput.StopISt"), exceptionInfo);
} 

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_AnaDig_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_AnaDig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANADIG_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANADIG_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANADIG_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANADIG_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANADIG_ADDRESS_STSREG01, UN_CONFIG_ANADIG_ADDRESS_STSREG02),
				makeDynInt(UN_CONFIG_ANADIG_ADDRESS_EVSTSREG01, UN_CONFIG_ANADIG_ADDRESS_EVSTSREG02),
				makeDynInt(UN_CONFIG_ANADIG_ADDRESS_MANREG01),
				UN_CONFIG_ANADIG_NORMAL_POSITION,
				exceptionInfo);
	// 3. Float block
		unConfigGenericFunctions_checkFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANADIG_ADDRESS_POSST, UN_CONFIG_ANADIG_ADDRESS_OUTOVST, UN_CONFIG_ANADIG_ADDRESS_AUPOSRST, UN_CONFIG_ANADIG_ADDRESS_MPOSRST),
				makeDynInt(UN_CONFIG_ANADIG_ADDRESS_MPOSR, UN_CONFIG_ANADIG_ADDRESS_PLION, UN_CONFIG_ANADIG_ADDRESS_PLIOFF), 
				makeDynInt(UN_CONFIG_ANADIG_UNIT, UN_CONFIG_ANADIG_FORMAT, UN_CONFIG_ANADIG_DEADBANDE, UN_CONFIG_ANADIG_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_ANADIG_RANGEMIN, UN_CONFIG_ANADIG_RANGEMAX),
				makeDynInt(UN_CONFIG_ANADIG_ARCHIVE_ACTIVE, UN_CONFIG_ANADIG_ARCHIVE_TIME_FILTER),
				exceptionInfo);
	// 4. Particular dpes: Limit on // Limit off
		unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANADIG_LIMITOFF], 
										    configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANADIG_LIMITON], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_AnaDig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_AnaDig_setConfig
/**
Purpose:  set configuration of given AnaDig datapoint for S7_PLC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: Win XP
	. distributed system: yes.
*/

S7_PLC_AnaDig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
		
	// DebugN("FUNCTION: S7_PLC_AnaDig_setConfig(dsConfigs= "+dsConfigs);
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANADIG_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANADIG_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANADIG_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
			 makeDynString("StsReg01","StsReg02"),
			 makeDynInt(UN_CONFIG_ANADIG_ADDRESS_STSREG01, UN_CONFIG_ANADIG_ADDRESS_STSREG02),
			 makeDynInt(UN_CONFIG_ANADIG_ADDRESS_EVSTSREG01, UN_CONFIG_ANADIG_ADDRESS_EVSTSREG02),
			 makeDynString("ManReg01"),
			 makeDynInt(UN_CONFIG_ANADIG_ADDRESS_MANREG01),
			 makeDynString("StartISt;StopISt"),
			 makeDynString(UN_STSREG01_STARTIST + ";" + UN_STSREG01_STOPIST),
			 UN_CONFIG_ANADIG_NORMAL_POSITION,
			 exceptionInfo);
// 2.3. Float values
	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,
			PVSS_S7_FLOAT,
			makeDynString("PosSt", "OutOVSt", "AuPosRSt", "MPosRSt"),
			makeDynInt(UN_CONFIG_ANADIG_ADDRESS_POSST, UN_CONFIG_ANADIG_ADDRESS_OUTOVST, UN_CONFIG_ANADIG_ADDRESS_AUPOSRST, UN_CONFIG_ANADIG_ADDRESS_MPOSRST),
			makeDynString("MPosR", "PLiOn", "PLiOff"),
			makeDynInt(UN_CONFIG_ANADIG_ADDRESS_MPOSR, UN_CONFIG_ANADIG_ADDRESS_PLION, UN_CONFIG_ANADIG_ADDRESS_PLIOFF),
			makeDynInt(UN_CONFIG_ANADIG_UNIT, UN_CONFIG_ANADIG_FORMAT, UN_CONFIG_ANADIG_DEADBANDE, UN_CONFIG_ANADIG_DEADBANDE_TYPE),		
			makeDynInt(UN_CONFIG_ANADIG_RANGEMIN, UN_CONFIG_ANADIG_RANGEMAX),
			makeDynString("PosSt","OutOVSt"),
			makeDynInt(UN_CONFIG_ANADIG_ARCHIVE_ACTIVE, UN_CONFIG_ANADIG_ARCHIVE_TIME_FILTER),
			exceptionInfo);
// 2.4. Particular dpes : PLiOn & PLiOff -> unit + format + value for ProcessOutput values
	dpSet(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_LIMITON]);
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_FORMAT], exceptionInfo);
	dpSet(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_LIMITOFF]);	
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANADIG_FORMAT], exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																								".ProcessInput.StartISt", ".ProcessInput.StopISt"), 
																					exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_AnaDig_checkConfig
/**
Purpose: check data before configuration for S7_PLC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP
	. distributed system: yes.
*/

S7_PLC_AnaDig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	// DebugN("FUNCTION: S7_PLC_ANADIG_checkConfig(configLine="+configLine);
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANADIG_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANADIG_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANADIG_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANADIG_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ANADIG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANADIG_ADDRESS_STSREG01, UN_CONFIG_ANADIG_ADDRESS_STSREG02),
				makeDynInt(UN_CONFIG_ANADIG_ADDRESS_EVSTSREG01, UN_CONFIG_ANADIG_ADDRESS_EVSTSREG02),
				makeDynInt(UN_CONFIG_ANADIG_ADDRESS_MANREG01),
				UN_CONFIG_ANADIG_NORMAL_POSITION,
				exceptionInfo);
	// 3. Float block
		unConfigGenericFunctions_checkFloatIntBlockS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANADIG_ADDRESS_POSST, UN_CONFIG_ANADIG_ADDRESS_OUTOVST, UN_CONFIG_ANADIG_ADDRESS_AUPOSRST, UN_CONFIG_ANADIG_ADDRESS_MPOSRST),
				makeDynInt(UN_CONFIG_ANADIG_ADDRESS_MPOSR, UN_CONFIG_ANADIG_ADDRESS_PLION, UN_CONFIG_ANADIG_ADDRESS_PLIOFF), 
				makeDynInt(UN_CONFIG_ANADIG_UNIT, UN_CONFIG_ANADIG_FORMAT, UN_CONFIG_ANADIG_DEADBANDE, UN_CONFIG_ANADIG_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_ANADIG_RANGEMIN, UN_CONFIG_ANADIG_RANGEMAX),
				makeDynInt(UN_CONFIG_ANADIG_ARCHIVE_ACTIVE, UN_CONFIG_ANADIG_ARCHIVE_TIME_FILTER),
				exceptionInfo);
	// 4. Particular dpes: Limit on // Limit off
		unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANADIG_LIMITOFF], 
										    configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANADIG_LIMITON], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_AnaDig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//------------------------------------------------------------------------------------------------------------------------

//@}