/**@name LIBRARY: unConfigAnalog.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002

Modification History: 
        20/10/2008:Quique
            - Analog object may have double definition of the stsReg01, evStsReg01 in some former cases. New generations will
              have allways the doubled definition. IN the fields *StsREg01 there will be two addresses separated by ","
              
	18/04/2006: Herve
		- implementation of mask event

	14/06/2005: Enrique
		- S7 adaptation, new functions:
			. S7_PLC_Analog_checkConfig
			. S7_PLC_Analog_setConfig
		
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_Analog_checkConfig, _UnPlc_Analog_setConfig

version 1.0

External Function : 
	. unConfigAnalog_setConfig :  set configuration of given Analog datapoint
	. unConfigAnalog_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos Analog configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const unsigned UN_CONFIG_ANALOG_LENGTH = 21;

const unsigned UN_CONFIG_ANALOG_UNIT = 1;
const unsigned UN_CONFIG_ANALOG_FORMAT = 2;
const unsigned UN_CONFIG_ANALOG_RANGEMAX = 3;
const unsigned UN_CONFIG_ANALOG_RANGEMIN = 4;
const unsigned UN_CONFIG_ANALOG_LIMITON = 5;
const unsigned UN_CONFIG_ANALOG_LIMITOFF = 6;
const unsigned UN_CONFIG_ANALOG_DEADBANDE = 7;
const unsigned UN_CONFIG_ANALOG_DEADBANDE_TYPE = 8;
const unsigned UN_CONFIG_ANALOG_ARCHIVE_ACTIVE = 9;
const unsigned UN_CONFIG_ANALOG_ARCHIVE_TIME_FILTER = 10;
const unsigned UN_CONFIG_ANALOG_NORMAL_POSITION = 11;
const unsigned UN_CONFIG_ANALOG_ADDRESS_STSREG01 = 12;
const unsigned UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01 = 13;
const unsigned UN_CONFIG_ANALOG_ADDRESS_POSST = 14;
const unsigned UN_CONFIG_ANALOG_ADDRESS_AUPOSRST = 15;
const unsigned UN_CONFIG_ANALOG_ADDRESS_MPOSRST = 16;
const unsigned UN_CONFIG_ANALOG_ADDRESS_OUTOVST = 17;
const unsigned UN_CONFIG_ANALOG_ADDRESS_MANREG01 = 18;
const unsigned UN_CONFIG_ANALOG_ADDRESS_MPOSR = 19;
const unsigned UN_CONFIG_ANALOG_ADDRESS_PLION = 20;
const unsigned UN_CONFIG_ANALOG_ADDRESS_PLIOFF = 21;

const unsigned UN_CONFIG_ANALOG_ADDITIONAL_LENGTH = 1;
const unsigned UN_CONFIG_ANALOG_MASKEVENT = 1;

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigAnalog_setConfig
/**
Purpose:  set configuration of given Analog datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigAnalog_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
       // DebugN("FUNCTION: unConfigAnalog_setConfig(dsConfigs= "+dsConfigs);
         
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "Analog", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "Analog", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "Analog", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigAnalog_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigAnalog_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
       // DebugN("FUNCTION: unConfigAnalog_checkConfig(configLine= "+configLine);
        
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "Analog", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "Analog", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigAnalog_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_Analog_setConfig
/**
Purpose:  set configuration of given Analog datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_Analog_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
	dyn_string dsDoubleStsReg01, dsDoubleEvStsReg01;
        
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOG_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOG_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOG_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
        if (strpos(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ANALOG_ADDRESS_STSREG01],",")<0)  // former case
    	  unConfigGenericFunctions_setAllRegisters(dsConfigs,
			 makeDynString("StsReg01"),
			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
			 makeDynString("ManReg01"),
			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MANREG01),
			 makeDynString("StartISt;StopISt"),
			 makeDynString(UN_STSREG01_STARTIST + ";" + UN_STSREG01_STOPIST),
			 UN_CONFIG_ANALOG_NORMAL_POSITION,
			 exceptionInfo);
        else  // Double definition of the stsReg, evStsReg
          {
              // DebugN("--> doubled stsReg definition: ",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01]);
              
              // Extract the double definition for StsReg, EvStsReg 
              dsDoubleStsReg01=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01],",");
              dsDoubleEvStsReg01=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01],",");
              
              // Call for StsReg02, evStsReg02
              dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01]=dsDoubleStsReg01[2];
              dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01]=dsDoubleEvStsReg01[2];
              // DebugN("--> dsDoubleStsReg01=",dsDoubleStsReg01);
              // DebugN("--> dsDoubleEvStsReg01=",dsDoubleEvStsReg01);
              // DebugN("--> <StsReg02>- dsConfigs= "+dsConfigs);
              unConfigGenericFunctions_setAllRegisters(dsConfigs,
        			 makeDynString("StsReg02"),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
        			 makeDynString(),
        			 makeDynInt(),
        			 makeDynString(),
        			 makeDynString(),
        			 0,
        			 exceptionInfo);
                  
              // Call for StsReg01, evStsReg01
              dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01]=dsDoubleStsReg01[1];
              dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01]=dsDoubleEvStsReg01[1];
              // DebugN("--> <StsReg01>- dsConfigs= "+dsConfigs);
              unConfigGenericFunctions_setAllRegisters(dsConfigs,
        			 makeDynString("StsReg01"),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
        			 makeDynString("ManReg01"),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MANREG01),
        			 makeDynString("StartISt;StopISt"),
        			 makeDynString(UN_STSREG01_STARTIST + ";" + UN_STSREG01_STOPIST),
        			 UN_CONFIG_ANALOG_NORMAL_POSITION,
        			 exceptionInfo);
          }
        
// 2.3. Float values
	unConfigGenericFunctions_setFloatIntBlock(dsConfigs,
			PVSS_MODBUS_FLOAT,
			makeDynString("PosSt", "OutOVSt", "AuPosRSt", "MPosRSt"),
			makeDynInt(UN_CONFIG_ANALOG_ADDRESS_POSST, UN_CONFIG_ANALOG_ADDRESS_OUTOVST, UN_CONFIG_ANALOG_ADDRESS_AUPOSRST, UN_CONFIG_ANALOG_ADDRESS_MPOSRST),
			makeDynString("MPosR", "PLiOn", "PLiOff"),
			makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MPOSR, UN_CONFIG_ANALOG_ADDRESS_PLION, UN_CONFIG_ANALOG_ADDRESS_PLIOFF),
			makeDynInt(UN_CONFIG_ANALOG_UNIT, UN_CONFIG_ANALOG_FORMAT, UN_CONFIG_ANALOG_DEADBANDE, UN_CONFIG_ANALOG_DEADBANDE_TYPE),		
			makeDynInt(UN_CONFIG_ANALOG_RANGEMIN, UN_CONFIG_ANALOG_RANGEMAX),
			makeDynString("PosSt","OutOVSt"),
			makeDynInt(UN_CONFIG_ANALOG_ARCHIVE_ACTIVE, UN_CONFIG_ANALOG_ARCHIVE_TIME_FILTER),
			exceptionInfo);
// 2.4. Particular dpes : PLiOn & PLiOff -> unit + format + value for ProcessOutput values
	dpSet(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_LIMITON]);
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_FORMAT], exceptionInfo);
	dpSet(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_LIMITOFF]);	
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_FORMAT], exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																								".ProcessInput.StartISt", ".ProcessInput.StopISt"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_Analog_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_Analog_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	dyn_string dsDoubleStsReg01, dsDoubleEvStsReg01;
        
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOG_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOG_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
                if (strpos(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_ANALOG_ADDRESS_STSREG01],",")<0)  // former case 
        	  unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MANREG01),
        				UN_CONFIG_ANALOG_NORMAL_POSITION,
        				exceptionInfo);
                else
                  // Double StsReg01, evStsReg01 then replace the contents of the stsReg01
                  {
                      // DebugN("doubled stsReg definition: ",configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_ANALOG_ADDRESS_STSREG01]);
                      // Extract the double definition for StsReg, EvStsReg 
                      dsDoubleStsReg01=strsplit(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01],",");
                      dsDoubleEvStsReg01=strsplit(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01],",");
                  
                      // Call for StsReg01, evStsReg01
                      configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01]=dsDoubleStsReg01[1];
                      configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01]=dsDoubleEvStsReg01[1];
                      unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MANREG01),
                                        				UN_CONFIG_ANALOG_NORMAL_POSITION,
                                        				exceptionInfo);
                      // Call for StsReg02,evStsReg02
                      configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01]=dsDoubleStsReg01[2];
                      configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01]=dsDoubleEvStsReg01[2];
                      unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MANREG01),
                                        				UN_CONFIG_ANALOG_NORMAL_POSITION,
                                        				exceptionInfo);
                  }
                  
		unConfigGenericFunctions_checkFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_POSST, UN_CONFIG_ANALOG_ADDRESS_OUTOVST, UN_CONFIG_ANALOG_ADDRESS_AUPOSRST, UN_CONFIG_ANALOG_ADDRESS_MPOSRST),
				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MPOSR, UN_CONFIG_ANALOG_ADDRESS_PLION, UN_CONFIG_ANALOG_ADDRESS_PLIOFF), 
				makeDynInt(UN_CONFIG_ANALOG_UNIT, UN_CONFIG_ANALOG_FORMAT, UN_CONFIG_ANALOG_DEADBANDE, UN_CONFIG_ANALOG_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_ANALOG_RANGEMIN, UN_CONFIG_ANALOG_RANGEMAX),
				makeDynInt(UN_CONFIG_ANALOG_ARCHIVE_ACTIVE, UN_CONFIG_ANALOG_ARCHIVE_TIME_FILTER),
				exceptionInfo);
	// 4. Particular dpes: Limit on // Limit off
		unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_LIMITOFF], 
										    configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_LIMITON], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_Analog_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}
//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_Analog_setConfig
/**
Purpose:  set configuration of given Analog datapoint for S7_PLC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

S7_PLC_Analog_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
	dyn_string dsDoubleStsReg01, dsDoubleEvStsReg01;
        
	// DebugN("FUNCTION: S7_PLC_Analog_setConfig(dsConfigs= "+dsConfigs);
       
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOG_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOG_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOG_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
        // DebugN("--> dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ANALOG_ADDRESS_STSREG01]=",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ANALOG_ADDRESS_STSREG01]);
        if (strpos(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_ANALOG_ADDRESS_STSREG01],",")<0)  // former case
        	unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
        			 makeDynString("StsReg01"),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
        			 makeDynString("ManReg01"),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MANREG01),
        			 makeDynString("StartISt;StopISt"),
        			 makeDynString(UN_STSREG01_STARTIST + ";" + UN_STSREG01_STOPIST),
        			 UN_CONFIG_ANALOG_NORMAL_POSITION,
        			 exceptionInfo);
        else  // Double definition of the stsReg, evStsReg
          {
              // DebugN("--> doubled stsReg definition: ",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01]);
              
              // Extract the double definition for StsReg, EvStsReg 
              dsDoubleStsReg01=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01],",");
              dsDoubleEvStsReg01=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01],",");
              // DebugN("--> dsDoubleStsReg01=",dsDoubleStsReg01);
              // DebugN("--> dsDoubleEvStsReg01=",dsDoubleEvStsReg01);
                           
              // Call for StsReg02, evStsReg02
              dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01]=dsDoubleStsReg01[2];
              dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01]=dsDoubleEvStsReg01[2];
 
              // DebugN("--> <StsReg02>- dsConfigs= "+dsConfigs);
              unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
        			 makeDynString("StsReg02"),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
        			 makeDynString(),
        			 makeDynInt(),
        			 makeDynString(),
        			 makeDynString(),
        			 0,
        			 exceptionInfo);
                  
              // Call for StsReg01, evStsReg01
              dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01]=dsDoubleStsReg01[1];
              dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01]=dsDoubleEvStsReg01[1];

              // DebugN("--> <StsReg01>- dsConfigs= "+dsConfigs);
              unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
        			 makeDynString("StsReg01"),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
        			 makeDynString("ManReg01"),
        			 makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MANREG01),
        			 makeDynString("StartISt;StopISt"),
        			 makeDynString(UN_STSREG01_STARTIST + ";" + UN_STSREG01_STOPIST),
        			 UN_CONFIG_ANALOG_NORMAL_POSITION,
        			 exceptionInfo);  
          }
            
// 2.3. Float values
	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,
			PVSS_S7_FLOAT,
			makeDynString("PosSt", "OutOVSt", "AuPosRSt", "MPosRSt"),
			makeDynInt(UN_CONFIG_ANALOG_ADDRESS_POSST, UN_CONFIG_ANALOG_ADDRESS_OUTOVST, UN_CONFIG_ANALOG_ADDRESS_AUPOSRST, UN_CONFIG_ANALOG_ADDRESS_MPOSRST),
			makeDynString("MPosR", "PLiOn", "PLiOff"),
			makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MPOSR, UN_CONFIG_ANALOG_ADDRESS_PLION, UN_CONFIG_ANALOG_ADDRESS_PLIOFF),
			makeDynInt(UN_CONFIG_ANALOG_UNIT, UN_CONFIG_ANALOG_FORMAT, UN_CONFIG_ANALOG_DEADBANDE, UN_CONFIG_ANALOG_DEADBANDE_TYPE),		
			makeDynInt(UN_CONFIG_ANALOG_RANGEMIN, UN_CONFIG_ANALOG_RANGEMAX),
			makeDynString("PosSt","OutOVSt"),
			makeDynInt(UN_CONFIG_ANALOG_ARCHIVE_ACTIVE, UN_CONFIG_ANALOG_ARCHIVE_TIME_FILTER),
			exceptionInfo);
// 2.4. Particular dpes : PLiOn & PLiOff -> unit + format + value for ProcessOutput values
	dpSet(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_LIMITON]);
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_FORMAT], exceptionInfo);
	dpSet(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_LIMITOFF]);	
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOG_FORMAT], exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
					      makeDynString(".ProcessInput.evStsReg01",".ProcessInput.evStsReg02",
                                                            ".ProcessInput.StartISt", ".ProcessInput.StopISt"), 
                                              exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_Analog_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

S7_PLC_Analog_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	// DebugN("FUNCTION: S7_PLC_ANALOG_checkConfig(configLine="+configLine);
  
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
        dyn_string dsDoubleStsReg01, dsDoubleEvStsReg01;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOG_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOG_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ANALOG_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
                // DebugN("UN_CONFIG_ALLCOMMON_LENGTH=",UN_CONFIG_ALLCOMMON_LENGTH);
                // DebugN("UN_CONFIG_COMMON_LENGTH=",UN_CONFIG_COMMON_LENGTH);
                if (strpos(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_ANALOG_ADDRESS_STSREG01],",")<0)  // former case
                  unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
                                				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
                                				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
                                				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MANREG01),
                                				UN_CONFIG_ANALOG_NORMAL_POSITION,
                                				exceptionInfo);
                else  // Double StsReg01, evStsReg01 then replace the contents of the stsReg01
                  {
                      // DebugN("doubled stsReg definition: ",configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_ANALOG_ADDRESS_STSREG01]);
                      // Extract the double definition for StsReg, EvStsReg 
                      dsDoubleStsReg01=strsplit(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01],",");
                      dsDoubleEvStsReg01=strsplit(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01],",");
                  
                      // Call for StsReg01, evStsReg01
                      configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01]=dsDoubleStsReg01[1];
                      configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01]=dsDoubleEvStsReg01[1];
                      unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MANREG01),
                                        				UN_CONFIG_ANALOG_NORMAL_POSITION,
                                        				exceptionInfo);
                      // Call for StsReg02,evStsReg02
                      configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_STSREG01]=dsDoubleStsReg01[2];
                      configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01]=dsDoubleEvStsReg01[2];
                      unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_STSREG01),
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_EVSTSREG01),
                                        				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MANREG01),
                                        				UN_CONFIG_ANALOG_NORMAL_POSITION,
                                        				exceptionInfo);
                  }
                
	// 3. Float block
		unConfigGenericFunctions_checkFloatIntBlockS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_POSST, UN_CONFIG_ANALOG_ADDRESS_OUTOVST, UN_CONFIG_ANALOG_ADDRESS_AUPOSRST, UN_CONFIG_ANALOG_ADDRESS_MPOSRST),
				makeDynInt(UN_CONFIG_ANALOG_ADDRESS_MPOSR, UN_CONFIG_ANALOG_ADDRESS_PLION, UN_CONFIG_ANALOG_ADDRESS_PLIOFF), 
				makeDynInt(UN_CONFIG_ANALOG_UNIT, UN_CONFIG_ANALOG_FORMAT, UN_CONFIG_ANALOG_DEADBANDE, UN_CONFIG_ANALOG_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_ANALOG_RANGEMIN, UN_CONFIG_ANALOG_RANGEMAX),
				makeDynInt(UN_CONFIG_ANALOG_ARCHIVE_ACTIVE, UN_CONFIG_ANALOG_ARCHIVE_TIME_FILTER),
				exceptionInfo);
	// 4. Particular dpes: Limit on // Limit off
		unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_LIMITOFF], 
										    configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOG_LIMITON], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_Analog_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------
//@}
