/**@name LIBRARY: unConfigAnalogOutput.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002

Modification History: 
	18/04/2006: Herve
		- implementation of mask event

	10/06/05: Enrique 
		- new function: S7_PLC_AnalogOutput_checkConfig to check the S7_PLC configuration
		- new function: S7_PLC_AnalogOutput_setConfig to set the S7_PLC configuration 
		
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_AnalogOutput_checkConfig, _UnPlc_AnalogOutput_setConfig

version 1.0

External Function : 
	. unConfigAnalogOutput_setConfig :  set configuration of given Analog Output datapoint
	. unConfigAnalogOutput_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos Analog Output configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const unsigned UN_CONFIG_ANALOGOUTPUT_LENGTH = 19;

const unsigned UN_CONFIG_ANALOGOUTPUT_UNIT = 1;
const unsigned UN_CONFIG_ANALOGOUTPUT_FORMAT = 2;
const unsigned UN_CONFIG_ANALOGOUTPUT_RANGEMAX = 3;
const unsigned UN_CONFIG_ANALOGOUTPUT_RANGEMIN = 4;
const unsigned UN_CONFIG_ANALOGOUTPUT_HHLIMIT = 5;
const unsigned UN_CONFIG_ANALOGOUTPUT_HLIMIT = 6;
const unsigned UN_CONFIG_ANALOGOUTPUT_LLIMIT = 7;
const unsigned UN_CONFIG_ANALOGOUTPUT_LLLIMIT = 8;
const unsigned UN_CONFIG_ANALOGOUTPUT_DEADBANDE = 9;
const unsigned UN_CONFIG_ANALOGOUTPUT_DEADBANDE_TYPE = 10;
const unsigned UN_CONFIG_ANALOGOUTPUT_ARCHIVE_ACTIVE = 11;
const unsigned UN_CONFIG_ANALOGOUTPUT_ARCHIVE_TIME_FILTER = 12;
const unsigned UN_CONFIG_ANALOGOUTPUT_NORMAL_POSITION = 13;
const unsigned UN_CONFIG_ANALOGOUTPUT_ADDRESS_STSREG01 = 14;
const unsigned UN_CONFIG_ANALOGOUTPUT_ADDRESS_EVSTSREG01 = 15;
const unsigned UN_CONFIG_ANALOGOUTPUT_ADDRESS_POSST = 16;
const unsigned UN_CONFIG_ANALOGOUTPUT_ADDRESS_AUPOSRST = 17;
const unsigned UN_CONFIG_ANALOGOUTPUT_ADDRESS_MANREG01 = 18;
const unsigned UN_CONFIG_ANALOGOUTPUT_ADDRESS_MPOSR = 19;

const unsigned UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH = 1;
const unsigned UN_CONFIG_ANALOGOUTPUT_MASKEVENT = 1;

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigAnalogOutput_setConfig
/**
Purpose: set configuration of given Analog Output datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigAnalogOutput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "AnalogOutput", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "AnalogOutput", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "AnalogOutput", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigAnalogOutput_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigAnalogOutput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "AnalogOutput", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "AnalogOutput", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigAnalogOutput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_AnalogOutput_setConfig
/**
Purpose: set configuration of given Analog Output datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_AnalogOutput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOGOUTPUT_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGOUTPUT_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_STSREG01),
					 makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_EVSTSREG01),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_MANREG01),
					 makeDynString(),
					 makeDynString(),
					 -1,
					 exceptionInfo);
// 2.3. Float values
	unConfigGenericFunctions_setFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("PosSt", "HFStOrAuPosRSt"),
					makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_POSST, UN_CONFIG_ANALOGOUTPUT_ADDRESS_AUPOSRST),
					makeDynString("MPosR"),
					makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_MPOSR),
					makeDynInt(UN_CONFIG_ANALOGOUTPUT_UNIT, UN_CONFIG_ANALOGOUTPUT_FORMAT, UN_CONFIG_ANALOGOUTPUT_DEADBANDE, UN_CONFIG_ANALOGOUTPUT_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_ANALOGOUTPUT_RANGEMIN, UN_CONFIG_ANALOGOUTPUT_RANGEMAX),
					makeDynString("PosSt"),
					makeDynInt(UN_CONFIG_ANALOGOUTPUT_ARCHIVE_ACTIVE, UN_CONFIG_ANALOGOUTPUT_ARCHIVE_TIME_FILTER),
					exceptionInfo);
	unConfigGenericFunctions_set5RangesAlert(dsConfigs, sDpName + ".ProcessInput.PosSt", "Position Status", UN_CONFIG_ANALOGOUTPUT_NORMAL_POSITION,
													 makeDynInt(UN_CONFIG_ANALOGOUTPUT_LLLIMIT, UN_CONFIG_ANALOGOUTPUT_LLIMIT, 
													 			UN_CONFIG_ANALOGOUTPUT_HLIMIT, UN_CONFIG_ANALOGOUTPUT_HHLIMIT),
													 exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_AnalogOutput_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_AnalogOutput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOGOUTPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGOUTPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_STSREG01),
				makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_EVSTSREG01),
				makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_MANREG01),
				-1,
				exceptionInfo);
	// 3. Float block
		unConfigGenericFunctions_checkFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(	UN_CONFIG_ANALOGOUTPUT_ADDRESS_POSST, 
										UN_CONFIG_ANALOGOUTPUT_ADDRESS_AUPOSRST),
				makeDynInt(	UN_CONFIG_ANALOGOUTPUT_ADDRESS_MPOSR), 
				makeDynInt(	UN_CONFIG_ANALOGOUTPUT_UNIT, 
										UN_CONFIG_ANALOGOUTPUT_FORMAT, 
										UN_CONFIG_ANALOGOUTPUT_DEADBANDE, 
										UN_CONFIG_ANALOGOUTPUT_DEADBANDE_TYPE),		
				makeDynInt(	UN_CONFIG_ANALOGOUTPUT_RANGEMIN, 
										UN_CONFIG_ANALOGOUTPUT_RANGEMAX),
				makeDynInt(	UN_CONFIG_ANALOGOUTPUT_ARCHIVE_ACTIVE, 
										UN_CONFIG_ANALOGOUTPUT_ARCHIVE_TIME_FILTER),
				exceptionInfo);
	// 4. Particular : limits
		unConfigGenericFunctions_check5RangesAlert(configLine, 	UN_CONFIG_ANALOGOUTPUT_NORMAL_POSITION,
												   											makeDynInt(	UN_CONFIG_ANALOGOUTPUT_LLLIMIT, 
												   																	UN_CONFIG_ANALOGOUTPUT_LLIMIT, 
												 			  														UN_CONFIG_ANALOGOUTPUT_HLIMIT, 
												 			  														UN_CONFIG_ANALOGOUTPUT_HHLIMIT),
												   											exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_AnalogOutput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_AnalogOutput_checkConfig
/**
Purpose: check data before configuration for S7_PLC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP.
	. distributed system: yes.
*/

S7_PLC_AnalogOutput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	// DebugN("FUNCTION: S7_PLC_ANALOGOUTPUT_checkConfig(configLine="+configLine);
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOGOUTPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGOUTPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_STSREG01),
				makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_EVSTSREG01),
				makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_MANREG01),
				-1,
				exceptionInfo);
	// 3. Float block	
		unConfigGenericFunctions_checkFloatIntBlockS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt( UN_CONFIG_ANALOGOUTPUT_ADDRESS_POSST, 
										UN_CONFIG_ANALOGOUTPUT_ADDRESS_AUPOSRST),
				makeDynInt( UN_CONFIG_ANALOGOUTPUT_ADDRESS_MPOSR), 
				makeDynInt( UN_CONFIG_ANALOGOUTPUT_UNIT, 
										UN_CONFIG_ANALOGOUTPUT_FORMAT, 
										UN_CONFIG_ANALOGOUTPUT_DEADBANDE, 
										UN_CONFIG_ANALOGOUTPUT_DEADBANDE_TYPE),		
				makeDynInt( UN_CONFIG_ANALOGOUTPUT_RANGEMIN, 
										UN_CONFIG_ANALOGOUTPUT_RANGEMAX),
				makeDynInt(	UN_CONFIG_ANALOGOUTPUT_ARCHIVE_ACTIVE, 
										UN_CONFIG_ANALOGOUTPUT_ARCHIVE_TIME_FILTER),
				exceptionInfo);
				
	// 4. Particular : limits
		
		unConfigGenericFunctions_check5RangesAlert(configLine,	UN_CONFIG_ANALOGOUTPUT_NORMAL_POSITION,
												   											makeDynInt(	UN_CONFIG_ANALOGOUTPUT_LLLIMIT, 
												   																	UN_CONFIG_ANALOGOUTPUT_LLIMIT, 
												 			  														UN_CONFIG_ANALOGOUTPUT_HLIMIT, 
												 			  														UN_CONFIG_ANALOGOUTPUT_HHLIMIT),
												   											exceptionInfo);
		
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_AnalogOutput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_AnalogOutput_setConfig
/**
Purpose: set configuration of given Analog Output datapoint for S7_PLC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP
	. distributed system: yes.
*/

S7_PLC_AnalogOutput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
	
	// DebugN("FUNCTION: S7_PLC_AnalogOutput_setConfig(dsConfigs= "+dsConfigs+" )");
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOGOUTPUT_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGOUTPUT_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGOUTPUT_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOGOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_STSREG01),
					 makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_EVSTSREG01),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_MANREG01),
					 makeDynString(),
					 makeDynString(),
					 -1,
					 exceptionInfo);
// 2.3. Float values
	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,
					PVSS_S7_FLOAT,
					makeDynString("PosSt", "HFStOrAuPosRSt"),
					makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_POSST, UN_CONFIG_ANALOGOUTPUT_ADDRESS_AUPOSRST),
					makeDynString("MPosR"),
					makeDynInt(UN_CONFIG_ANALOGOUTPUT_ADDRESS_MPOSR),
					makeDynInt(UN_CONFIG_ANALOGOUTPUT_UNIT, UN_CONFIG_ANALOGOUTPUT_FORMAT, UN_CONFIG_ANALOGOUTPUT_DEADBANDE, UN_CONFIG_ANALOGOUTPUT_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_ANALOGOUTPUT_RANGEMIN, UN_CONFIG_ANALOGOUTPUT_RANGEMAX),
					makeDynString("PosSt"),
					makeDynInt(UN_CONFIG_ANALOGOUTPUT_ARCHIVE_ACTIVE, UN_CONFIG_ANALOGOUTPUT_ARCHIVE_TIME_FILTER),
					exceptionInfo);
					
	unConfigGenericFunctions_set5RangesAlert(dsConfigs, sDpName + ".ProcessInput.PosSt", "Position Status", UN_CONFIG_ANALOGOUTPUT_NORMAL_POSITION,
													 makeDynInt(UN_CONFIG_ANALOGOUTPUT_LLLIMIT, UN_CONFIG_ANALOGOUTPUT_LLIMIT, 
													 			UN_CONFIG_ANALOGOUTPUT_HLIMIT, UN_CONFIG_ANALOGOUTPUT_HHLIMIT),
													 exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
}



//@}