/**@name LIBRARY: unConfigDigitalInput.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002

Modification History: 
  02/05/2011: Herve
  - IS-510: CPC 5 and CPC 6 compatibility of the export/import of the CPC device configuration with all the front-end types
  
	16/11/2006: Herve
		- case no version & archive = N --> mask =0;
		
	18/04/2006: Herve
		- implementation of mask event
		- if archive = N, do not delete the archive on evStsReg01

	13/05/2005: Enrique
		- S7 adaptation, new functions:
			. S7_PLC_DigitalInput_checkConfig
			. S7_PLC_DigitalInput_setConfig
		
	12/05/2004: Herve
		- delete archive on evStsReg01 if no archive on PosSt
		
	22/04/2005: Herve
		- remove UN_CONFIG_DEFAULT_ARCHIVE_SMOOTHING and replace in libs by DPATTR_ARCH_PROC_SIMPLESM
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_DigitalInput_checkConfig, _UnPlc_DigitalInput_setConfig

version 1.0

External Function : 
	. unConfigDigitalInput_setConfig :  set configuration of given Digital Input datapoint
	. unConfigDigitalInput_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos Digital Input configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const unsigned UN_CONFIG_DIGITALINPUT_LENGTH = 5;

const unsigned UN_CONFIG_DIGITALINPUT_ARCHIVE_ACTIVE = 1;
const unsigned UN_CONFIG_DIGITALINPUT_NORMAL_POSITION = 2;
const unsigned UN_CONFIG_DIGITALINPUT_ADDRESS_STSREG01 = 3;
const unsigned UN_CONFIG_DIGITALINPUT_ADDRESS_EVSTSREG01 = 4;
const unsigned UN_CONFIG_DIGITALINPUT_ADDRESS_MANREG01 = 5;

const unsigned UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH = 1;
const unsigned UN_CONFIG_DIGITALINPUT_MASKEVENT = 1;

// constant declaration for DigitalInput device used with CMW front-end
const unsigned UN_CONFIG_CMW_DIGITALINPUT_LENGTH = 7;
const unsigned UN_CONFIG_CMW_DIGITALINPUT_ARCHIVE_ACTIVE = 1;
const unsigned UN_CONFIG_CMW_DIGITALINPUT_NORMAL_POSITION = 2;
const unsigned UN_CONFIG_CMW_DIGITALINPUT_TYPE = 3;
const unsigned UN_CONFIG_CMW_DIGITALINPUT_GROUP = 4;
const unsigned UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMWDEVICE = 5;
const unsigned UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERNAME = 6;
const unsigned UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERVALUE = 7;

// constant declaration for DigitalInput device used with DIP front-end
const unsigned UN_CONFIG_DIP_DIGITALINPUT_LENGTH = 5;
const unsigned UN_CONFIG_DIP_DIGITALINPUT_ARCHIVE_ACTIVE = 1;
const unsigned UN_CONFIG_DIP_DIGITALINPUT_NORMAL_POSITION = 2;
const unsigned UN_CONFIG_DIP_DIGITALINPUT_ADDRESS_DIPITEM_POSST = 3;
const unsigned UN_CONFIG_DIP_DIGITALINPUT_ADDRESS_DIPTAG_POSST = 4;
const unsigned UN_CONFIG_DIP_DIGITALINPUT_ADDRESS_DIPBUFFER_POSST = 5;

// constant declaration for DigitalInput device used with OPC front-end
const unsigned UN_CONFIG_OPC_DIGITALINPUT_LENGTH = 6;
const unsigned UN_CONFIG_OPC_DIGITALINPUT_ARCHIVE_ACTIVE = 1;
const unsigned UN_CONFIG_OPC_DIGITALINPUT_NORMAL_POSITION = 2;
const unsigned UN_CONFIG_OPC_DIGITALINPUT_ADDRESS_OPCITEM_STSREG01 = 3;
const unsigned UN_CONFIG_OPC_DIGITALINPUT_ADDRESS_OPCITEM_EVSTSREG01 = 4;
const unsigned UN_CONFIG_OPC_DIGITALINPUT_ADDRESS_OPCITEM_POSST = 5;
const unsigned UN_CONFIG_OPC_DIGITALINPUT_ADDRESS_OPCITEM_MANREG01 = 6;

// constant declaration for DigitalInput device used with SOFT_FE front-end
const unsigned UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH = 2;
const unsigned UN_CONFIG_SOFT_FE_DIGITALINPUT_ARCHIVE_ACTIVE = 1;
const unsigned UN_CONFIG_SOFT_FE_DIGITALINPUT_NORMAL_POSITION = 2;

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigDigitalInput_setConfig
/**
Purpose:  set configuration of given Digital Input datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigDigitalInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bool archiveActive;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "DigitalInput", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "DigitalInput", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "DigitalInput", exceptionInfo);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigDigitalInput_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigDigitalInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "DigitalInput", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "DigitalInput", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigDigitalInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_DigitalInput_setConfig
/**
Purpose:  set configuration of given Digital Input datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_DigitalInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bool archiveActive;
	dyn_string exInfo;
	int archiveTimeFilter, iArchiveType, iSmoothProcedure;
	string sMask = "0";
	int iMask;
	bool bVersion;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALINPUT_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALINPUT_MASKEVENT];
		sscanf(sMask, "%d", iMask);
		bVersion = true;
		
		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_STSREG01),
					 makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_EVSTSREG01),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_MANREG01),
					 makeDynString("PosSt"),
					 makeDynString(UN_STSREG01_POSST),
					 UN_CONFIG_DIGITALINPUT_NORMAL_POSITION,
					 exceptionInfo);
// 2.3. Particular dpe : PosSt : Archive + faceplate range
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALINPUT_ARCHIVE_ACTIVE], 
																					archiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
	if (archiveActive)			// Archive
	{
		fwArchive_set(sDpName + ".ProcessInput.PosSt",
							  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
							  DPATTR_ARCH_PROC_SIMPLESM,
							  DPATTR_COMPARE_OLD_NEW,
							  0,
							  0,
							  exInfo);
		dynAppend(exceptionInfo, exInfo);
	}
	else						// No Archive
	{
		fwArchive_delete(sDpName + ".ProcessInput.PosSt", exceptionInfo);
		if(!bVersion) {
			iMask = 0;
			fwArchive_delete(sDpName + ".ProcessInput.evStsReg01", exceptionInfo);
		}
	}
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString("PosSt"), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MAX), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MIN), exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosSt"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_DigitalInput_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_DigitalInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALINPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALINPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_STSREG01),
				makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_EVSTSREG01),
				makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_MANREG01),
				UN_CONFIG_DIGITALINPUT_NORMAL_POSITION,
				exceptionInfo);
	// 3. Particular dpe : PosSt
		unConfigGenericFunctions_checkArchiveParameters("", configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALINPUT_ARCHIVE_ACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_DigitalInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_DigitalInput_setConfig
/**
Purpose:  set configuration of given Digital Input datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: Win XP
	. distributed system: yes.
*/

S7_PLC_DigitalInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bool archiveActive;
	dyn_string exInfo;
	int archiveTimeFilter, iArchiveType, iSmoothProcedure;
	string sMask = "0";
	int iMask;
	bool bVersion;
	
	// DebugN("FUNCTION: S7_PLC_DigitalInput_setConfig(dsConfigs="+dsConfigs);
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALINPUT_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALINPUT_MASKEVENT];
		sscanf(sMask, "%d", iMask);
		bVersion = true;

		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_STSREG01),
					 makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_EVSTSREG01),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_MANREG01),
					 makeDynString("PosSt"),
					 makeDynString(UN_STSREG01_POSST),
					 UN_CONFIG_DIGITALINPUT_NORMAL_POSITION,
					 exceptionInfo);
// 2.3. Particular dpe : PosSt : Archive + faceplate range
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALINPUT_ARCHIVE_ACTIVE], 
																					archiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
	if (archiveActive)			// Archive
	{
		fwArchive_set(sDpName + ".ProcessInput.PosSt",
							  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
							  DPATTR_ARCH_PROC_SIMPLESM,
							  DPATTR_COMPARE_OLD_NEW,
							  0,
							  0,
							  exInfo);
		dynAppend(exceptionInfo, exInfo);
	}
	else						// No Archive
	{
		fwArchive_delete(sDpName + ".ProcessInput.PosSt", exceptionInfo);
		if(!bVersion) {
			iMask = 0;
			fwArchive_delete(sDpName + ".ProcessInput.evStsReg01", exceptionInfo);
		}
	}
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString("PosSt"), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MAX), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MIN), exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosSt"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_DigitalInput_checkConfig
/**
Purpose: check data before configuration for S7_PLC front-end type

Parameters:
		configLine: 		dyn_string, input, 	configs (see dyn_string indexes in constants defined)
		exceptionInfo: 	dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: Win XP
	. distributed system: yes.
*/

S7_PLC_DigitalInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	// DebugN("FUNCTION: S7_PLC_DIGITALINPUT_checkConfig(configLine="+configLine);
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALINPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALINPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALINPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_STSREG01),
				makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_EVSTSREG01),
				makeDynInt(UN_CONFIG_DIGITALINPUT_ADDRESS_MANREG01),
				UN_CONFIG_DIGITALINPUT_NORMAL_POSITION,
				exceptionInfo);
	// 3. Particular dpe : PosSt
		unConfigGenericFunctions_checkArchiveParameters("", configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALINPUT_ARCHIVE_ACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_DigitalInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CMW_DigitalInput_setConfig
/**
Purpose:  set configuration of given Digital Input datapoint for CMW front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CMW_DigitalInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bool archiveActive;
	dyn_string exInfo;
	int iDriverNumber;
	bit32 b32;
	string sCMWSubscription;
	int archiveTimeFilter, iArchiveType, iSmoothProcedure;
	
	setBit(b32, UN_STSREG01_AUMOST, true);
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	iDriverNumber = dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER];

// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnCMW_config);
// set stsreg01 to automode
	dpSet(sDpName + ".ProcessInput.StsReg01", b32);
	dpSet(sDpName + ".eventMask", 1);

// 2.2. Registers
	unConfigGenericFunctions_setCommonAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynString("PosSt"),
					 makeDynString(UN_STSREG01_POSST),
					 UN_CONFIG_CMW_DIGITALINPUT_NORMAL_POSITION,
					 exceptionInfo);
// 2.3. Particular dpe : PosSt : Archive + faceplate range
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ARCHIVE_ACTIVE], 
																						archiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
	if (archiveActive)			// Archive
	{
		fwArchive_set(sDpName + ".ProcessInput.PosSt",
							  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
							  DPATTR_ARCH_PROC_SIMPLESM,
							  DPATTR_COMPARE_OLD_NEW,
							  0,
							  0,
							  exInfo);
		dynAppend(exceptionInfo, exInfo);
	}
	else						// No Archive
	{
		fwArchive_delete(sDpName + ".ProcessInput.PosSt", exceptionInfo);
		fwArchive_delete(sDpName + ".ProcessInput.evStsReg01", exceptionInfo);
	}
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString("PosSt"), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MAX), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MIN), exceptionInfo);

// input
	unConfigCMW_setAddress(sDpName + ".ProcessInput.PosSt", 
													dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_TYPE], 
													CMW_BOOL_TYPE,
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_GROUP],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERNAME],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERVALUE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMWDEVICE],
													"PosSt",
													"PosSt",
													true,
													exceptionInfo);
	unConfigCMW_setAddress(sDpName + ".ProcessInput.StsReg01", 
													dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_TYPE], 
													CMW_LONG_TYPE,
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_GROUP],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERNAME],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERVALUE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMWDEVICE],
													"StsReg01",
													"StsReg01",
													true,
													exceptionInfo);
	unConfigCMW_setAddress(sDpName + ".ProcessInput.evStsReg01", 
													dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_TYPE], 
													CMW_LONG_TYPE,
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_GROUP],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERNAME],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERVALUE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMWDEVICE],
													"evStsReg01",
													"evStsReg01",
													true,
													exceptionInfo);
// output
	unConfigCMW_setAddress(sDpName + ".ProcessOutput.ManReg01", 
													dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
													DPATTR_ADDR_MODE_OUTPUT_SINGLE, 
													CMW_LONG_TYPE+CMW_NO_MONITORON,
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_TYPE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERNAME],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERVALUE],
													dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMWDEVICE],
													"ManReg01",
													"ManReg01",
													false,
													exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// CMW_DigitalInput_checkConfig
/**
Purpose: check data before configuration for CMW front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CMW_DigitalInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_CMW_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
	// 2. check normal position
		unConfigGenericFunctions_checkNormalPosition(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_NORMAL_POSITION], exceptionInfo);

	// 3. Particular dpe : PosSt
		unConfigGenericFunctions_checkArchiveParameters("", configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CMW_DIGITALINPUT_ARCHIVE_ACTIVE], exceptionInfo);

	// check group
//DebugN(configLine, configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_DIGITALINPUT_TYPE], configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_DIGITALINPUT_GROUP]);
		unConfigCMW_checkType(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_DIGITALINPUT_TYPE], configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_DIGITALINPUT_GROUP], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CMW_DigitalInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// DIP_DigitalInput_setConfig
/**
Purpose:  set configuration of given Digital Input datapoint for DIP front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

DIP_DigitalInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bool archiveActive;
	dyn_string exInfo;
	int iDriverNumber;
	bit32 b32;
	string sDipSubscription;
	int archiveTimeFilter, iArchiveType, iSmoothProcedure;
	
	setBit(b32, UN_STSREG01_AUMOST, true);
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	iDriverNumber = dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER];

// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnDIP_config);
// set stsreg01 to automode
	dpSet(sDpName + ".ProcessInput.StsReg01", b32);
// set periph address posst

	sDipSubscription = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIP_DIGITALINPUT_ADDRESS_DIPITEM_POSST] +
											fwDIP_SEPERATOR +
											dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIP_DIGITALINPUT_ADDRESS_DIPTAG_POSST];
	fwPeriphAddress_set(sDpName + ".ProcessInput.PosSt",
						makeDynString("DIP",
									  iDriverNumber, 
									  sDipSubscription, 
									  0,
									  0,
									  0,
									  "",
									  "",
									  "",
									  "", 
									  g_dsImport_UnDIP_config[UNDIP_CLIENTLIST],
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIP_DIGITALINPUT_ADDRESS_DIPBUFFER_POSST]),
						exceptionInfo);

// 2.2. Registers
	unConfigGenericFunctions_setCommonAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynString("PosSt"),
					 makeDynString(UN_STSREG01_POSST),
					 UN_CONFIG_DIP_DIGITALINPUT_NORMAL_POSITION,
					 exceptionInfo);
// 2.3. Particular dpe : PosSt : Archive + faceplate range
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIP_DIGITALINPUT_ARCHIVE_ACTIVE], 
																						archiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
	if (archiveActive)			// Archive
	{
		fwArchive_set(sDpName + ".ProcessInput.PosSt",
							  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
							  DPATTR_ARCH_PROC_SIMPLESM,
							  DPATTR_COMPARE_OLD_NEW,
							  0,
							  0,
							  exInfo);
		dynAppend(exceptionInfo, exInfo);
	}
	else						// No Archive
	{
		fwArchive_delete(sDpName + ".ProcessInput.PosSt", exceptionInfo);
		fwArchive_delete(sDpName + ".ProcessInput.evStsReg01", exceptionInfo);
	}
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString("PosSt"), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MAX), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MIN), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// DIP_DigitalInput_checkConfig
/**
Purpose: check data before configuration for DIP front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

DIP_DigitalInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_DIP_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
	// 2. check normal position
		unConfigGenericFunctions_checkNormalPosition(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIP_DIGITALINPUT_NORMAL_POSITION], exceptionInfo);

	// 3. Particular dpe : PosSt
		unConfigGenericFunctions_checkArchiveParameters("", configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIP_DIGITALINPUT_ARCHIVE_ACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","DIP_DigitalInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// OPC_DigitalInput_setConfig
/**
Purpose:  set configuration of given Digital Input datapoint for OPC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

OPC_DigitalInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bool archiveActive;
	dyn_string exInfo;
	int iDriverNumber;
	int archiveTimeFilter, iArchiveType, iSmoothProcedure;
		
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	iDriverNumber = dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER];

// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnOPC_config);
// set periph address stsreg01, evstsreg01, posst
	fwPeriphAddress_set(sDpName + ".ProcessInput.StsReg01",
						makeDynString("OPC",
									  iDriverNumber, 
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_DIGITALINPUT_ADDRESS_OPCITEM_STSREG01], 
									  DPATTR_ADDR_MODE_INPUT_SPONT,
									  OPC_UINT16,
									  UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
									  "",
									  "",
									  "",
									  "", 
									  true,
									  0,
									  g_dsImport_UnOPC_config[UNOPC_SERVERNAME], 
									  g_dsImport_UnOPC_config[UNOPC_GROUPINNAME],
									  g_dsImport_UnOPC_config[UNOPC_GROUPOUTNAME]
									  ),
						exceptionInfo);
	fwPeriphAddress_set(sDpName + ".ProcessInput.evStsReg01",
						makeDynString("OPC",
									  iDriverNumber, 
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_DIGITALINPUT_ADDRESS_OPCITEM_EVSTSREG01], 
									  DPATTR_ADDR_MODE_INPUT_SPONT,
									  OPC_INT32,
									  UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
									  "",
									  "",
									  "",
									  "", 
									  true,
									  0,
									  g_dsImport_UnOPC_config[UNOPC_SERVERNAME], 
									  g_dsImport_UnOPC_config[UNOPC_GROUPINNAME],
									  g_dsImport_UnOPC_config[UNOPC_GROUPOUTNAME]
									  ),
						exceptionInfo);
	fwPeriphAddress_set(sDpName + ".ProcessInput.PosSt",
						makeDynString("OPC",
									  iDriverNumber, 
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_DIGITALINPUT_ADDRESS_OPCITEM_POSST], 
									  DPATTR_ADDR_MODE_INPUT_SPONT,
									  OPC_BOOL,
									  UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
									  "",
									  "",
									  "",
									  "", 
									  true,
									  0,
									  g_dsImport_UnOPC_config[UNOPC_SERVERNAME], 
									  g_dsImport_UnOPC_config[UNOPC_GROUPINNAME],
									  g_dsImport_UnOPC_config[UNOPC_GROUPOUTNAME]
									  ),
						exceptionInfo);

// set periph address manreg01
	fwPeriphAddress_set(sDpName + ".ProcessOutput.ManReg01",
						makeDynString("OPC",
									  iDriverNumber, 
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_DIGITALINPUT_ADDRESS_OPCITEM_MANREG01], 
									  DPATTR_ADDR_MODE_OUTPUT_SINGLE,
									  OPC_UINT16,
									  UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
									  "",
									  "",
									  "",
									  "", 
									  false,
									  0,
									  g_dsImport_UnOPC_config[UNOPC_SERVERNAME], 
									  g_dsImport_UnOPC_config[UNOPC_GROUPINNAME],
									  g_dsImport_UnOPC_config[UNOPC_GROUPOUTNAME]
										),
						exceptionInfo);

// 2.2. Registers
	unConfigGenericFunctions_setCommonAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynString("PosSt"),
					 makeDynString(UN_STSREG01_POSST),
					 UN_CONFIG_OPC_DIGITALINPUT_NORMAL_POSITION,
					 exceptionInfo);
// 2.3. Particular dpe : PosSt : Archive + faceplate range
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_DIGITALINPUT_ARCHIVE_ACTIVE], 
																						archiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
	if (archiveActive)			// Archive
	{
		fwArchive_set(sDpName + ".ProcessInput.PosSt",
							  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
							  DPATTR_ARCH_PROC_SIMPLESM,
							  DPATTR_COMPARE_OLD_NEW,
							  0,
							  0,
							  exInfo);
		dynAppend(exceptionInfo, exInfo);
	}
	else						// No Archive
	{
		fwArchive_delete(sDpName + ".ProcessInput.PosSt", exceptionInfo);
		fwArchive_delete(sDpName + ".ProcessInput.evStsReg01", exceptionInfo);
	}
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString("PosSt"), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MAX), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MIN), exceptionInfo);
// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], 1, makeDynString(), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// OPC_DigitalInput_checkConfig
/**
Purpose: check data before configuration for OPC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

OPC_DigitalInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_OPC_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
	// 2. check normal position
		unConfigGenericFunctions_checkNormalPosition(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_OPC_DIGITALINPUT_NORMAL_POSITION], exceptionInfo);

	// 3. Particular dpe : PosSt
		unConfigGenericFunctions_checkArchiveParameters("", configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_OPC_DIGITALINPUT_ARCHIVE_ACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","OPC_DigitalInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// SOFT_FE_DigitalInput_setConfig
/**
Purpose:  set configuration of given Digital Input datapoint for SOFT_FE front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
SOFT_FE_DigitalInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bool archiveActive;
	dyn_string exInfo;
	int iDriverNumber;
	bit32 b32;
	string sSOFT_FESubscription;
	int archiveTimeFilter, iArchiveType, iSmoothProcedure;
	
	setBit(b32, UN_STSREG01_AUMOST, true);
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	iDriverNumber = dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER];

// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnSOFT_FE_config);

// 2.2. Registers
	unConfigGenericFunctions_setCommonAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynString("PosSt"),
					 makeDynString(UN_STSREG01_POSST),
					 UN_CONFIG_SOFT_FE_DIGITALINPUT_NORMAL_POSITION,
					 exceptionInfo);
// 2.3. Particular dpe : PosSt : Archive + faceplate range
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALINPUT_ARCHIVE_ACTIVE], 
																						archiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
	if (archiveActive)			// Archive
	{
		fwArchive_set(sDpName + ".ProcessInput.PosSt",
							  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
							  DPATTR_ARCH_PROC_SIMPLESM,
							  DPATTR_COMPARE_OLD_NEW,
							  0,
							  0,
							  exInfo);
		dynAppend(exceptionInfo, exInfo);
	}
	else						// No Archive
	{
		fwArchive_delete(sDpName + ".ProcessInput.PosSt", exceptionInfo);
		fwArchive_delete(sDpName + ".ProcessInput.evStsReg01", exceptionInfo);
	}
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString("PosSt"), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MAX), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MIN), exceptionInfo);

// set stsreg & evStsReg
// set mask
// set stsreg01 to automode
	dpSet(sDpName + ".ProcessInput.StsReg01", b32, sDpName + ".ProcessInput.evStsReg01", b32, sDpName + ".eventMask", 1);
}

//------------------------------------------------------------------------------------------------------------------------

// SOFT_FE_DigitalInput_checkConfig
/**
Purpose: check data before configuration for SOFT_FE front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
SOFT_FE_DigitalInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_SOFT_FE_DIGITALINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
	// 2. check normal position
		unConfigGenericFunctions_checkNormalPosition(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALINPUT_NORMAL_POSITION], exceptionInfo);

	// 3. Particular dpe : PosSt
		unConfigGenericFunctions_checkArchiveParameters("", configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_DIGITALINPUT_ARCHIVE_ACTIVE], exceptionInfo);

	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","SOFT_FE_DigitalInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//@}
