/**@name LIBRARY: unConfigDigitalOutput.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002

Modification History: 
	16/11/2006: Herve
		- case no version & archive = N --> mask =0;

	18/04/2006: Herve
		- implementation of mask event

	13/05/2005: Enrique
		- S7 adaptation, new functions:
					. S7_PLC_DigitalOutput_checkConfig
					. S7_PLC_DigitalOutput_setConfig
		
	12/05/2004: Herve
		- delete archive on evStsReg01 if no archive on PosSt
		
	22/04/2005: Herve
		- remove UN_CONFIG_DEFAULT_ARCHIVE_SMOOTHING and replace in libs by DPATTR_ARCH_PROC_SIMPLESM
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_DigitalOutput_checkConfig, _UnPlc_DigitalOutput_setConfig

version 1.0

External Function : 
	. unConfigDigitalOutput_setConfig :  set configuration of given Digital Output datapoint
	. unConfigDigitalOutput_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos Digital Output configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const unsigned UN_CONFIG_DIGITALOUTPUT_LENGTH = 5;

const unsigned UN_CONFIG_DIGITALOUTPUT_ARCHIVE_ACTIVE = 1;
const unsigned UN_CONFIG_DIGITALOUTPUT_NORMAL_POSITION = 2;
const unsigned UN_CONFIG_DIGITALOUTPUT_ADDRESS_STSREG01 = 3;
const unsigned UN_CONFIG_DIGITALOUTPUT_ADDRESS_EVSTSREG01 = 4;
const unsigned UN_CONFIG_DIGITALOUTPUT_ADDRESS_MANREG01 = 5;

const unsigned UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH = 1;
const unsigned UN_CONFIG_DIGITALOUTPUT_MASKEVENT = 1;

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigDigitalOutput_setConfig
/**
Purpose:  set configuration of given Digital Output datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigDigitalOutput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bool archiveActive;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "DigitalOutput", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "DigitalOutput", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "DigitalOutput", exceptionInfo);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigDigitalOutput_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigDigitalOutput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "DigitalOutput", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "DigitalOutput", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigDigitalOutput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_DigitalOutput_setConfig
/**
Purpose:  set configuration of given Digital Output datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_DigitalOutput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bool archiveActive;
	dyn_string exInfo;
	int archiveTimeFilter, iArchiveType, iSmoothProcedure;
	string sMask = "0";
	int iMask;
	bool bVersion;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALOUTPUT_MASKEVENT];
		sscanf(sMask, "%d", iMask);
		bVersion = true;

		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_STSREG01),
					 makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_EVSTSREG01),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_MANREG01),
					 makeDynString("PosSt"),
					 makeDynString(UN_STSREG01_POSST),
					 UN_CONFIG_DIGITALOUTPUT_NORMAL_POSITION,
					 exceptionInfo);
// 2.3. Particular dpe : PosSt : Archive + faceplate range
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_ARCHIVE_ACTIVE], 
																					archiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
	if (archiveActive)			// Archive
	{
		fwArchive_set(sDpName + ".ProcessInput.PosSt",
					  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
					  DPATTR_ARCH_PROC_SIMPLESM,
					  DPATTR_COMPARE_OLD_NEW,
					  0,
					  0,
						exInfo);
		dynAppend(exceptionInfo, exInfo);
	}
	else						// No Archive
	{
		fwArchive_delete(sDpName + ".ProcessInput.PosSt", exceptionInfo);
		if(!bVersion) {
			iMask = 0;
			fwArchive_delete(sDpName + ".ProcessInput.evStsReg01", exceptionInfo);
		}
	}
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString("PosSt"), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MAX), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MIN), exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosSt"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_DigitalOutput_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_DigitalOutput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALOUTPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_STSREG01),
				makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_EVSTSREG01),
				makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_MANREG01),
				UN_CONFIG_DIGITALOUTPUT_NORMAL_POSITION,
				exceptionInfo);
	// 3. Particular dpe : PosSt
		unConfigGenericFunctions_checkArchiveParameters("",	configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_ARCHIVE_ACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_DigitalOutput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_DigitalOutput_setConfig
/**
Purpose:  set configuration of given Digital Output datapoint for S7_PLC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win. XP
	. distributed system: yes.
*/

S7_PLC_DigitalOutput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bool archiveActive;
	dyn_string exInfo;
	int archiveTimeFilter, iArchiveType, iSmoothProcedure;
	string sMask = "0";
	int iMask;
	bool bVersion;

	// DebugN("FUNCTION: S7_PLC_DigitalOutput_setConfig(dsConfigs= "+dsConfigs);
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALOUTPUT_MASKEVENT];
		sscanf(sMask, "%d", iMask);
		bVersion = true;

		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_STSREG01),
					 makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_EVSTSREG01),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_MANREG01),
					 makeDynString("PosSt"),
					 makeDynString(UN_STSREG01_POSST),
					 UN_CONFIG_DIGITALOUTPUT_NORMAL_POSITION,
					 exceptionInfo);
// 2.3. Particular dpe : PosSt : Archive + faceplate range
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_ARCHIVE_ACTIVE], 
																					archiveActive, archiveTimeFilter, iArchiveType, iSmoothProcedure);
	if (archiveActive)			// Archive
	{
		fwArchive_set(sDpName + ".ProcessInput.PosSt",
					  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
					  DPATTR_ARCH_PROC_SIMPLESM,
					  DPATTR_COMPARE_OLD_NEW,
					  0,
					  0,
						exInfo);
		dynAppend(exceptionInfo, exInfo);
	}
	else						// No Archive
	{
		fwArchive_delete(sDpName + ".ProcessInput.PosSt", exceptionInfo);
		if(!bVersion) {
			iMask = 0;
			fwArchive_delete(sDpName + ".ProcessInput.evStsReg01", exceptionInfo);
		}
	}
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString("PosSt"), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MAX), makeDynString(UN_FACEPLATE_TREND_DIGITAL_DEFAULT_MIN), exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosSt"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_DigitalOutput_checkConfig
/**
Purpose: check data before configuration for S7_PLC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0	 
	. operating system: Win. XP
	. distributed system: yes.
*/

S7_PLC_DigitalOutput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	// DebugN("FUNCTION: S7_PLC_DIGITALOUTPUT_checkConfig(configLine="+configLine);
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALOUTPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_DIGITALOUTPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_DIGITALOUTPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_STSREG01),
				makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_EVSTSREG01),
				makeDynInt(UN_CONFIG_DIGITALOUTPUT_ADDRESS_MANREG01),
				UN_CONFIG_DIGITALOUTPUT_NORMAL_POSITION,
				exceptionInfo);
	// 3. Particular dpe : PosSt
		unConfigGenericFunctions_checkArchiveParameters("",	configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_DIGITALOUTPUT_ARCHIVE_ACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_DigitalOutput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//@}