/**@name LIBRARY: unConfigOnOff.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002
	18/04/2006: Herve
		- implementation of mask event

	14/06/2005: Enrique
		- S7 adaptation, new functions:
			. S7_PLC_OnOff_checkConfig
			. S7_PLC_OnOff_setConfig
		
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_OnOff_checkConfig, _UnPlc_OnOff_setConfig

Modification History: 

version 1.0

External Function : 
	. unConfigOnOff_setConfig :  set configuration of given OnOff datapoint
	. unConfigOnOff_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos OnOff configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const unsigned UN_CONFIG_ONOFF_LENGTH = 6;

const unsigned UN_CONFIG_ONOFF_NORMAL_POSITION = 1;
const unsigned UN_CONFIG_ONOFF_ADDRESS_STSREG01 = 2;
const unsigned UN_CONFIG_ONOFF_ADDRESS_STSREG02 = 3;
const unsigned UN_CONFIG_ONOFF_ADDRESS_EVSTSREG01 = 4;
const unsigned UN_CONFIG_ONOFF_ADDRESS_EVSTSREG02 = 5;
const unsigned UN_CONFIG_ONOFF_ADDRESS_MANREG01 = 6;

const unsigned UN_CONFIG_ONOFF_ADDITIONAL_LENGTH = 1;
const unsigned UN_CONFIG_ONOFF_MASKEVENT = 1;

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigOnOff_setConfig
/**
Purpose:  set configuration of given OnOff datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigOnOff_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "OnOff", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "OnOff", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "OnOff", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigOnOff_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigOnOff_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "OnOff", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "OnOff", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigOnOff_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_OnOff_setConfig
/**
Purpose:  set configuration of given OnOff datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_OnOff_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ONOFF_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ONOFF_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ONOFF_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ONOFF_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ONOFF_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegisters(dsConfigs,
					 makeDynString("StsReg01","StsReg02"),
					 makeDynInt(UN_CONFIG_ONOFF_ADDRESS_STSREG01, UN_CONFIG_ONOFF_ADDRESS_STSREG02),
					 makeDynInt(UN_CONFIG_ONOFF_ADDRESS_EVSTSREG01, UN_CONFIG_ONOFF_ADDRESS_EVSTSREG02),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_ONOFF_ADDRESS_MANREG01),
					 makeDynString("StartISt;StopISt"),
					 makeDynString(UN_STSREG01_STARTIST + ";" + UN_STSREG01_STOPIST),
					 UN_CONFIG_ONOFF_NORMAL_POSITION,
					 exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																								".ProcessInput.StartISt", ".ProcessInput.StopISt"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_OnOff_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_OnOff_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ONOFF_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ONOFF_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ONOFF_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ONOFF_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ONOFF_ADDRESS_STSREG01, UN_CONFIG_ONOFF_ADDRESS_STSREG02),
				makeDynInt(UN_CONFIG_ONOFF_ADDRESS_EVSTSREG01, UN_CONFIG_ONOFF_ADDRESS_EVSTSREG02),
				makeDynInt(UN_CONFIG_ONOFF_ADDRESS_MANREG01),
				UN_CONFIG_ONOFF_NORMAL_POSITION,
				exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_OnOff_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_OnOff_setConfig
/**
Purpose:  set configuration of given OnOff datapoint for S7_PLC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: win XP.
	. distributed system: yes.
*/

S7_PLC_OnOff_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
	
	// DebugN("FUNCTION: S7_PLC_OnOff_setConfig(dsConfigs= "+dsConfigs);
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ONOFF_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ONOFF_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ONOFF_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ONOFF_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ONOFF_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
					 makeDynString("StsReg01","StsReg02"),
					 makeDynInt(UN_CONFIG_ONOFF_ADDRESS_STSREG01, UN_CONFIG_ONOFF_ADDRESS_STSREG02),
					 makeDynInt(UN_CONFIG_ONOFF_ADDRESS_EVSTSREG01, UN_CONFIG_ONOFF_ADDRESS_EVSTSREG02),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_ONOFF_ADDRESS_MANREG01),
					 makeDynString("StartISt;StopISt"),
					 makeDynString(UN_STSREG01_STARTIST + ";" + UN_STSREG01_STOPIST),
					 UN_CONFIG_ONOFF_NORMAL_POSITION,
					 exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																								".ProcessInput.StartISt", ".ProcessInput.StopISt"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_OnOff_checkConfig
/**
Purpose: check data before configuration for S7_PLC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

S7_PLC_OnOff_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	// DebugN("FUNCTION: S7_PLC_ONOFF_checkConfig(configLine="+configLine);
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ONOFF_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ONOFF_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ONOFF_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ONOFF_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ONOFF_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ONOFF_ADDRESS_STSREG01, UN_CONFIG_ONOFF_ADDRESS_STSREG02),
				makeDynInt(UN_CONFIG_ONOFF_ADDRESS_EVSTSREG01, UN_CONFIG_ONOFF_ADDRESS_EVSTSREG02),
				makeDynInt(UN_CONFIG_ONOFF_ADDRESS_MANREG01),
				UN_CONFIG_ONOFF_NORMAL_POSITION,
				exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_OnOff_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//@}