/**@name LIBRARY: unConfigProcessControlObject.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002

Modification History: 
	18/04/2006: Herve
		- implementation of mask event

	15/06/2005: Enrique
		- S7 adaptation, new functions:
			
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_ProcessControlObject_checkConfig, _UnPlc_ProcessControlObject_setConfig

	20/10/2003: Herve 
		- config of "OpMoSt", "AuOpMo" and "MOpMoR" set to PVSS_MODBUS_FLOAT instead of PVSS_MODBUS_INT16

version 2.0

External Function : 
	. unConfigProcessControlObject_setConfig :  set configuration of given ProcessControlObject datapoint
	. unConfigProcessControlObject_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos ProcessControlObject configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH = 26;

const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_PCONAME = 1;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_NORMAL_POSITION = 2;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_STSREG01 = 3;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_STSREG02 = 4;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG01 = 5;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG02 = 6;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_AUOPMO = 7;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_OPMOST = 8;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_MANREG01 = 9;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_MOPMOR = 10;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_1 = 11;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_2 = 12;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_3 = 13;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_4 = 14;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_5 = 15;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_6 = 16;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_7 = 17;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_8 = 18;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_1 = 19;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_2 = 20;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_3 = 21;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_4 = 22;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_5 = 23;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_6 = 24;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_7 = 25;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_8 = 26;

const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH = 1;
const unsigned UN_CONFIG_PROCESSCONTROLOBJECT_MASKEVENT = 1;

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigProcessControlObject_setConfig
/**
Purpose:  set configuration of given ProcessControlObject datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigProcessControlObject_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	int iRes;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "ProcessControlObject", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "ProcessControlObject", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "ProcessControlObject", exceptionInfo);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigProcessControlObject_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigProcessControlObject_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	int i;
	string allowance;
	
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "ProcessControlObject", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "ProcessControlObject", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigProcessControlObject_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_ProcessControlObject_setConfig
/**
Purpose:  set configuration of given ProcessControlObject datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_ProcessControlObject_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	int iRes;
	string sMask = "0";
	int iMask;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

	unPCO_setName(sDpName, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_PCONAME], exceptionInfo);
// 2.2. Registers
	unConfigGenericFunctions_setAllRegisters(dsConfigs,
					 makeDynString("StsReg01","StsReg02"),
					 makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_STSREG01, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_STSREG02),
					 makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG01, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG02),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_MANREG01),
					 makeDynString("FuStopAl;TStopAl;StartAl"),
					 makeDynString(UN_STSREG01_FUSTOPAL + ";" + UN_STSREG01_TSTOPAL + ";" + UN_STSREG01_STARTAL),
					 UN_CONFIG_PROCESSCONTROLOBJECT_NORMAL_POSITION,
					 exceptionInfo);
// 2.3. Int values
	unConfigGenericFunctions_setFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("OpMoSt", "AuOpMo"),
					makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_OPMOST, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_AUOPMO),
					makeDynString("MOpMoR"),
					makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_MOPMOR),
					makeDynInt(),
					makeDynInt(),
					makeDynString(),
					makeDynInt(),
					exceptionInfo);
// 2.4. Particular parameters : Option modes parameters
	unPCO_setOptionModes(sDpName,
								 makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_1],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_2],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_3],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_4],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_5],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_6],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_7],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_8]),
								 makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_1],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_2],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_3],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_4],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_5],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_6],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_7],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_8]),
								 iRes);
	if (iRes < 0)
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_ProcessControlObject_setConfig: set option modes failed","");
	}

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																								".ProcessInput.FuStopAl", ".ProcessInput.TStopAl", ".ProcessInput.StartAl"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_ProcessControlObject_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_ProcessControlObject_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	int i;
	string allowance;
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_STSREG01, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_STSREG02),
				makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG01, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG02),
				makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_MANREG01),
				UN_CONFIG_PROCESSCONTROLOBJECT_NORMAL_POSITION,
				exceptionInfo);
	// 3. Float block
		unConfigGenericFunctions_checkFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_OPMOST, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_AUOPMO),
				makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_MOPMOR), 
				makeDynInt(),		
				makeDynInt(),
				makeDynInt(),
				exceptionInfo);
	// 4. Particular parameters : allowances
		for (i=UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_1;i<=UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_8;i++)
		{
			allowance = configLine[UN_CONFIG_COMMON_LENGTH + i];
			if ((strlen(allowance) != 0) && (strlen(allowance) != 8))
			{
				fwException_raise(exceptionInfo,"ERROR","_UnPlc_ProcessControlObject_checkConfig: " + getCatStr("unGeneration","BADALLOWANCE"),"");
			}
			else
			{
				strreplace(allowance, "1", "");
				strreplace(allowance, "0", "");
				if (allowance != "")
				{
					fwException_raise(exceptionInfo,"ERROR","_UnPlc_ProcessControlObject_checkConfig: " + getCatStr("unGeneration","BADALLOWANCE"),"");
				}
			}
		}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_ProcessControlObject_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_ProcessControlObject_setConfig
/**
Purpose:  set configuration of given ProcessControlObject datapoint for S7_PLC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: win XP.
	. distributed system: yes.
*/

S7_PLC_ProcessControlObject_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	int iRes;
	string sMask = "0";
	int iMask;
	
	// DebugN("FUNCTION: S7_PLC_ProcessControlObject_setConfig(dsConfigs= "+dsConfigs);
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

	unPCO_setName(sDpName, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_PCONAME], exceptionInfo);
// 2.2. Registers
	unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
					 makeDynString("StsReg01","StsReg02"),
					 makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_STSREG01, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_STSREG02),
					 makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG01, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG02),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_MANREG01),
					 makeDynString("FuStopAl;TStopAl;StartAl"),
					 makeDynString(UN_STSREG01_FUSTOPAL + ";" + UN_STSREG01_TSTOPAL + ";" + UN_STSREG01_STARTAL),
					 UN_CONFIG_PROCESSCONTROLOBJECT_NORMAL_POSITION,
					 exceptionInfo);
// 2.3. Int values
	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,
					PVSS_S7_FLOAT,
					makeDynString("OpMoSt", "AuOpMo"),
					makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_OPMOST, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_AUOPMO),
					makeDynString("MOpMoR"),
					makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_MOPMOR),
					makeDynInt(),
					makeDynInt(),
					makeDynString(),
					makeDynInt(),
					exceptionInfo);
// 2.4. Particular parameters : Option modes parameters
	unPCO_setOptionModes(sDpName,
								 makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_1],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_2],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_3],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_4],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_5],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_6],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_7],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_NAME_8]),
								 makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_1],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_2],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_3],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_4],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_5],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_6],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_7],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_8]),
								 iRes);
	if (iRes < 0)
	{
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_ProcessControlObject_setConfig: set option modes failed","");
	}

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																								".ProcessInput.FuStopAl", ".ProcessInput.TStopAl", ".ProcessInput.StartAl"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_ProcessControlObject_checkConfig
/**
Purpose: check data before configuration for S7_PLC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: win XP.
	. distributed system: yes.
*/

S7_PLC_ProcessControlObject_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	int i;
	string allowance;
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	// DebugN("FUNCTION: S7_PLC_ProcessControlObject_checkConfig(configLine= "+configLine);
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_PROCESSCONTROLOBJECT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_PROCESSCONTROLOBJECT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_STSREG01, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_STSREG02),
				makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG01, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG02),
				makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_MANREG01),
				UN_CONFIG_PROCESSCONTROLOBJECT_NORMAL_POSITION,
				exceptionInfo);
	// 3. Float block
		unConfigGenericFunctions_checkFloatIntBlockS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_OPMOST, UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_AUOPMO),
				makeDynInt(UN_CONFIG_PROCESSCONTROLOBJECT_ADDRESS_MOPMOR), 
				makeDynInt(),		
				makeDynInt(),
				makeDynInt(),
				exceptionInfo);
	// 4. Particular parameters : allowances
		for (i=UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_1;i<=UN_CONFIG_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_8;i++)
		{
			allowance = configLine[UN_CONFIG_COMMON_LENGTH + i];
			if ((strlen(allowance) != 0) && (strlen(allowance) != 8))
			{
				fwException_raise(exceptionInfo,"ERROR","S7_PLC_ProcessControlObject_checkConfig: " + getCatStr("unGeneration","BADALLOWANCE"),"");
			}
			else
			{
				strreplace(allowance, "1", "");
				strreplace(allowance, "0", "");
				if (allowance != "")
				{
					fwException_raise(exceptionInfo,"ERROR","S7_PLC_ProcessControlObject_checkConfig: " + getCatStr("unGeneration","BADALLOWANCE"),"");
				}
			}
		}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_ProcessControlObject_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//@}